@@ -1,41 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 Uwe Kindler
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or (at your option) any later version.
-** The licenses are as published by the Free Software Foundation
-** and appearing in the file LICENSE.LGPLv21 included in the packaging
-** of this file. Please review the following information to ensure
-** the GNU Lesser General Public License version 2.1 requirements
-** will be met: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2020 Uwe Kindler
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-2.1-or-later OR GPL-3.0-or-later
 
 #include "dockwidgettab.h"
 
 #include "ads_globals.h"
+#include "ads_globals_p.h"
+#include "advanceddockingsystemtr.h"
 #include "dockareawidget.h"
 #include "dockmanager.h"
 #include "dockoverlay.h"
@@ -45,6 +15,8 @@
 #include "floatingdragpreview.h"
 #include "iconprovider.h"
 
+#include <utils/theme/theme.h>
+
 #include <QApplication>
 #include <QBoxLayout>
 #include <QLabel>
@@ -54,12 +26,13 @@
 #include <QPushButton>
 #include <QSplitter>
 #include <QStyle>
+#include <QStyleOption>
 #include <QToolButton>
+#include <QPainter>
+#include <QStylePainter>
 
 #include <iostream>
 
-static Q_LOGGING_CATEGORY(adsLog, "qtc.qmldesigner.advanceddockingsystem", QtWarningMsg)
-
 namespace ADS
 {
     using TabLabelType = ElidingLabel;
@@ -81,7 +54,7 @@ namespace ADS
         eDragState m_dragState = DraggingInactive;
         AbstractFloatingWidget *m_floatingWidget = nullptr;
         QIcon m_icon;
-        QAbstractButton *m_closeButton = nullptr;
+        TabButton *m_closeButton = nullptr;
         QPoint m_tabDragStartPosition;
 
         /**
@@ -116,14 +89,15 @@ namespace ADS
          */
         bool testConfigFlag(DockManager::eConfigFlag flag) const
         {
-            return DockManager::configFlags().testFlag(flag);
+            return DockManager::testConfigFlag(flag);
         }
 
         /**
          * Creates the close button as QPushButton or as QToolButton
          */
-        QAbstractButton *createCloseButton() const
+        TabButton *createCloseButton() const
         {
+            /*
             if (testConfigFlag(DockManager::TabCloseButtonIsToolButton)) {
                 auto button = new QToolButton();
                 button->setAutoRaise(true);
@@ -131,6 +105,8 @@ namespace ADS
             } else {
                 return new QPushButton();
             }
+            */
+            return new TabButton();
         }
 
         template<typename T>
@@ -168,14 +144,21 @@ namespace ADS
         m_titleLabel->setText(m_dockWidget->windowTitle());
         m_titleLabel->setObjectName("dockWidgetTabLabel");
         m_titleLabel->setAlignment(Qt::AlignCenter);
-        QObject::connect(m_titleLabel, &ElidingLabel::elidedChanged, q, &DockWidgetTab::elidedChanged);
+        QObject::connect(m_titleLabel,
+                         &ElidingLabel::elidedChanged,
+                         q,
+                         &DockWidgetTab::elidedChanged);
 
         m_closeButton = createCloseButton();
         m_closeButton->setObjectName("tabCloseButton");
-        internal::setButtonIcon(m_closeButton, QStyle::SP_TitleBarCloseButton, TabCloseIcon);
+        internal::setButtonIcon(m_closeButton,
+                                QStyle::SP_TitleBarCloseButton,
+                                TabCloseIcon);
         m_closeButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+        m_closeButton->setIconSize(QSize(11, 11));
+        m_closeButton->setFixedSize(QSize(17, 17));
         q->onDockWidgetFeaturesChanged();
-        internal::setToolTip(m_closeButton, QObject::tr("Close Tab"));
+        internal::setToolTip(m_closeButton, Tr::tr("Close Tab"));
         QObject::connect(m_closeButton,
                          &QAbstractButton::clicked,
                          q,
@@ -189,11 +172,14 @@ namespace ADS
         boxLayout->setContentsMargins(2 * spacing, 0, 0, 0);
         boxLayout->setSpacing(0);
         q->setLayout(boxLayout);
-        boxLayout->addWidget(m_titleLabel, 1);
-        boxLayout->addSpacing(spacing);
-        boxLayout->addWidget(m_closeButton);
+        boxLayout->addWidget(m_titleLabel, 1, Qt::AlignVCenter);
         boxLayout->addSpacing(qRound(spacing * 4.0 / 3.0));
-        boxLayout->setAlignment(Qt::AlignCenter);
+        boxLayout->addWidget(m_closeButton, 0, Qt::AlignVCenter);
+        boxLayout->addSpacing(1);
+        boxLayout->setAlignment(Qt::AlignCenter | Qt::AlignVCenter);
+
+        if (DockManager::testConfigFlag(DockManager::FocusHighlighting))
+            m_closeButton->setCheckable(true);
 
         m_titleLabel->setVisible(true);
     }
@@ -201,7 +187,7 @@ namespace ADS
     void DockWidgetTabPrivate::moveTab(QMouseEvent *event)
     {
         event->accept();
-        QPoint distance = event->globalPos() - m_globalDragStartMousePosition;
+        QPoint distance = event->globalPosition().toPoint() - m_globalDragStartMousePosition;
         distance.setY(0);
         auto targetPos = distance + m_tabDragStartPosition;
         targetPos.rx() = qMax(targetPos.x(), 0);
@@ -226,18 +212,20 @@ namespace ADS
 
         qCInfo(adsLog) << "startFloating";
         m_dragState = draggingState;
-        QSize size = m_dockArea->size();
         AbstractFloatingWidget *floatingWidget = nullptr;
-        bool opaqueUndocking = DockManager::configFlags().testFlag(DockManager::OpaqueUndocking)
+        bool opaqueUndocking = DockManager::testConfigFlag(DockManager::OpaqueUndocking)
                                || (DraggingFloatingWidget != draggingState);
 
         // If section widget has multiple tabs, we take only one tab
         // If it has only one single tab, we can move the complete
         // dock area into floating widget
+        QSize size;
         if (m_dockArea->dockWidgetsCount() > 1) {
             floatingWidget = createFloatingWidget(m_dockWidget, opaqueUndocking);
+            size = m_dockWidget->size();
         } else {
             floatingWidget = createFloatingWidget(m_dockArea, opaqueUndocking);
+            size = m_dockArea->size();
         }
 
         if (DraggingFloatingWidget == draggingState) {
@@ -252,6 +240,39 @@ namespace ADS
         return true;
     }
 
+
+    TabButton::TabButton(QWidget *parent)
+        : TabButtonType(parent)
+        , m_active(false)
+        , m_focus(false)
+    {}
+
+    void TabButton::setActive(bool value) { m_active = value; }
+    void TabButton::setFocus(bool value) { m_focus = value; }
+
+    void TabButton::paintEvent(QPaintEvent *event)
+    {
+        Q_UNUSED(event)
+
+        QStylePainter p(this);
+        QStyleOptionToolButton opt;
+        initStyleOption(&opt);
+        opt.icon = QIcon(); // set to null icon otherwise it is drawn twice
+        p.drawComplexControl(QStyle::CC_ToolButton, opt);
+
+        QIcon::Mode mode = QIcon::Mode::Normal;
+        if (m_active)
+            mode = QIcon::Mode::Active;
+        if (m_focus)
+            mode = QIcon::Mode::Selected;
+
+        const QPoint iconPosition = rect().center() - QPoint(iconSize().width() * 0.5,
+                                                             iconSize().height() * 0.5);
+
+        p.drawPixmap(iconPosition, icon().pixmap(iconSize(), mode));
+    }
+
+
     DockWidgetTab::DockWidgetTab(DockWidget *dockWidget, QWidget *parent)
         : QFrame(parent)
         , d(new DockWidgetTabPrivate(this))
@@ -259,6 +280,8 @@ namespace ADS
         setAttribute(Qt::WA_NoMousePropagation, true);
         d->m_dockWidget = dockWidget;
         d->createLayout();
+        if (DockManager::testConfigFlag(DockManager::FocusHighlighting))
+            setFocusPolicy(Qt::ClickFocus);
     }
 
     DockWidgetTab::~DockWidgetTab()
@@ -271,7 +294,7 @@ namespace ADS
     {
         if (event->button() == Qt::LeftButton) {
             event->accept();
-            d->saveDragStartMousePosition(event->globalPos());
+            d->saveDragStartMousePosition(event->globalPosition().toPoint());
             d->m_dragState = DraggingMousePressed;
             emit clicked();
             return;
@@ -291,7 +314,7 @@ namespace ADS
             case DraggingTab:
                 // End of tab moving, emit signal
                 if (d->m_dockArea) {
-                    emit moved(event->globalPos());
+                    emit moved(event->globalPosition().toPoint());
                 }
                 break;
 
@@ -331,7 +354,7 @@ namespace ADS
         auto mappedPos = mapToParent(event->pos());
         bool mouseOutsideBar = (mappedPos.x() < 0) || (mappedPos.x() > parentWidget()->rect().right());
         // Maybe a fixed drag distance is better here ?
-        int dragDistanceY = qAbs(d->m_globalDragStartMousePosition.y() - event->globalPos().y());
+        int dragDistanceY = qAbs(d->m_globalDragStartMousePosition.y() - event->globalPosition().toPoint().y());
         if (dragDistanceY >= DockManager::startDragDistance() || mouseOutsideBar) {
             // If this is the last dock area in a dock container with only
             // one single dock widget it does not make  sense to move it to a new
@@ -352,21 +375,21 @@ namespace ADS
                 // If we undock, we need to restore the initial position of this
                 // tab because it looks strange if it remains on its dragged position
                 if (d->isDraggingState(DraggingTab)
-                    && !DockManager::configFlags().testFlag(DockManager::OpaqueUndocking)) {
+                    && !DockManager::testConfigFlag(DockManager::OpaqueUndocking))
                     parentWidget()->layout()->update();
-                }
+
                 d->startFloating();
             }
             return;
         } else if (d->m_dockArea->openDockWidgetsCount() > 1
-                   && (event->globalPos() - d->m_globalDragStartMousePosition).manhattanLength()
+                   && (event->globalPosition().toPoint() - d->m_globalDragStartMousePosition).manhattanLength()
                           >= QApplication::startDragDistance()) // Wait a few pixels before start moving
         {
             // If we start dragging the tab, we save its initial position to
             // restore it later
-            if (DraggingTab != d->m_dragState) {
+            if (DraggingTab != d->m_dragState)
                 d->m_tabDragStartPosition = this->pos();
-            }
+
             d->m_dragState = DraggingTab;
             return;
         }
@@ -377,9 +400,8 @@ namespace ADS
     void DockWidgetTab::contextMenuEvent(QContextMenuEvent *event)
     {
         event->accept();
-        if (d->isDraggingState(DraggingFloatingWidget)) {
+        if (d->isDraggingState(DraggingFloatingWidget))
             return;
-        }
 
         d->saveDragStartMousePosition(event->globalPos());
         QMenu menu(this);
@@ -388,12 +410,12 @@ namespace ADS
         const bool isNotOnlyTabInContainer =  !d->m_dockArea->dockContainer()->hasTopLevelDockWidget();
         const bool isDetachable = isFloatable && isNotOnlyTabInContainer;
 
-        auto action = menu.addAction(tr("Detach"), this, &DockWidgetTab::detachDockWidget);
+        auto action = menu.addAction(Tr::tr("Detach"), this, &DockWidgetTab::detachDockWidget);
         action->setEnabled(isDetachable);
         menu.addSeparator();
-        action = menu.addAction(tr("Close"), this, &DockWidgetTab::closeRequested);
+        action = menu.addAction(Tr::tr("Close"), this, &DockWidgetTab::closeRequested);
         action->setEnabled(isClosable());
-        menu.addAction(tr("Close Others"), this, &DockWidgetTab::closeOtherTabsRequested);
+        menu.addAction(Tr::tr("Close Others"), this, &DockWidgetTab::closeOtherTabsRequested);
         menu.exec(event->globalPos());
     }
 
@@ -407,16 +429,28 @@ namespace ADS
         bool allTabsHaveCloseButton = d->testConfigFlag(DockManager::AllTabsHaveCloseButton);
         bool tabHasCloseButton = (activeTabHasCloseButton && active) | allTabsHaveCloseButton;
         d->m_closeButton->setVisible(dockWidgetClosable && tabHasCloseButton);
-        if (d->m_isActiveTab == active) {
+
+        d->m_closeButton->setActive(active);
+
+        // Focus related stuff
+        if (DockManager::testConfigFlag(DockManager::FocusHighlighting)
+            && !d->m_dockWidget->dockManager()->isRestoringState()) {
+            bool updateFocusStyle = false;
+            if (active && !hasFocus()) {
+                setFocus(Qt::OtherFocusReason);
+                updateFocusStyle = true;
+            }
+            if (d->m_isActiveTab == active) {
+                if (updateFocusStyle)
+                    updateStyle();
+                return;
+            }
+        } else if (d->m_isActiveTab == active) {
             return;
         }
 
         d->m_isActiveTab = active;
-
-        style()->unpolish(this);
-        style()->polish(this);
-        d->m_titleLabel->style()->unpolish(d->m_titleLabel);
-        d->m_titleLabel->style()->polish(d->m_titleLabel);
+        updateStyle();
         update();
         updateGeometry();
 
@@ -432,9 +466,8 @@ namespace ADS
     void DockWidgetTab::setIcon(const QIcon &icon)
     {
         QBoxLayout *boxLayout = qobject_cast<QBoxLayout *>(layout());
-        if (!d->m_iconLabel && icon.isNull()) {
+        if (!d->m_iconLabel && icon.isNull())
             return;
-        }
 
         if (!d->m_iconLabel) {
             d->m_iconLabel = new QLabel();
@@ -469,7 +502,7 @@ namespace ADS
         // sense to move it to a new floating widget and leave this one empty
         if ((!d->m_dockArea->dockContainer()->isFloating() || d->m_dockArea->dockWidgetsCount() > 1)
             && d->m_dockWidget->features().testFlag(DockWidget::DockWidgetFloatable)) {
-            d->saveDragStartMousePosition(event->globalPos());
+            d->saveDragStartMousePosition(event->globalPosition().toPoint());
             d->startFloating(DraggingInactive);
         }
 
@@ -493,9 +526,9 @@ namespace ADS
 
     void DockWidgetTab::detachDockWidget()
     {
-        if (!d->m_dockWidget->features().testFlag(DockWidget::DockWidgetFloatable)) {
+        if (!d->m_dockWidget->features().testFlag(DockWidget::DockWidgetFloatable))
             return;
-        }
+
         d->saveDragStartMousePosition(QCursor::pos());
         d->startFloating(DraggingInactive);
     }
@@ -508,6 +541,7 @@ namespace ADS
             d->m_titleLabel->setToolTip(text);
         }
 #endif
+
         return Super::event(event);
     }
 
@@ -521,4 +555,17 @@ namespace ADS
         d->m_closeButton->setSizePolicy(sizePolicy);
     }
 
+    void DockWidgetTab::setElideMode(Qt::TextElideMode mode)
+    {
+        d->m_titleLabel->setElideMode(mode);
+    }
+
+    void DockWidgetTab::updateStyle()
+    {
+        if (DockManager::testConfigFlag(DockManager::FocusHighlighting))
+            d->m_closeButton->setFocus(property("focused").toBool());
+
+        internal::repolishStyle(this, internal::RepolishDirectChildren);
+    }
+
 } // namespace ADS
