@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 /*
   Copyright 2005 Roberto Raggi <roberto@kdevelop.org>
@@ -52,32 +30,31 @@
 #include <cplusplus/Literals.h>
 #include <cplusplus/cppassert.h>
 
-#include <utils/executeondestruction.h>
+#include <utils/filepath.h>
 #include <utils/scopedswap.h>
 
 #include <QDebug>
 #include <QList>
 #include <QDate>
+#include <QLoggingCategory>
 #include <QTime>
 #include <QPair>
+#include <QScopeGuard>
 
 #include <cctype>
+#include <deque>
 #include <list>
 #include <algorithm>
 
-#define NO_DEBUG
-
-#ifndef NO_DEBUG
-#  include <iostream>
-#endif // NO_DEBUG
-
-#include <deque>
+// FIXME: This is used for errors that should appear in the editor.
+static Q_LOGGING_CATEGORY(lexerLog, "qtc.cpp.lexer", QtWarningMsg)
 
 using namespace Utils;
 
 namespace {
 enum {
     MAX_FUNCTION_LIKE_ARGUMENTS_COUNT = 100,
+    MAX_INCLUDE_DEPTH = 200,
     MAX_TOKEN_EXPANSION_COUNT = 5000,
     MAX_TOKEN_BUFFER_DEPTH = 16000 // for when macros are using some kind of right-folding, this is the list of "delayed" buffers waiting to be expanded after the current one.
 };
@@ -139,13 +116,6 @@ static bool isQtReservedWord(const char *name, int size)
     return false;
 }
 
-static void nestingTooDeep()
-{
-#ifndef NO_DEBUG
-        std::cerr << "*** WARNING #if / #ifdef nesting exceeded the max level " << MAX_LEVEL << std::endl;
-#endif
-}
-
 } // anonymous namespace
 
 namespace CPlusPlus {
@@ -223,6 +193,24 @@ struct Value
     inline bool is_zero () const
     { return l == 0; }
 
+    template<typename T> static int cmpImpl(T v1, T v2)
+    {
+        if (v1 < v2)
+            return -1;
+        if (v1 > v2)
+            return 1;
+        return 0;
+    }
+    Value cmp(const Value &other) const
+    {
+        Value v = *this;
+        if (v.is_ulong() || other.is_ulong())
+            v.set_long(cmpImpl(v.ul, other.ul));
+        else
+            v.set_long(cmpImpl(v.l, other.l));
+        return v;
+    }
+
 #define PP_DEFINE_BIN_OP(name, op) \
     inline Value operator op(const Value &other) const \
     { \
@@ -257,7 +245,6 @@ struct Value
 };
 
 } // namespace Internal
-} // namespace CPlusPlus
 
 using namespace CPlusPlus;
 using namespace CPlusPlus::Internal;
@@ -273,7 +260,7 @@ Macro *macroDefinition(const ByteArrayRef &name,
                        unsigned bytesOffset,
                        unsigned utf16charsOffset,
                        unsigned line,
-                       Environment *env,
+                       CPlusPlus::Environment *env,
                        Client *client)
 {
     Macro *m = env->resolve(name);
@@ -342,7 +329,7 @@ class ExpressionEvaluator
     void operator = (const ExpressionEvaluator &other);
 
 public:
-    ExpressionEvaluator(Client *client, Environment *env)
+    ExpressionEvaluator(Client *client, CPlusPlus::Environment *env)
         : client(client), env(env), _lex(nullptr)
     { }
 
@@ -510,24 +497,25 @@ private:
     inline int precedence(int tokenKind) const
     {
         switch (tokenKind) {
-        case T_PIPE_PIPE:       return 0;
-        case T_AMPER_AMPER:     return 1;
-        case T_PIPE:            return 2;
-        case T_CARET:           return 3;
-        case T_AMPER:           return 4;
+        case T_PIPE_PIPE:           return 0;
+        case T_AMPER_AMPER:         return 1;
+        case T_PIPE:                return 2;
+        case T_CARET:               return 3;
+        case T_AMPER:               return 4;
         case T_EQUAL_EQUAL:
-        case T_EXCLAIM_EQUAL:   return 5;
+        case T_EXCLAIM_EQUAL:       return 5;
         case T_GREATER:
         case T_LESS:
         case T_LESS_EQUAL:
-        case T_GREATER_EQUAL:   return 6;
+        case T_GREATER_EQUAL:       return 6;
+        case T_LESS_EQUAL_GREATER:  return 7;
         case T_LESS_LESS:
-        case T_GREATER_GREATER: return 7;
+        case T_GREATER_GREATER:     return 8;
         case T_PLUS:
-        case T_MINUS:           return 8;
+        case T_MINUS:               return 9;
         case T_STAR:
         case T_SLASH:
-        case T_PERCENT:         return 9;
+        case T_PERCENT:             return 10;
 
         default:
             return -1;
@@ -547,6 +535,7 @@ private:
         case T_GREATER:
         case T_LESS:
         case T_LESS_EQUAL:
+        case T_LESS_EQUAL_GREATER:
         case T_GREATER_EQUAL:
         case T_LESS_LESS:
         case T_GREATER_GREATER:
@@ -565,24 +554,25 @@ private:
     static inline Value evaluate_expression(int tokenKind, const Value &lhs, const Value &rhs)
     {
         switch (tokenKind) {
-        case T_PIPE_PIPE:       return lhs || rhs;
-        case T_AMPER_AMPER:     return lhs && rhs;
-        case T_PIPE:            return lhs | rhs;
-        case T_CARET:           return lhs ^ rhs;
-        case T_AMPER:           return lhs & rhs;
-        case T_EQUAL_EQUAL:     return lhs == rhs;
-        case T_EXCLAIM_EQUAL:   return lhs != rhs;
-        case T_GREATER:         return lhs > rhs;
-        case T_LESS:            return lhs < rhs;
-        case T_LESS_EQUAL:      return lhs <= rhs;
-        case T_GREATER_EQUAL:   return lhs >= rhs;
-        case T_LESS_LESS:       return lhs << rhs;
-        case T_GREATER_GREATER: return lhs >> rhs;
-        case T_PLUS:            return lhs + rhs;
-        case T_MINUS:           return lhs - rhs;
-        case T_STAR:            return lhs * rhs;
-        case T_SLASH:           return rhs.is_zero() ? Value() : lhs / rhs;
-        case T_PERCENT:         return rhs.is_zero() ? Value() : lhs % rhs;
+        case T_PIPE_PIPE:            return lhs || rhs;
+        case T_AMPER_AMPER:          return lhs && rhs;
+        case T_PIPE:                 return lhs | rhs;
+        case T_CARET:                return lhs ^ rhs;
+        case T_AMPER:                return lhs & rhs;
+        case T_EQUAL_EQUAL:          return lhs == rhs;
+        case T_EXCLAIM_EQUAL:        return lhs != rhs;
+        case T_GREATER:              return lhs > rhs;
+        case T_LESS:                 return lhs < rhs;
+        case T_LESS_EQUAL:           return lhs <= rhs;
+        case T_LESS_EQUAL_GREATER:   return lhs.cmp(rhs);
+        case T_GREATER_EQUAL:        return lhs >= rhs;
+        case T_LESS_LESS:            return lhs << rhs;
+        case T_GREATER_GREATER:      return lhs >> rhs;
+        case T_PLUS:                 return lhs + rhs;
+        case T_MINUS:                return lhs - rhs;
+        case T_STAR:                 return lhs * rhs;
+        case T_SLASH:                return rhs.is_zero() ? Value() : lhs / rhs;
+        case T_PERCENT:              return rhs.is_zero() ? Value() : lhs % rhs;
 
         default:
             return Value();
@@ -738,7 +728,11 @@ void Preprocessor::State::updateIncludeGuardState_helper(IncludeGuardStateHint h
 #endif // DEBUG_INCLUDE_GUARD_TRACKING
 }
 
-QString Preprocessor::configurationFileName() { return QStringLiteral("<configuration>"); }
+const FilePath &Preprocessor::configurationFileName()
+{
+    const static FilePath configurationFile = FilePath::fromPathPart(u"<configuration>");
+    return configurationFile;
+}
 
 Preprocessor::Preprocessor(Client *client, Environment *env)
     : m_client(client)
@@ -748,9 +742,12 @@ Preprocessor::Preprocessor(Client *client, Environment *env)
 {
 }
 
-QByteArray Preprocessor::run(const QString &fileName, const QString &source)
+QByteArray Preprocessor::run(const Utils::FilePath &filePath,
+                             const QByteArray &source,
+                             bool noLines,
+                             bool markGeneratedTokens)
 {
-    return run(fileName, source.toUtf8());
+    return run(filePath.toString(), source, noLines, markGeneratedTokens);
 }
 
 QByteArray Preprocessor::run(const QString &fileName,
@@ -1133,7 +1130,8 @@ bool Preprocessor::handleIdentifier(PPToken *tk)
         }
     }
 
-    m_state.pushTokenBuffer(body.constBegin(), body.constEnd(), macro);
+    const PPToken *start = body.constData();
+    m_state.pushTokenBuffer(start, start + body.size(), macro);
 
     if (m_client && !idTk.generated())
         m_client->stopExpandingMacro(idTk.byteOffset, *macro);
@@ -1447,7 +1445,7 @@ void Preprocessor::preprocess(const QString &fileName, const QByteArray &source,
                 trackedLine = tk.lineno;
                 trackedColumn = unsigned(computeDistance(tk, true));
             }
-            m_state.m_expandedTokensInfo.append(qMakePair(trackedLine, trackedColumn));
+            m_state.m_expandedTokensInfo.push_back({trackedLine, trackedColumn});
         } else if (m_state.m_expansionStatus == JustFinishedExpansion) {
             m_state.setExpansionStatus(NotExpanding);
             macroExpanded = true;
@@ -1515,14 +1513,15 @@ bool Preprocessor::collectActualArguments(PPToken *tk, QVector<QVector<PPToken>
     Q_ASSERT(tk);
     Q_ASSERT(actuals);
 
-    ExecuteOnDestruction removeBlockedName;
-    if (m_state.m_tokenBuffer) {
-        removeBlockedName.reset([this] {
-            if (m_state.m_tokenBuffer && !m_state.m_tokenBuffer->blockedMacroNames.empty())
-                m_state.m_tokenBuffer->blockedMacroNames.pop_back();
-        });
+    QScopeGuard cleanup([this] {
+        if (m_state.m_tokenBuffer && !m_state.m_tokenBuffer->blockedMacroNames.empty())
+            m_state.m_tokenBuffer->blockedMacroNames.pop_back();
+    });
+
+    if (m_state.m_tokenBuffer)
         m_state.m_tokenBuffer->blockedMacroNames.push_back(parentMacroName);
-    }
+    else
+        cleanup.dismiss();
 
     lex(tk); // consume the identifier
 
@@ -1670,6 +1669,12 @@ void Preprocessor::handleIncludeDirective(PPToken *tk, bool includeNext)
     if (m_cancelChecker && m_cancelChecker())
         return;
 
+    GuardLocker depthLocker(m_includeDepthGuard);
+    if (m_includeDepthGuard.lockCount() > MAX_INCLUDE_DEPTH) {
+        qCWarning(lexerLog) << "Maximum include depth exceeded" << m_state.m_currentFileName;
+        return;
+    }
+
     m_state.m_lexer->setScanAngleStringLiteralTokens(true);
     lex(tk); // consume "include" token
     m_state.m_lexer->setScanAngleStringLiteralTokens(false);
@@ -1702,7 +1707,7 @@ void Preprocessor::handleIncludeDirective(PPToken *tk, bool includeNext)
 
     if (m_client) {
         QString inc = QString::fromUtf8(included.constData() + 1, included.size() - 2);
-        m_client->sourceNeeded(line, inc, mode);
+        m_client->sourceNeeded(line, FilePath::fromString(inc), mode);
     }
 }
 
@@ -1718,7 +1723,7 @@ void Preprocessor::handleDefineDirective(PPToken *tk)
         return;
 
     Macro macro;
-    macro.setFileName(m_env->currentFile);
+    macro.setFilePath(FilePath::fromString(m_env->currentFile));
     macro.setLine(tk->lineno);
     QByteArray macroName = tk->asByteArrayRef().toByteArray();
     macro.setName(macroName);
@@ -1912,10 +1917,8 @@ void Preprocessor::handleIfDirective(PPToken *tk)
     Value result;
     const PPToken lastExpressionToken = evalExpression(tk, result);
 
-    if (m_state.m_ifLevel >= MAX_LEVEL - 1) {
-        nestingTooDeep();
+    if (!checkConditionalNesting())
         return;
-    }
 
     const bool value = !result.is_zero();
 
@@ -1936,7 +1939,7 @@ void Preprocessor::handleIfDirective(PPToken *tk)
 void Preprocessor::handleElifDirective(PPToken *tk, const PPToken &poundToken)
 {
     if (m_state.m_ifLevel == 0) {
-//        std::cerr << "*** WARNING #elif without #if" << std::endl;
+        qCWarning(lexerLog) << "#elif without #if";
         handleIfDirective(tk);
     } else {
         lex(tk); // consume "elif" token
@@ -1983,22 +1986,18 @@ void Preprocessor::handleElseDirective(PPToken *tk, const PPToken &poundToken)
             else if (m_client && !wasSkipping && startSkipping)
                 startSkippingBlocks(poundToken);
         }
-#ifndef NO_DEBUG
     } else {
-        std::cerr << "*** WARNING #else without #if" << std::endl;
-#endif // NO_DEBUG
+        qCWarning(lexerLog) << "#else without #if";
     }
 }
 
 void Preprocessor::handleEndIfDirective(PPToken *tk, const PPToken &poundToken)
 {
     if (m_state.m_ifLevel == 0) {
-#ifndef NO_DEBUG
-        std::cerr << "*** WARNING #endif without #if";
+        qCWarning(lexerLog) << "#endif without #if";
         if (!tk->generated())
-            std::cerr << " on line " << tk->lineno << " of file " << m_state.m_currentFileName.toUtf8().constData();
-        std::cerr << std::endl;
-#endif // NO_DEBUG
+            qCWarning(lexerLog) << "on line" << tk->lineno << "of file"
+                              << m_state.m_currentFileName.toUtf8().constData();
     } else {
         bool wasSkipping = m_state.m_skipping[m_state.m_ifLevel];
         m_state.m_skipping[m_state.m_ifLevel] = false;
@@ -2029,7 +2028,7 @@ void Preprocessor::handleIfDefDirective(bool checkUndefined, PPToken *tk)
 
             // the macro is a feature constraint(e.g. QT_NO_XXX)
             if (checkUndefined && macroName.startsWith("QT_NO_")) {
-                if (macro->fileName() == configurationFileName()) {
+                if (macro->filePath() == configurationFileName()) {
                     // and it' defined in a pro file (e.g. DEFINES += QT_NO_QOBJECT)
 
                     value = false; // take the branch
@@ -2044,22 +2043,18 @@ void Preprocessor::handleIfDefDirective(bool checkUndefined, PPToken *tk)
 
         const bool wasSkipping = m_state.m_skipping[m_state.m_ifLevel];
 
-        if (m_state.m_ifLevel < MAX_LEVEL - 1) {
+        if (checkConditionalNesting()) {
             ++m_state.m_ifLevel;
             m_state.m_trueTest[m_state.m_ifLevel] = value;
             m_state.m_skipping[m_state.m_ifLevel] = wasSkipping ? wasSkipping : !value;
 
             if (m_client && !wasSkipping && !value)
                 startSkippingBlocks(*tk);
-        } else {
-            nestingTooDeep();
         }
 
         lex(tk); // consume the identifier
-#ifndef NO_DEBUG
     } else {
-        std::cerr << "*** WARNING #ifdef without identifier" << std::endl;
-#endif // NO_DEBUG
+        qCWarning(lexerLog) << "#ifdef without identifier";
     }
 }
 
@@ -2086,10 +2081,8 @@ void Preprocessor::handleUndefDirective(PPToken *tk)
             m_client->macroAdded(*macro);
         }
         lex(tk); // consume macro name
-#ifndef NO_DEBUG
     } else {
-        std::cerr << "*** WARNING #undef without identifier" << std::endl;
-#endif // NO_DEBUG
+        qCWarning(lexerLog) << "#undef without identifier";
     }
 }
 
@@ -2185,3 +2178,15 @@ void Preprocessor::maybeStartOutputLine()
     if (*ch == '\\')
         buffer.append('\n');
 }
+
+bool Preprocessor::checkConditionalNesting() const
+{
+    if (m_state.m_ifLevel >= MAX_LEVEL - 1) {
+        qCWarning(lexerLog) << "#if/#ifdef nesting exceeding maximum level" << MAX_LEVEL;
+        return false;
+    }
+    return true;
+}
+
+
+} // namespace CPlusPlus
