@@ -17,6 +17,8 @@
 #include <android/androidconstants.h>
 
 #include <coreplugin/icore.h>
+#include <coreplugin/documentmanager.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
@@ -30,7 +32,11 @@
 #include <projectexplorer/target.h>
 #include <projectexplorer/taskhub.h>
 
+#include <texteditor/texteditor.h>
+#include <texteditor/textdocument.h>
+
 #include <qmljs/qmljsmodelmanagerinterface.h>
+#include <qmljstools/qmljstoolsconstants.h>
 #include <qtsupport/qtcppkitinfo.h>
 #include <qtsupport/qtkitinformation.h>
 
@@ -40,8 +46,8 @@
 #include <utils/checkablemessagebox.h>
 #include <utils/fileutils.h>
 #include <utils/macroexpander.h>
+#include <utils/process.h>
 #include <utils/qtcassert.h>
-#include <utils/qtcprocess.h>
 
 #include <QClipboard>
 #include <QGuiApplication>
@@ -51,74 +57,11 @@
 #include <QLoggingCategory>
 
 using namespace ProjectExplorer;
+using namespace TextEditor;
 using namespace Utils;
 
 namespace CMakeProjectManager::Internal {
 
-static void copySourcePathsToClipboard(const FilePaths &srcPaths, const ProjectNode *node)
-{
-    QClipboard *clip = QGuiApplication::clipboard();
-
-    QString data = Utils::transform(srcPaths, [projDir = node->filePath()](const FilePath &path) {
-                       return path.relativePathFrom(projDir).cleanPath().toString();
-                   }).join(" ");
-    clip->setText(data);
-}
-
-static void noAutoAdditionNotify(const FilePaths &filePaths, const ProjectNode *node)
-{
-    const FilePaths srcPaths = Utils::filtered(filePaths, [](const FilePath &file) {
-        const auto mimeType = Utils::mimeTypeForFile(file).name();
-        return mimeType == CppEditor::Constants::C_SOURCE_MIMETYPE ||
-               mimeType == CppEditor::Constants::C_HEADER_MIMETYPE ||
-               mimeType == CppEditor::Constants::CPP_SOURCE_MIMETYPE ||
-               mimeType == CppEditor::Constants::CPP_HEADER_MIMETYPE ||
-               mimeType == ProjectExplorer::Constants::FORM_MIMETYPE ||
-               mimeType == ProjectExplorer::Constants::RESOURCE_MIMETYPE ||
-               mimeType == ProjectExplorer::Constants::SCXML_MIMETYPE;
-    });
-
-    if (!srcPaths.empty()) {
-        auto settings = CMakeSpecificSettings::instance();
-        switch (settings->afterAddFileSetting.value()) {
-        case AskUser: {
-            bool checkValue{false};
-            QDialogButtonBox::StandardButton reply = CheckableMessageBox::question(
-                Core::ICore::dialogParent(),
-                Tr::tr("Copy to Clipboard?"),
-                Tr::tr("Files are not automatically added to the "
-                                "CMakeLists.txt file of the CMake project."
-                                "\nCopy the path to the source files to the clipboard?"),
-                "Remember My Choice",
-                &checkValue,
-                QDialogButtonBox::Yes | QDialogButtonBox::No,
-                QDialogButtonBox::Yes);
-            if (checkValue) {
-                if (QDialogButtonBox::Yes == reply)
-                    settings->afterAddFileSetting.setValue(CopyFilePath);
-                else if (QDialogButtonBox::No == reply)
-                    settings->afterAddFileSetting.setValue(NeverCopyFilePath);
-
-                settings->writeSettings(Core::ICore::settings());
-            }
-
-            if (QDialogButtonBox::Yes == reply)
-                copySourcePathsToClipboard(srcPaths, node);
-
-            break;
-        }
-
-        case CopyFilePath: {
-            copySourcePathsToClipboard(srcPaths, node);
-            break;
-        }
-
-        case NeverCopyFilePath:
-            break;
-        }
-    }
-}
-
 static Q_LOGGING_CATEGORY(cmakeBuildSystemLog, "qtc.cmake.buildsystem", QtWarningMsg);
 
 // --------------------------------------------------------------------
@@ -232,8 +175,8 @@ void CMakeBuildSystem::triggerParsing()
     qCDebug(cmakeBuildSystemLog) << "Parse called with flags:"
                                  << reparseParametersString(reparseParameters);
 
-    const QString cache = m_parameters.buildDirectory.pathAppended("CMakeCache.txt").toString();
-    if (!QFileInfo::exists(cache)) {
+    const FilePath cache = m_parameters.buildDirectory.pathAppended("CMakeCache.txt");
+    if (!cache.exists()) {
         reparseParameters |= REPARSE_FORCE_INITIAL_CONFIGURATION | REPARSE_FORCE_CMAKE_RUN;
         qCDebug(cmakeBuildSystemLog)
             << "No" << cache
@@ -258,29 +201,447 @@ void CMakeBuildSystem::triggerParsing()
 bool CMakeBuildSystem::supportsAction(Node *context, ProjectAction action, const Node *node) const
 {
     if (dynamic_cast<CMakeTargetNode *>(context))
-        return action == ProjectAction::AddNewFile;
-
-    if (dynamic_cast<CMakeListsNode *>(context))
-        return action == ProjectAction::AddNewFile;
+        return action == ProjectAction::AddNewFile || action == ProjectAction::AddExistingFile
+               || action == ProjectAction::AddExistingDirectory || action == ProjectAction::Rename
+               || action == ProjectAction::RemoveFile;
 
     return BuildSystem::supportsAction(context, action, node);
 }
 
-bool CMakeBuildSystem::addFiles(Node *context, const FilePaths &filePaths, FilePaths *notAdded)
+static QString newFilesForFunction(const std::string &cmakeFunction,
+                                   const FilePaths &filePaths,
+                                   const FilePath &projDir)
 {
-    if (auto n = dynamic_cast<CMakeProjectNode *>(context)) {
-        noAutoAdditionNotify(filePaths, n);
-        return true; // Return always true as autoadd is not supported!
+    auto relativeFilePaths = [projDir](const FilePaths &filePaths) {
+        return Utils::transform(filePaths, [projDir](const FilePath &path) {
+            return path.canonicalPath().relativePathFrom(projDir).cleanPath().toString();
+        });
+    };
+
+    if (cmakeFunction == "qt_add_qml_module" || cmakeFunction == "qt6_add_qml_module") {
+        FilePaths sourceFiles;
+        FilePaths resourceFiles;
+        FilePaths qmlFiles;
+
+        for (const auto &file : filePaths) {
+            const auto mimeType = Utils::mimeTypeForFile(file);
+            if (mimeType.matchesName(CppEditor::Constants::CPP_SOURCE_MIMETYPE)
+                || mimeType.matchesName(CppEditor::Constants::CPP_HEADER_MIMETYPE)
+                || mimeType.matchesName(CppEditor::Constants::OBJECTIVE_C_SOURCE_MIMETYPE)
+                || mimeType.matchesName(CppEditor::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE)) {
+                sourceFiles << file;
+            } else if (mimeType.matchesName(QmlJSTools::Constants::QML_MIMETYPE)
+                       || mimeType.matchesName(QmlJSTools::Constants::QMLUI_MIMETYPE)
+                       || mimeType.matchesName(QmlJSTools::Constants::QMLPROJECT_MIMETYPE)
+                       || mimeType.matchesName(QmlJSTools::Constants::JS_MIMETYPE)
+                       || mimeType.matchesName(QmlJSTools::Constants::JSON_MIMETYPE)) {
+                qmlFiles << file;
+            } else {
+                resourceFiles << file;
+            }
+        }
+
+        QStringList result;
+        if (!sourceFiles.isEmpty())
+            result << QString("SOURCES %1").arg(relativeFilePaths(sourceFiles).join(" "));
+        if (!resourceFiles.isEmpty())
+            result << QString("RESOURCES %1").arg(relativeFilePaths(resourceFiles).join(" "));
+        if (!qmlFiles.isEmpty())
+            result << QString("QML_FILES %1").arg(relativeFilePaths(qmlFiles).join(" "));
+
+        return result.join("\n");
     }
 
+    return relativeFilePaths(filePaths).join(" ");
+}
+
+bool CMakeBuildSystem::addFiles(Node *context, const FilePaths &filePaths, FilePaths *notAdded)
+{
     if (auto n = dynamic_cast<CMakeTargetNode *>(context)) {
-        noAutoAdditionNotify(filePaths, n);
-        return true; // Return always true as autoadd is not supported!
+        const QString targetName = n->buildKey();
+        auto target = Utils::findOrDefault(buildTargets(),
+                                           [targetName](const CMakeBuildTarget &target) {
+                                               return target.title == targetName;
+                                           });
+
+        if (target.backtrace.isEmpty()) {
+            *notAdded = filePaths;
+            return false;
+        }
+        const FilePath targetCMakeFile = target.backtrace.last().path;
+        const int targetDefinitionLine = target.backtrace.last().line;
+
+        // Have a fresh look at the CMake file, not relying on a cached value
+        Core::DocumentManager::saveModifiedDocumentSilently(
+            Core::DocumentModel::documentForFilePath(targetCMakeFile));
+        expected_str<QByteArray> fileContent = targetCMakeFile.fileContents();
+        cmListFile cmakeListFile;
+        std::string errorString;
+        if (fileContent) {
+            fileContent = fileContent->replace("\r\n", "\n");
+            if (!cmakeListFile.ParseString(fileContent->toStdString(),
+                                           targetCMakeFile.fileName().toStdString(),
+                                           errorString)) {
+                *notAdded = filePaths;
+                return false;
+            }
+        }
+
+        auto function = std::find_if(cmakeListFile.Functions.begin(),
+                                     cmakeListFile.Functions.end(),
+                                     [targetDefinitionLine](const auto &func) {
+                                         return func.Line() == targetDefinitionLine;
+                                     });
+
+        if (function == cmakeListFile.Functions.end()) {
+            *notAdded = filePaths;
+            return false;
+        }
+
+        // Special case: when qt_add_executable and qt_add_qml_module use the same target name
+        // then qt_add_qml_module function should be used
+        const std::string target_name = targetName.toStdString();
+        auto add_qml_module_func
+            = std::find_if(cmakeListFile.Functions.begin(),
+                           cmakeListFile.Functions.end(),
+                           [target_name](const auto &func) {
+                               return (func.LowerCaseName() == "qt_add_qml_module"
+                                       || func.LowerCaseName() == "qt6_add_qml_module")
+                                      && func.Arguments().front().Value == target_name;
+                           });
+        if (add_qml_module_func != cmakeListFile.Functions.end())
+            function = add_qml_module_func;
+
+        const QString newSourceFiles = newFilesForFunction(function->LowerCaseName(),
+                                                           filePaths,
+                                                           n->filePath().canonicalPath());
+
+        static QSet<std::string> knownFunctions{"add_executable",
+                                                "add_library",
+                                                "qt_add_executable",
+                                                "qt_add_library",
+                                                "qt6_add_executable",
+                                                "qt6_add_library",
+                                                "qt_add_qml_module",
+                                                "qt6_add_qml_module"};
+
+        int line = 0;
+        int column = 0;
+        int extraChars = 0;
+        QString snippet;
+
+        auto afterFunctionLastArgument =
+            [&line, &column, &snippet, &extraChars, newSourceFiles](const auto &f) {
+                auto lastArgument = f->Arguments().back();
+
+                line = lastArgument.Line;
+                column = lastArgument.Column + static_cast<int>(lastArgument.Value.size()) - 1;
+                snippet = QString("\n%1").arg(newSourceFiles);
+
+                // Take into consideration the quotes
+                if (lastArgument.Delim == cmListFileArgument::Quoted)
+                    extraChars = 2;
+            };
+
+        if (knownFunctions.contains(function->LowerCaseName())) {
+            afterFunctionLastArgument(function);
+        } else {
+            auto targetSourcesFunc = std::find_if(cmakeListFile.Functions.begin(),
+                                                  cmakeListFile.Functions.end(),
+                                                  [target_name](const auto &func) {
+                                                      return func.LowerCaseName()
+                                                                 == "target_sources"
+                                                             && func.Arguments().front().Value
+                                                                    == target_name;
+                                                  });
+
+            if (targetSourcesFunc == cmakeListFile.Functions.end()) {
+                line = function->LineEnd() + 1;
+                column = 0;
+                snippet = QString("\ntarget_sources(%1\n  PRIVATE\n    %2\n)\n")
+                              .arg(targetName)
+                              .arg(newSourceFiles);
+            } else {
+                afterFunctionLastArgument(targetSourcesFunc);
+            }
+        }
+
+        BaseTextEditor *editor = qobject_cast<BaseTextEditor *>(
+            Core::EditorManager::openEditorAt({targetCMakeFile, line, column + extraChars},
+                                              Constants::CMAKE_EDITOR_ID,
+                                              Core::EditorManager::DoNotMakeVisible));
+        if (!editor) {
+            *notAdded = filePaths;
+            return false;
+        }
+
+        editor->insert(snippet);
+        editor->editorWidget()->autoIndent();
+        if (!Core::DocumentManager::saveDocument(editor->document()))
+            return false;
+
+        return true;
     }
 
     return BuildSystem::addFiles(context, filePaths, notAdded);
 }
 
+std::optional<CMakeBuildSystem::ProjectFileArgumentPosition>
+CMakeBuildSystem::projectFileArgumentPosition(const QString &targetName, const QString &fileName)
+{
+    auto target = Utils::findOrDefault(buildTargets(), [targetName](const CMakeBuildTarget &target) {
+        return target.title == targetName;
+    });
+
+    if (target.backtrace.isEmpty())
+        return std::nullopt;
+
+    const FilePath targetCMakeFile = target.backtrace.last().path;
+
+    // Have a fresh look at the CMake file, not relying on a cached value
+    Core::DocumentManager::saveModifiedDocumentSilently(
+        Core::DocumentModel::documentForFilePath(targetCMakeFile));
+    expected_str<QByteArray> fileContent = targetCMakeFile.fileContents();
+    cmListFile cmakeListFile;
+    std::string errorString;
+    if (fileContent) {
+        fileContent = fileContent->replace("\r\n", "\n");
+        if (!cmakeListFile.ParseString(fileContent->toStdString(),
+                                       targetCMakeFile.fileName().toStdString(),
+                                       errorString))
+            return std::nullopt;
+    }
+
+    const int targetDefinitionLine = target.backtrace.last().line;
+
+    auto function = std::find_if(cmakeListFile.Functions.begin(),
+                                 cmakeListFile.Functions.end(),
+                                 [targetDefinitionLine](const auto &func) {
+                                     return func.Line() == targetDefinitionLine;
+                                 });
+
+    const std::string target_name = targetName.toStdString();
+    auto targetSourcesFunc = std::find_if(cmakeListFile.Functions.begin(),
+                                          cmakeListFile.Functions.end(),
+                                          [target_name](const auto &func) {
+                                              return func.LowerCaseName() == "target_sources"
+                                                     && func.Arguments().size() > 1
+                                                     && func.Arguments().front().Value
+                                                            == target_name;
+                                          });
+    auto addQmlModuleFunc = std::find_if(cmakeListFile.Functions.begin(),
+                                         cmakeListFile.Functions.end(),
+                                         [target_name](const auto &func) {
+                                             return (func.LowerCaseName() == "qt_add_qml_module"
+                                                     || func.LowerCaseName() == "qt6_add_qml_module")
+                                                    && func.Arguments().size() > 1
+                                                    && func.Arguments().front().Value
+                                                           == target_name;
+                                         });
+
+    for (const auto &func : {function, targetSourcesFunc, addQmlModuleFunc}) {
+        if (func == cmakeListFile.Functions.end())
+            continue;
+        auto filePathArgument = Utils::findOrDefault(func->Arguments(),
+                                                     [file_name = fileName.toStdString()](
+                                                         const auto &arg) {
+                                                         return arg.Value == file_name;
+                                                     });
+
+        if (!filePathArgument.Value.empty()) {
+            return ProjectFileArgumentPosition{filePathArgument, targetCMakeFile, fileName};
+        } else {
+            // Check if the filename is part of globbing variable result
+            const auto globFunctions = std::get<0>(
+                Utils::partition(cmakeListFile.Functions, [](const auto &f) {
+                    return f.LowerCaseName() == "file" && f.Arguments().size() > 2
+                           && (f.Arguments().front().Value == "GLOB"
+                               || f.Arguments().front().Value == "GLOB_RECURSE");
+                }));
+
+            const auto globVariables = Utils::transform<QSet>(globFunctions, [](const auto &func) {
+                return std::string("${") + func.Arguments()[1].Value + "}";
+            });
+
+            const auto haveGlobbing = Utils::anyOf(func->Arguments(),
+                                                   [globVariables](const auto &arg) {
+                                                       return globVariables.contains(arg.Value);
+                                                   });
+
+            if (haveGlobbing) {
+                return ProjectFileArgumentPosition{filePathArgument,
+                                                   targetCMakeFile,
+                                                   fileName,
+                                                   true};
+            }
+
+            // Check if the filename is part of a variable set by the user
+            const auto setFunctions = std::get<0>(
+                Utils::partition(cmakeListFile.Functions, [](const auto &f) {
+                    return f.LowerCaseName() == "set" && f.Arguments().size() > 1;
+                }));
+
+            for (const auto &arg : func->Arguments()) {
+                auto matchedFunctions = Utils::filtered(setFunctions, [arg](const auto &f) {
+                    return arg.Value == std::string("${") + f.Arguments()[0].Value + "}";
+                });
+
+                for (const auto &f : matchedFunctions) {
+                    filePathArgument = Utils::findOrDefault(f.Arguments(),
+                                                            [file_name = fileName.toStdString()](
+                                                                const auto &arg) {
+                                                                return arg.Value == file_name;
+                                                            });
+
+                    if (!filePathArgument.Value.empty()) {
+                        return ProjectFileArgumentPosition{filePathArgument,
+                                                           targetCMakeFile,
+                                                           fileName};
+                    }
+                }
+            }
+        }
+    }
+
+    return std::nullopt;
+}
+
+RemovedFilesFromProject CMakeBuildSystem::removeFiles(Node *context,
+                                                      const FilePaths &filePaths,
+                                                      FilePaths *notRemoved)
+{
+    FilePaths badFiles;
+    if (auto n = dynamic_cast<CMakeTargetNode *>(context)) {
+        const FilePath projDir = n->filePath().canonicalPath();
+        const QString targetName = n->buildKey();
+
+        for (const auto &file : filePaths) {
+            const QString fileName
+                = file.canonicalPath().relativePathFrom(projDir).cleanPath().toString();
+
+            auto filePos = projectFileArgumentPosition(targetName, fileName);
+            if (filePos) {
+                if (!filePos.value().cmakeFile.exists()) {
+                    badFiles << file;
+                    continue;
+                }
+
+                BaseTextEditor *editor = qobject_cast<BaseTextEditor *>(
+                    Core::EditorManager::openEditorAt({filePos.value().cmakeFile,
+                                                       static_cast<int>(filePos.value().argumentPosition.Line),
+                                                       static_cast<int>(filePos.value().argumentPosition.Column
+                                                                        - 1)},
+                                                      Constants::CMAKE_EDITOR_ID,
+                                                      Core::EditorManager::DoNotMakeVisible));
+                if (!editor) {
+                    badFiles << file;
+                    continue;
+                }
+
+                // If quotes were used for the source file, remove the quotes too
+                int extraChars = 0;
+                if (filePos->argumentPosition.Delim == cmListFileArgument::Quoted)
+                    extraChars = 2;
+
+                if (!filePos.value().fromGlobbing)
+                    editor->replace(filePos.value().relativeFileName.length() + extraChars, "");
+
+                editor->editorWidget()->autoIndent();
+                if (!Core::DocumentManager::saveDocument(editor->document())) {
+                    badFiles << file;
+                    continue;
+                }
+            } else {
+                badFiles << file;
+            }
+        }
+
+        if (notRemoved && !badFiles.isEmpty())
+            *notRemoved = badFiles;
+
+        return badFiles.isEmpty() ? RemovedFilesFromProject::Ok : RemovedFilesFromProject::Error;
+    }
+
+    return RemovedFilesFromProject::Error;
+}
+
+bool CMakeBuildSystem::canRenameFile(Node *context,
+                                     const FilePath &oldFilePath,
+                                     const FilePath &newFilePath)
+{
+    // "canRenameFile" will cause an actual rename after the function call.
+    // This will make the a sequence like
+    //    canonicalPath().relativePathFrom(projDir).cleanPath().toString()
+    // to fail if the file doesn't exist on disk
+    // therefore cache the results for the subsequent "renameFile" call
+    // where oldFilePath has already been renamed as newFilePath.
+
+    if (auto n = dynamic_cast<CMakeTargetNode *>(context)) {
+        const FilePath projDir = n->filePath().canonicalPath();
+        const QString oldRelPathName
+            = oldFilePath.canonicalPath().relativePathFrom(projDir).cleanPath().toString();
+
+        const QString targetName = n->buildKey();
+
+        const QString key
+            = QStringList{projDir.path(), targetName, oldFilePath.path(), newFilePath.path()}
+                  .join(";");
+
+        auto filePos = projectFileArgumentPosition(targetName, oldRelPathName);
+        if (!filePos)
+            return false;
+
+        m_filesToBeRenamed.insert(key, filePos.value());
+        return true;
+    }
+    return false;
+}
+
+bool CMakeBuildSystem::renameFile(Node *context,
+                                  const FilePath &oldFilePath,
+                                  const FilePath &newFilePath)
+{
+    if (auto n = dynamic_cast<CMakeTargetNode *>(context)) {
+        const FilePath projDir = n->filePath().canonicalPath();
+        const QString newRelPathName
+            = newFilePath.canonicalPath().relativePathFrom(projDir).cleanPath().toString();
+
+        const QString targetName = n->buildKey();
+        const QString key
+            = QStringList{projDir.path(), targetName, oldFilePath.path(), newFilePath.path()}.join(
+                ";");
+
+        auto fileToRename = m_filesToBeRenamed.take(key);
+        if (!fileToRename.cmakeFile.exists())
+            return false;
+
+        BaseTextEditor *editor = qobject_cast<BaseTextEditor *>(
+            Core::EditorManager::openEditorAt({fileToRename.cmakeFile,
+                                               static_cast<int>(fileToRename.argumentPosition.Line),
+                                               static_cast<int>(fileToRename.argumentPosition.Column
+                                                                - 1)},
+                                              Constants::CMAKE_EDITOR_ID,
+                                              Core::EditorManager::DoNotMakeVisible));
+        if (!editor)
+            return false;
+
+        // If quotes were used for the source file, skip the starting quote
+        if (fileToRename.argumentPosition.Delim == cmListFileArgument::Quoted)
+            editor->setCursorPosition(editor->position() + 1);
+
+        if (!fileToRename.fromGlobbing)
+            editor->replace(fileToRename.relativeFileName.length(), newRelPathName);
+
+        editor->editorWidget()->autoIndent();
+        if (!Core::DocumentManager::saveDocument(editor->document()))
+            return false;
+
+        return true;
+    }
+
+    return false;
+}
+
 FilePaths CMakeBuildSystem::filesGeneratedFrom(const FilePath &sourceFile) const
 {
     FilePath project = projectDirectory();
@@ -297,9 +658,31 @@ FilePaths CMakeBuildSystem::filesGeneratedFrom(const FilePath &sourceFile) const
     FilePath generatedFilePath = buildConfiguration()->buildDirectory().resolvePath(relativePath);
 
     if (sourceFile.suffix() == "ui") {
-        generatedFilePath = generatedFilePath
-                                .pathAppended("ui_" + sourceFile.completeBaseName() + ".h");
-        return {generatedFilePath};
+        const QString generatedFileName = "ui_" + sourceFile.completeBaseName() + ".h";
+
+        auto targetNode = this->project()->nodeForFilePath(sourceFile);
+        while (targetNode && !dynamic_cast<const CMakeTargetNode *>(targetNode))
+            targetNode = targetNode->parentFolderNode();
+
+        FilePaths generatedFilePaths;
+        if (targetNode) {
+            const QString autogenSignature = targetNode->buildKey() + "_autogen/include";
+
+            // If AUTOUIC reports the generated header file name, use that path
+            generatedFilePaths = this->project()->files(
+                [autogenSignature, generatedFileName](const Node *n) {
+                    const FilePath filePath = n->filePath();
+                    if (!filePath.contains(autogenSignature))
+                        return false;
+
+                    return Project::GeneratedFiles(n) && filePath.endsWith(generatedFileName);
+                });
+        }
+
+        if (generatedFilePaths.empty())
+            generatedFilePaths = {generatedFilePath.pathAppended(generatedFileName)};
+
+        return generatedFilePaths;
     }
     if (sourceFile.suffix() == "scxml") {
         generatedFilePath = generatedFilePath.pathAppended(sourceFile.completeBaseName());
@@ -522,13 +905,22 @@ void CMakeBuildSystem::checkAndReportError(QString &errorMessage)
     }
 }
 
+static QSet<FilePath> projectFilesToWatch(const QSet<CMakeFileInfo> &cmakeFiles)
+{
+    return Utils::transform(Utils::filtered(cmakeFiles,
+                                            [](const CMakeFileInfo &info) {
+                                                return !info.isGenerated;
+                                            }),
+                            [](const CMakeFileInfo &info) { return info.path; });
+}
+
 void CMakeBuildSystem::updateProjectData()
 {
     qCDebug(cmakeBuildSystemLog) << "Updating CMake project data";
 
     QTC_ASSERT(m_treeScanner.isFinished() && !m_reader.isParsing(), return );
 
-    buildConfiguration()->project()->setExtraProjectFiles(m_reader.projectFilesToWatch());
+    buildConfiguration()->project()->setExtraProjectFiles(projectFilesToWatch(m_cmakeFiles));
 
     CMakeConfig patchedConfig = configurationFromCMake();
     {
@@ -601,7 +993,7 @@ void CMakeBuildSystem::updateProjectData()
     for (RawProjectPart &rpp : rpps) {
         rpp.setQtVersion(
                     kitInfo.projectPartQtVersion); // TODO: Check if project actually uses Qt.
-        const QString includeFileBaseDir = buildConfiguration()->buildDirectory().toString();
+        const FilePath includeFileBaseDir = buildConfiguration()->buildDirectory();
         QStringList cxxFlags = rpp.flagsForCxx.commandLineFlags;
         QStringList cFlags = rpp.flagsForC.commandLineFlags;
         addTargetFlagForIos(cxxFlags, cFlags, this, [this] {
@@ -735,6 +1127,8 @@ void CMakeBuildSystem::handleParsingSucceeded(bool restoredFromBackup)
             return result;
         });
         m_buildTargets += m_reader.takeBuildTargets(errorMessage);
+        m_cmakeFiles = m_reader.takeCMakeFileInfos(errorMessage);
+
         checkAndReportError(errorMessage);
     }
 
@@ -747,7 +1141,11 @@ void CMakeBuildSystem::handleParsingSucceeded(bool restoredFromBackup)
         m_ctestPath = tool->cmakeExecutable().withNewPath(m_reader.ctestPath());
 
     setApplicationTargets(appTargets());
-    setDeploymentData(deploymentData());
+
+    // Note: This is practically always wrong and resulting in an empty view.
+    // Setting the real data is triggered from a successful run of a
+    // MakeInstallStep.
+    setDeploymentData(deploymentDataFromFile());
 
     QTC_ASSERT(m_waitingForParse, return );
     m_waitingForParse = false;
@@ -901,11 +1299,11 @@ void CMakeBuildSystem::runCTest()
     QTC_ASSERT(parameters.isValid(), return);
 
     ensureBuildDirectory(parameters);
-    m_ctestProcess.reset(new QtcProcess);
+    m_ctestProcess.reset(new Process);
     m_ctestProcess->setEnvironment(buildConfiguration()->environment());
     m_ctestProcess->setWorkingDirectory(parameters.buildDirectory);
     m_ctestProcess->setCommand({m_ctestPath, { "-N", "--show-only=json-v1"}});
-    connect(m_ctestProcess.get(), &QtcProcess::done, this, [this] {
+    connect(m_ctestProcess.get(), &Process::done, this, [this] {
         if (m_ctestProcess->result() == ProcessResult::FinishedWithSuccess) {
             const QJsonDocument json
                 = QJsonDocument::fromJson(m_ctestProcess->readAllRawStandardOutput());
@@ -1064,7 +1462,7 @@ CommandLine CMakeBuildSystem::commandLineForTests(const QList<QString> &tests,
     return {m_ctestPath, args};
 }
 
-DeploymentData CMakeBuildSystem::deploymentData() const
+DeploymentData CMakeBuildSystem::deploymentDataFromFile() const
 {
     DeploymentData result;
 
@@ -1226,7 +1624,6 @@ void CMakeBuildSystem::updateInitialCMakeExpandableVars()
         "CMAKE_CXX_COMPILER",
         "QT_QMAKE_EXECUTABLE",
         "QT_HOST_PATH",
-        "CMAKE_PROJECT_INCLUDE_BEFORE",
         "CMAKE_TOOLCHAIN_FILE"
     };
     for (const auto &var : singlePathList) {
@@ -1299,7 +1696,7 @@ MakeInstallCommand CMakeBuildSystem::makeInstallCommand(const FilePath &installR
         buildDirectory = bc->buildDirectory();
 
     cmd.command.addArg("--build");
-    cmd.command.addArg(buildDirectory.onDevice(cmd.command.executable()).path());
+    cmd.command.addArg(buildDirectory.path());
     cmd.command.addArg("--target");
     cmd.command.addArg(installTarget);
 
@@ -1341,20 +1738,20 @@ void CMakeBuildSystem::runGenerator(Id id)
     const CMakeTool * const cmakeTool
             = CMakeKitAspect::cmakeTool(buildConfiguration()->target()->kit());
     if (!cmakeTool) {
-        showError(Tr::tr("Kit does not have a cmake binary set"));
+        showError(Tr::tr("Kit does not have a cmake binary set."));
         return;
     }
     const QString generator = id.toSetting().toString();
     const FilePath outDir = buildConfiguration()->buildDirectory()
             / ("qtc_" + FileUtils::fileSystemFriendlyName(generator));
     if (!outDir.ensureWritableDir()) {
-        showError(Tr::tr("Cannot create output directory \"%1\"").arg(outDir.toString()));
+        showError(Tr::tr("Cannot create output directory \"%1\".").arg(outDir.toString()));
         return;
     }
     CommandLine cmdLine(cmakeTool->cmakeExecutable(), {"-S", buildConfiguration()->target()
                         ->project()->projectDirectory().toUserOutput(), "-G", generator});
     if (!cmdLine.executable().isExecutableFile()) {
-        showError(Tr::tr("No valid cmake executable"));
+        showError(Tr::tr("No valid cmake executable."));
         return;
     }
     const auto itemFilter = [](const CMakeConfigItem &item) {
@@ -1380,19 +1777,19 @@ void CMakeBuildSystem::runGenerator(Id id)
             optionsAspect && !optionsAspect->value().isEmpty()) {
         cmdLine.addArgs(optionsAspect->value(), CommandLine::Raw);
     }
-    const auto proc = new QtcProcess(this);
-    connect(proc, &QtcProcess::done, proc, &QtcProcess::deleteLater);
-    connect(proc, &QtcProcess::readyReadStandardOutput, this, [proc] {
+    const auto proc = new Process(this);
+    connect(proc, &Process::done, proc, &Process::deleteLater);
+    connect(proc, &Process::readyReadStandardOutput, this, [proc] {
         Core::MessageManager::writeFlashing(QString::fromLocal8Bit(proc->readAllRawStandardOutput()));
     });
-    connect(proc, &QtcProcess::readyReadStandardError, this, [proc] {
+    connect(proc, &Process::readyReadStandardError, this, [proc] {
         Core::MessageManager::writeDisrupting(QString::fromLocal8Bit(proc->readAllRawStandardError()));
     });
     proc->setWorkingDirectory(outDir);
     proc->setEnvironment(buildConfiguration()->environment());
     proc->setCommand(cmdLine);
-    Core::MessageManager::writeFlashing(Tr::tr("Running in %1: %2")
-                                        .arg(outDir.toUserOutput(), cmdLine.toUserOutput()));
+    Core::MessageManager::writeFlashing(
+        Tr::tr("Running in %1: %2.").arg(outDir.toUserOutput(), cmdLine.toUserOutput()));
     proc->start();
 }
 
