@@ -1,48 +1,24 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "cmaketoolsettingsaccessor.h"
 
+#include "cmakeprojectmanagertr.h"
+#include "cmakespecificsettings.h"
 #include "cmaketool.h"
-#include "cmaketoolmanager.h"
 
 #include <coreplugin/icore.h>
 
 #include <app/app_version.h>
-#include <utils/environment.h>
 
 #include <utils/algorithm.h>
+#include <utils/environment.h>
 
 #include <QDebug>
-#include <QDir>
-#include <QFileInfo>
 
 using namespace Utils;
 
-namespace CMakeProjectManager {
-namespace Internal {
+namespace CMakeProjectManager::Internal {
 
 // --------------------------------------------------------------------
 // CMakeToolSettingsUpgraders:
@@ -69,48 +45,32 @@ const char CMAKE_TOOL_FILENAME[] = "cmaketools.xml";
 
 static std::vector<std::unique_ptr<CMakeTool>> autoDetectCMakeTools()
 {
-    Environment env = Environment::systemEnvironment();
-
-    FilePaths path = env.path();
-    path = Utils::filteredUnique(path);
+    FilePaths extraDirs;
 
     if (HostOsInfo::isWindowsHost()) {
-        for (auto envVar : {"ProgramFiles", "ProgramFiles(x86)", "ProgramW6432"}) {
-            if (qEnvironmentVariableIsSet(envVar)) {
-                const QString progFiles = qEnvironmentVariable(envVar);
-                path.append(FilePath::fromUserInput(progFiles + "/CMake"));
-                path.append(FilePath::fromUserInput(progFiles + "/CMake/bin"));
+        for (const auto &envVar : QStringList{"ProgramFiles", "ProgramFiles(x86)", "ProgramW6432"}) {
+            if (qtcEnvironmentVariableIsSet(envVar)) {
+                const QString progFiles = qtcEnvironmentVariable(envVar);
+                extraDirs.append(FilePath::fromUserInput(progFiles + "/CMake"));
+                extraDirs.append(FilePath::fromUserInput(progFiles + "/CMake/bin"));
             }
         }
     }
 
     if (HostOsInfo::isMacHost()) {
-        path.append("/Applications/CMake.app/Contents/bin");
-        path.append("/usr/local/bin");
-        path.append("/opt/local/bin");
+        extraDirs.append("/Applications/CMake.app/Contents/bin");
+        extraDirs.append("/usr/local/bin");    // homebrew intel
+        extraDirs.append("/opt/homebrew/bin"); // homebrew arm
+        extraDirs.append("/opt/local/bin");    // macports
     }
 
-    const QStringList execs = env.appendExeExtensions(QLatin1String("cmake"));
-
-    FilePaths suspects;
-    for (const FilePath &base : qAsConst(path)) {
-        if (base.isEmpty())
-            continue;
-
-        QFileInfo fi;
-        for (const QString &exec : execs) {
-            fi.setFile(QDir(base.toString()), exec);
-            if (fi.exists() && fi.isFile() && fi.isExecutable())
-                suspects << FilePath::fromString(fi.absoluteFilePath());
-        }
-    }
+    const FilePaths suspects = FilePath("cmake").searchAllInPath(extraDirs);
 
     std::vector<std::unique_ptr<CMakeTool>> found;
-    for (const FilePath &command : qAsConst(suspects)) {
+    for (const FilePath &command : std::as_const(suspects)) {
         auto item = std::make_unique<CMakeTool>(CMakeTool::AutoDetection, CMakeTool::createId());
         item->setFilePath(command);
-        item->setDisplayName(::CMakeProjectManager::CMakeToolManager::tr("System CMake at %1")
-                                 .arg(command.toUserOutput()));
+        item->setDisplayName(Tr::tr("System CMake at %1").arg(command.toUserOutput()));
 
         found.emplace_back(std::move(item));
     }
@@ -169,11 +129,10 @@ mergeTools(std::vector<std::unique_ptr<CMakeTool>> &sdkTools,
 // CMakeToolSettingsAccessor:
 // --------------------------------------------------------------------
 
-CMakeToolSettingsAccessor::CMakeToolSettingsAccessor() :
-    UpgradingSettingsAccessor("QtCreatorCMakeTools",
-                              QCoreApplication::translate("CMakeProjectManager::CMakeToolManager", "CMake"),
-                              Core::Constants::IDE_DISPLAY_NAME)
+CMakeToolSettingsAccessor::CMakeToolSettingsAccessor()
 {
+    setDocType("QtCreatorCMakeTools");
+    setApplicationDisplayName(Core::Constants::IDE_DISPLAY_NAME);
     setBaseFilePath(Core::ICore::userResourcePath(CMAKE_TOOL_FILENAME));
 
     addVersionUpgrader(std::make_unique<CMakeToolSettingsUpgraderV0>());
@@ -216,9 +175,14 @@ void CMakeToolSettingsAccessor::saveCMakeTools(const QList<CMakeTool *> &cmakeTo
     data.insert(QLatin1String(CMAKE_TOOL_DEFAULT_KEY), defaultId.toSetting());
 
     int count = 0;
-    for (const CMakeTool *item : cmakeTools) {
+    for (CMakeTool *item : cmakeTools) {
         Utils::FilePath fi = item->cmakeExecutable();
 
+        // Gobal Autorun value will be set for all tools
+        // TODO: Remove in Qt Creator 13
+        const auto settings = CMakeSpecificSettings::instance();
+        item->setAutorun(settings->autorunCMake.value());
+
         if (fi.needsDevice() || fi.isExecutableFile()) { // be graceful for device related stuff
             QVariantMap tmp = item->toMap();
             if (tmp.isEmpty())
@@ -260,5 +224,4 @@ CMakeToolSettingsAccessor::cmakeTools(const QVariantMap &data, bool fromSdk) con
     return result;
 }
 
-} // namespace Internal
-} // namespace CMakeProjectManager
+} // CMakeProjectManager::Internal
