@@ -1,30 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "fileapiparser.h"
 
+#include "cmakeprojectmanagertr.h"
+
 #include <app/app_version.h>
 #include <coreplugin/messagemanager.h>
 #include <projectexplorer/rawprojectpart.h>
@@ -36,12 +16,13 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
+#include <QPromise>
 
-namespace CMakeProjectManager {
-namespace Internal {
+using namespace Utils;
+
+namespace CMakeProjectManager::Internal {
 
 using namespace FileApiDetails;
-using namespace Utils;
 
 const char CMAKE_RELATIVE_REPLY_PATH[] = ".cmake/api/v1/reply";
 const char CMAKE_RELATIVE_QUERY_PATH[] = ".cmake/api/v1/query";
@@ -54,7 +35,7 @@ const QStringList CMAKE_QUERY_FILENAMES = {"cache-v2", "codemodel-v2", "cmakeFil
 // Helper:
 // --------------------------------------------------------------------
 
-static FilePath cmakeReplyDirectory(const FilePath &buildDirectory)
+FilePath FileApiParser::cmakeReplyDirectory(const FilePath &buildDirectory)
 {
     return buildDirectory.pathAppended(CMAKE_RELATIVE_REPLY_PATH);
 }
@@ -62,9 +43,8 @@ static FilePath cmakeReplyDirectory(const FilePath &buildDirectory)
 static void reportFileApiSetupFailure()
 {
     Core::MessageManager::writeFlashing(
-        QCoreApplication::translate("CMakeProjectManager::Internal",
-                                    "Failed to set up CMake file API support. %1 cannot "
-                                    "extract project information.")
+        Tr::tr("Failed to set up CMake file API support. %1 cannot "
+               "extract project information.")
             .arg(Core::Constants::IDE_DISPLAY_NAME));
 }
 
@@ -73,7 +53,7 @@ static std::pair<int, int> cmakeVersion(const QJsonObject &obj)
     const QJsonObject version = obj.value("version").toObject();
     const int major = version.value("major").toInt(-1);
     const int minor = version.value("minor").toInt(-1);
-    return std::make_pair(major, minor);
+    return {major, minor};
 }
 
 static bool checkJsonObject(const QJsonObject &obj, const QString &kind, int major, int minor = -1)
@@ -88,7 +68,7 @@ static bool checkJsonObject(const QJsonObject &obj, const QString &kind, int maj
 
 static std::pair<QString, QString> nameValue(const QJsonObject &obj)
 {
-    return std::make_pair(obj.value("name").toString(), obj.value("value").toString());
+    return {obj.value("name").toString(), obj.value("value").toString()};
 }
 
 static QJsonDocument readJsonFile(const FilePath &filePath)
@@ -96,11 +76,10 @@ static QJsonDocument readJsonFile(const FilePath &filePath)
     qCDebug(cmakeFileApi) << "readJsonFile:" << filePath;
     QTC_ASSERT(!filePath.isEmpty(), return {});
 
-    QTC_CHECK(!filePath.needsDevice());
-
-    QFile file(filePath.path());
-    file.open(QIODevice::ReadOnly | QIODevice::Text);
-    const QJsonDocument doc = QJsonDocument::fromJson(file.readAll());
+    const expected_str<QByteArray> contents = filePath.fileContents();
+    if (!contents)
+        return {};
+    const QJsonDocument doc = QJsonDocument::fromJson(*contents);
 
     return doc;
 }
@@ -122,8 +101,7 @@ std::vector<int> indexList(const QJsonValue &v)
 static ReplyFileContents readReplyFile(const FilePath &filePath, QString &errorMessage)
 {
     const QJsonDocument document = readJsonFile(filePath);
-    static const QString msg = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                                           "Invalid reply file created by CMake.");
+    static const QString msg = Tr::tr("Invalid reply file created by CMake.");
 
     ReplyFileContents result;
     if (document.isNull() || document.isEmpty() || !document.isObject()) {
@@ -194,8 +172,7 @@ static CMakeConfig readCacheFile(const FilePath &cacheFile, QString &errorMessag
     const QJsonObject root = doc.object();
 
     if (!checkJsonObject(root, "cache", 2)) {
-        errorMessage = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                                   "Invalid cache file generated by CMake.");
+        errorMessage = Tr::tr("Invalid cache file generated by CMake.");
         return {};
     }
 
@@ -240,8 +217,7 @@ static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesF
     const QJsonObject root = doc.object();
 
     if (!checkJsonObject(root, "cmakeFiles", 1)) {
-        errorMessage = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                                   "Invalid cmakeFiles file generated by CMake.");
+        errorMessage = Tr::tr( "Invalid cmakeFiles file generated by CMake.");
         return {};
     }
 
@@ -249,7 +225,7 @@ static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesF
     for (const QJsonValue &v : inputs) {
         CMakeFileInfo info;
         const QJsonObject input = v.toObject();
-        info.path = FilePath::fromString(input.value("path").toString());
+        info.path = cmakeFilesFile.withNewPath(input.value("path").toString());
 
         info.isCMake = input.value("isCMake").toBool();
         const QString filename = info.path.fileName();
@@ -270,8 +246,7 @@ static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesF
 std::vector<Directory> extractDirectories(const QJsonArray &directories, QString &errorMessage)
 {
     if (directories.isEmpty()) {
-        errorMessage = QCoreApplication::translate(
-            "CMakeProjectManager::Internal",
+        errorMessage = Tr::tr(
             "Invalid codemodel file generated by CMake: No directories.");
         return {};
     }
@@ -280,8 +255,7 @@ std::vector<Directory> extractDirectories(const QJsonArray &directories, QString
     for (const QJsonValue &v : directories) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty directory object.");
             continue;
         }
@@ -302,8 +276,7 @@ std::vector<Directory> extractDirectories(const QJsonArray &directories, QString
 static std::vector<Project> extractProjects(const QJsonArray &projects, QString &errorMessage)
 {
     if (projects.isEmpty()) {
-        errorMessage = QCoreApplication::translate(
-            "CMakeProjectManager::Internal",
+        errorMessage = Tr::tr(
             "Invalid codemodel file generated by CMake: No projects.");
         return {};
     }
@@ -313,8 +286,7 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
             qCDebug(cmakeFileApi) << "Empty project skipped!";
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty project object.");
             continue;
         }
@@ -327,8 +299,7 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
 
         if (project.directories.empty()) {
             qCDebug(cmakeFileApi) << "Invalid project skipped!";
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Broken project data.");
             continue;
         }
@@ -345,8 +316,7 @@ static std::vector<Target> extractTargets(const QJsonArray &targets, QString &er
     for (const QJsonValue &v : targets) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty target object.");
             continue;
         }
@@ -359,8 +329,7 @@ static std::vector<Target> extractTargets(const QJsonArray &targets, QString &er
 
         if (target.name.isEmpty() || target.id.isEmpty() || target.jsonFile.isEmpty()
             || target.directory == -1 || target.project == -1) {
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Broken target data.");
             continue;
         }
@@ -460,8 +429,7 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
                                                         QString &errorMessage)
 {
     if (configs.isEmpty()) {
-        errorMessage = QCoreApplication::translate(
-            "CMakeProjectManager::Internal",
+        errorMessage = Tr::tr(
             "Invalid codemodel file generated by CMake: No configurations.");
         return {};
     }
@@ -470,8 +438,7 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
     for (const QJsonValue &v : configs) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
-            errorMessage = QCoreApplication::translate(
-                "CMakeProjectManager::Internal",
+            errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty configuration object.");
             continue;
         }
@@ -483,10 +450,8 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
         config.targets = extractTargets(obj.value("targets").toArray(), errorMessage);
 
         if (!validateIndexes(config)) {
-            errorMessage
-                = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                              "Invalid codemodel file generated by CMake: Broken "
-                                              "indexes in directories, projects, or targets.");
+            errorMessage = Tr::tr("Invalid codemodel file generated by CMake: Broken "
+                                  "indexes in directories, projects, or targets.");
             return {};
         }
 
@@ -502,8 +467,7 @@ static std::vector<Configuration> readCodemodelFile(const FilePath &codemodelFil
     const QJsonObject root = doc.object();
 
     if (!checkJsonObject(root, "codemodel", 2)) {
-        errorMessage = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                                   "Invalid codemodel file generated by CMake.");
+        errorMessage = Tr::tr("Invalid codemodel file generated by CMake.");
         return {};
     }
 
@@ -522,6 +486,26 @@ static std::vector<FileApiDetails::FragmentInfo> extractFragments(const QJsonObj
     });
 }
 
+static void addIncludeInfo(std::vector<IncludeInfo> *includes,
+                           const QJsonObject &compileGroups,
+                           const QString &section)
+{
+    const std::vector<IncludeInfo> add
+        = transform<std::vector>(compileGroups.value(section).toArray(), [](const QJsonValue &v) {
+              const QJsonObject i = v.toObject();
+              const QString path = i.value("path").toString();
+              const bool isSystem = i.value("isSystem").toBool();
+              const ProjectExplorer::HeaderPath hp(path,
+                                                   isSystem
+                                                       ? ProjectExplorer::HeaderPathType::System
+                                                       : ProjectExplorer::HeaderPathType::User);
+
+              return IncludeInfo{ProjectExplorer::RawProjectPart::frameworkDetectionHeuristic(hp),
+                                 i.value("backtrace").toInt(-1)};
+          });
+    std::copy(add.cbegin(), add.cend(), std::back_inserter(*includes));
+}
+
 static TargetDetails extractTargetDetails(const QJsonObject &root, QString &errorMessage)
 {
     TargetDetails t;
@@ -530,8 +514,7 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
     t.type = root.value("type").toString();
 
     if (t.name.isEmpty() || t.id.isEmpty() || t.type.isEmpty()) {
-        errorMessage = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                                   "Invalid target file: Information is missing.");
+        errorMessage = Tr::tr("Invalid target file: Information is missing.");
         return {};
     }
 
@@ -618,6 +601,10 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
         const QJsonArray compileGroups = root.value("compileGroups").toArray();
         t.compileGroups = transform<std::vector>(compileGroups, [](const QJsonValue &v) {
             const QJsonObject o = v.toObject();
+            std::vector<IncludeInfo> includes;
+            addIncludeInfo(&includes, o, "includes");
+            // new in CMake 3.27+:
+            addIncludeInfo(&includes, o, "frameworks");
             return CompileInfo{
                 transform<std::vector>(o.value("sourceIndexes").toArray(),
                                        [](const QJsonValue &v) { return v.toInt(-1); }),
@@ -627,21 +614,7 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
                                      const QJsonObject o = v.toObject();
                                      return o.value("fragment").toString();
                                  }),
-                transform<std::vector>(
-                    o.value("includes").toArray(),
-                    [](const QJsonValue &v) {
-                        const QJsonObject i = v.toObject();
-                        const QString path = i.value("path").toString();
-                        const bool isSystem = i.value("isSystem").toBool();
-                        const ProjectExplorer::HeaderPath
-                            hp(path,
-                               isSystem ? ProjectExplorer::HeaderPathType::System
-                                        : ProjectExplorer::HeaderPathType::User);
-
-                        return IncludeInfo{
-                            ProjectExplorer::RawProjectPart::frameworkDetectionHeuristic(hp),
-                            i.value("backtrace").toInt(-1)};
-                    }),
+                includes,
                 transform<std::vector>(o.value("defines").toArray(),
                                        [](const QJsonValue &v) {
                                            const QJsonObject d = v.toObject();
@@ -799,8 +772,7 @@ static TargetDetails readTargetFile(const FilePath &targetFile, QString &errorMe
 
     TargetDetails result = extractTargetDetails(root, errorMessage);
     if (errorMessage.isEmpty() && !validateTargetDetails(result)) {
-        errorMessage = QCoreApplication::translate(
-            "CMakeProjectManager::Internal",
+        errorMessage = Tr::tr(
             "Invalid target file generated by CMake: Broken indexes in target details.");
     }
     return result;
@@ -823,7 +795,7 @@ FilePath FileApiDetails::ReplyFileContents::jsonFile(const QString &kind, const
 // FileApi:
 // --------------------------------------------------------------------
 
-bool FileApiParser::setupCMakeFileApi(const FilePath &buildDirectory, Utils::FileSystemWatcher &watcher)
+bool FileApiParser::setupCMakeFileApi(const FilePath &buildDirectory)
 {
     // So that we have a directory to watch.
     buildDirectory.pathAppended(CMAKE_RELATIVE_REPLY_PATH).ensureWritableDir();
@@ -846,7 +818,6 @@ bool FileApiParser::setupCMakeFileApi(const FilePath &buildDirectory, Utils::Fil
         }
     }
 
-    watcher.addDirectory(cmakeReplyDirectory(buildDirectory).path(), FileSystemWatcher::WatchAllChanges);
     return true;
 }
 
@@ -864,20 +835,19 @@ static QStringList uniqueTargetFiles(const Configuration &config)
     return files;
 }
 
-FileApiData FileApiParser::parseData(QFutureInterface<std::shared_ptr<FileApiQtcData>> &fi,
+FileApiData FileApiParser::parseData(QPromise<std::shared_ptr<FileApiQtcData>> &promise,
                                      const FilePath &replyFilePath,
                                      const QString &cmakeBuildType,
                                      QString &errorMessage)
 {
     QTC_CHECK(errorMessage.isEmpty());
-    QTC_CHECK(!replyFilePath.needsDevice());
     const FilePath replyDir = replyFilePath.parentDir();
 
     FileApiData result;
 
-    const auto cancelCheck = [&fi, &errorMessage]() -> bool {
-        if (fi.isCanceled()) {
-            errorMessage = FileApiParser::tr("CMake parsing was cancelled.");
+    const auto cancelCheck = [&promise, &errorMessage] {
+        if (promise.isCanceled()) {
+            errorMessage = Tr::tr("CMake parsing was canceled.");
             return true;
         }
         return false;
@@ -897,7 +867,9 @@ FileApiData FileApiParser::parseData(QFutureInterface<std::shared_ptr<FileApiQtc
                                          errorMessage);
 
     if (codeModels.size() == 0) {
-        errorMessage = "No CMake configuration found!";
+        errorMessage = Tr::tr("CMake project configuration failed. No CMake configuration for "
+                              "build type \"%1\" found.")
+                           .arg(cmakeBuildType);
         return result;
     }
 
@@ -911,12 +883,12 @@ FileApiData FileApiParser::parseData(QFutureInterface<std::shared_ptr<FileApiQtc
             buildTypes << cfg.name;
 
         if (result.replyFile.isMultiConfig) {
-            errorMessage = tr("No \"%1\" CMake configuration found. Available configurations: \"%2\".\n"
+            errorMessage = Tr::tr("No \"%1\" CMake configuration found. Available configurations: \"%2\".\n"
                               "Make sure that CMAKE_CONFIGURATION_TYPES variable contains the \"Build type\" field.")
                            .arg(cmakeBuildType)
                            .arg(buildTypes.join(", "));
         } else {
-            errorMessage = tr("No \"%1\" CMake configuration found. Available configuration: \"%2\".\n"
+            errorMessage = Tr::tr("No \"%1\" CMake configuration found. Available configuration: \"%2\".\n"
                               "Make sure that CMAKE_BUILD_TYPE variable matches the \"Build type\" field.")
                            .arg(cmakeBuildType)
                            .arg(buildTypes.join(", "));
@@ -952,17 +924,16 @@ FilePath FileApiParser::scanForCMakeReplyFile(const FilePath &buildDirectory)
     if (!replyDir.exists())
         return {};
 
-    const FilePaths entries = replyDir.dirEntries({"index-*.json"}, QDir::Files, QDir::Name);
+    const FilePaths entries = replyDir.dirEntries({{"index-*.json"}, QDir::Files}, QDir::Name);
     return entries.isEmpty() ? FilePath() : entries.first();
 }
 
 FilePaths FileApiParser::cmakeQueryFilePaths(const FilePath &buildDirectory)
 {
-    FilePath queryDir = buildDirectory / CMAKE_RELATIVE_QUERY_PATH;
+    const FilePath queryDir = buildDirectory / CMAKE_RELATIVE_QUERY_PATH;
     return transform(CMAKE_QUERY_FILENAMES, [&queryDir](const QString &name) {
-        return queryDir.absoluteFilePath(FilePath::fromString(name));
+        return queryDir.resolvePath(name);
     });
 }
 
-} // namespace Internal
-} // namespace CMakeProjectManager
+} // CMakeProjectManager::Internal
