@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "editorview.h"
 
@@ -31,15 +9,18 @@
 #include "documentmodel_p.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editortoolbar.h>
+#include <coreplugin/findplaceholder.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/infobar.h>
 #include <coreplugin/locator/locatorconstants.h>
 #include <coreplugin/minisplitter.h>
-#include <coreplugin/editormanager/ieditor.h>
-#include <coreplugin/findplaceholder.h>
+#include <utils/algorithm.h>
+#include <utils/infobar.h>
 #include <utils/qtcassert.h>
 #include <utils/theme/theme.h>
+#include <utils/layoutbuilder.h>
+#include <utils/link.h>
 #include <utils/utilsicons.h>
 
 #include <QDebug>
@@ -67,7 +48,7 @@ EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
     m_toolBar(new EditorToolBar(this)),
     m_container(new QStackedWidget(this)),
     m_infoBarDisplay(new InfoBarDisplay(this)),
-    m_statusHLine(new QFrame(this)),
+    m_statusHLine(Layouting::createHr(this)),
     m_statusWidget(new QFrame(this))
 {
     auto tl = new QVBoxLayout(this);
@@ -99,8 +80,6 @@ EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
     tl->addWidget(new FindToolBarPlaceHolder(this));
 
     {
-        m_statusHLine->setFrameStyle(QFrame::HLine);
-
         m_statusWidget->setFrameStyle(QFrame::NoFrame);
         m_statusWidget->setLineWidth(0);
         m_statusWidget->setAutoFillBackground(true);
@@ -135,14 +114,14 @@ EditorView::EditorView(SplitterOrView *parentSplitterOrView, QWidget *parent) :
     m_container->addWidget(empty);
     m_widgetEditorMap.insert(empty, nullptr);
 
-    auto dropSupport = new DropSupport(this, [this](QDropEvent *event, DropSupport *dropSupport) -> bool {
+    const auto dropSupport = new DropSupport(this, [this](QDropEvent *event, DropSupport *) {
         // do not accept move events except from other editor views (i.e. their tool bars)
         // otherwise e.g. item views that support moving items within themselves would
         // also "move" the item into the editor view, i.e. the item would be removed from the
         // item view
         if (!qobject_cast<EditorToolBar*>(event->source()))
             event->setDropAction(Qt::CopyAction);
-        if (event->type() == QDropEvent::DragEnter && !dropSupport->isFileDrop(event))
+        if (event->type() == QDropEvent::DragEnter && !DropSupport::isFileDrop(event))
             return false; // do not accept drops without files
         return event->source() != m_toolBar; // do not accept drops on ourselves
     });
@@ -159,7 +138,7 @@ SplitterOrView *EditorView::parentSplitterOrView() const
     return m_parentSplitterOrView;
 }
 
-EditorView *EditorView::findNextView()
+EditorView *EditorView::findNextView() const
 {
     SplitterOrView *current = parentSplitterOrView();
     QTC_ASSERT(current, return nullptr);
@@ -182,7 +161,7 @@ EditorView *EditorView::findNextView()
     return nullptr;
 }
 
-EditorView *EditorView::findPreviousView()
+EditorView *EditorView::findPreviousView() const
 {
     SplitterOrView *current = parentSplitterOrView();
     QTC_ASSERT(current, return nullptr);
@@ -260,15 +239,14 @@ void EditorView::updateEditorHistory(IEditor *editor, QList<EditLocation> &histo
 
     EditLocation location;
     location.document = document;
-    location.fileName = document->filePath().toString();
+    location.filePath = document->filePath();
     location.id = document->id();
     location.state = QVariant(state);
 
     for (int i = 0; i < history.size(); ++i) {
         const EditLocation &item = history.at(i);
         if (item.document == document
-                || (!item.document
-                    && !DocumentModel::indexOfFilePath(FilePath::fromString(item.fileName)))) {
+                || (!item.document && !DocumentModel::indexOfFilePath(item.filePath))) {
             history.removeAt(i--);
         }
     }
@@ -363,7 +341,7 @@ void EditorView::listSelectionActivated(int index)
     EditorManagerPrivate::activateEditorForEntry(this, DocumentModel::entryAtRow(index));
 }
 
-void EditorView::fillListContextMenu(QMenu *menu)
+void EditorView::fillListContextMenu(QMenu *menu) const
 {
     IEditor *editor = currentEditor();
     DocumentModel::Entry *entry = editor ? DocumentModel::entryForDocument(editor->document())
@@ -400,14 +378,26 @@ void EditorView::closeSplit()
 
 void EditorView::openDroppedFiles(const QList<DropSupport::FileSpec> &files)
 {
-    const int count = files.size();
-    for (int i = 0; i < count; ++i) {
-        const DropSupport::FileSpec spec = files.at(i);
-        EditorManagerPrivate::openEditorAt(this, spec.filePath, spec.line, spec.column, Id(),
-                                  i < count - 1 ? EditorManager::DoNotChangeCurrentEditor
-                                                  | EditorManager::DoNotMakeVisible
-                                                : EditorManager::NoFlags);
-    }
+    bool first = true;
+    auto specToLink = [](const DropSupport::FileSpec &spec) {
+        return Utils::Link(spec.filePath, spec.line, spec.column);
+    };
+    auto openEntry = [&](const DropSupport::FileSpec &spec) {
+        if (first) {
+            first = !EditorManagerPrivate::openEditorAt(this, specToLink(spec));
+        } else if (spec.column != -1 || spec.line != -1) {
+            EditorManagerPrivate::openEditorAt(this,
+                                               specToLink(spec),
+                                               Id(),
+                                               EditorManager::DoNotChangeCurrentEditor
+                                                   | EditorManager::DoNotMakeVisible);
+        } else {
+            auto factory = IEditorFactory::preferredEditorFactories(spec.filePath).value(0);
+            DocumentModelPrivate::addSuspendedDocument(spec.filePath, {},
+                                                       factory ? factory->id() : Id());
+        }
+    };
+    Utils::reverseForeach(files, openEntry);
 }
 
 void EditorView::setParentSplitterOrView(SplitterOrView *splitterOrView)
@@ -452,10 +442,7 @@ QList<IEditor *> EditorView::editors() const
 
 IEditor *EditorView::editorForDocument(const IDocument *document) const
 {
-    foreach (IEditor *editor, m_editors)
-        if (editor->document() == document)
-            return editor;
-    return nullptr;
+    return Utils::findOrDefault(m_editors, Utils::equal(&IEditor::document, document));
 }
 
 void EditorView::updateEditorHistory(IEditor *editor)
@@ -463,6 +450,8 @@ void EditorView::updateEditorHistory(IEditor *editor)
     updateEditorHistory(editor, m_editorHistory);
 }
 
+constexpr int navigationHistorySize = 100;
+
 void EditorView::addCurrentPositionToNavigationHistory(const QByteArray &saveState)
 {
     IEditor *editor = currentEditor();
@@ -481,15 +470,15 @@ void EditorView::addCurrentPositionToNavigationHistory(const QByteArray &saveSta
 
     EditLocation location;
     location.document = document;
-    location.fileName = document->filePath().toString();
+    location.filePath = document->filePath();
     location.id = document->id();
     location.state = QVariant(state);
     m_currentNavigationHistoryPosition = qMin(m_currentNavigationHistoryPosition, m_navigationHistory.size()); // paranoia
     m_navigationHistory.insert(m_currentNavigationHistoryPosition, location);
     ++m_currentNavigationHistoryPosition;
 
-    while (m_navigationHistory.size() >= 30) {
-        if (m_currentNavigationHistoryPosition > 15) {
+    while (m_navigationHistory.size() >= navigationHistorySize) {
+        if (m_currentNavigationHistoryPosition > navigationHistorySize / 2) {
             m_navigationHistory.removeFirst();
             --m_currentNavigationHistoryPosition;
         } else {
@@ -536,17 +525,15 @@ void EditorView::updateCurrentPositionInNavigationHistory()
         location = &m_navigationHistory[m_navigationHistory.size()-1];
     }
     location->document = document;
-    location->fileName = document->filePath().toString();
+    location->filePath = document->filePath();
     location->id = document->id();
     location->state = QVariant(editor->saveState());
 }
 
-namespace {
-static inline bool fileNameWasRemoved(const QString &fileName)
+static bool fileNameWasRemoved(const FilePath &filePath)
 {
-    return !fileName.isEmpty() && !QFileInfo::exists(fileName);
+    return !filePath.isEmpty() && !filePath.exists();
 }
-} // End of anonymous namespace
 
 void EditorView::goBackInNavigationHistory()
 {
@@ -560,11 +547,11 @@ void EditorView::goBackInNavigationHistory()
                                         EditorManager::IgnoreNavigationHistory);
         }
         if (!editor) {
-            if (fileNameWasRemoved(location.fileName)) {
+            if (fileNameWasRemoved(location.filePath)) {
                 m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
                 continue;
             }
-            editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
+            editor = EditorManagerPrivate::openEditor(this, location.filePath, location.id,
                                     EditorManager::IgnoreNavigationHistory);
             if (!editor) {
                 m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
@@ -591,11 +578,11 @@ void EditorView::goForwardInNavigationHistory()
                                                                      EditorManager::IgnoreNavigationHistory);
         }
         if (!editor) {
-            if (fileNameWasRemoved(location.fileName)) {
+            if (fileNameWasRemoved(location.filePath)) {
                 m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
                 continue;
             }
-            editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
+            editor = EditorManagerPrivate::openEditor(this, location.filePath, location.id,
                                                       EditorManager::IgnoreNavigationHistory);
             if (!editor) {
                 m_navigationHistory.removeAt(m_currentNavigationHistoryPosition);
@@ -620,10 +607,10 @@ void EditorView::goToEditLocation(const EditLocation &location)
     }
 
     if (!editor) {
-        if (fileNameWasRemoved(location.fileName))
+        if (fileNameWasRemoved(location.filePath))
             return;
 
-        editor = EditorManagerPrivate::openEditor(this, location.fileName, location.id,
+        editor = EditorManagerPrivate::openEditor(this, location.filePath, location.id,
                                                   EditorManager::IgnoreNavigationHistory);
     }
 
@@ -667,7 +654,7 @@ SplitterOrView::~SplitterOrView()
     m_splitter = nullptr;
 }
 
-EditorView *SplitterOrView::findFirstView()
+EditorView *SplitterOrView::findFirstView() const
 {
     if (m_splitter) {
         for (int i = 0; i < m_splitter->count(); ++i) {
@@ -680,7 +667,7 @@ EditorView *SplitterOrView::findFirstView()
     return m_view;
 }
 
-EditorView *SplitterOrView::findLastView()
+EditorView *SplitterOrView::findLastView() const
 {
     if (m_splitter) {
         for (int i = m_splitter->count() - 1; 0 < i; --i) {
@@ -907,13 +894,10 @@ QByteArray SplitterOrView::saveState() const
         // don't save state of temporary or ad-hoc editors
         if (e && (e->document()->isTemporary() || e->document()->filePath().isEmpty())) {
             // look for another editor that is more suited
-            e = nullptr;
-            foreach (IEditor *otherEditor, editors()) {
-                if (!otherEditor->document()->isTemporary() && !otherEditor->document()->filePath().isEmpty()) {
-                    e = otherEditor;
-                    break;
-                }
-            }
+            e = Utils::findOrDefault(editors(), [](IEditor *otherEditor) {
+                return !otherEditor->document()->isTemporary()
+                       && !otherEditor->document()->filePath().isEmpty();
+            });
         }
 
         if (!e) {
@@ -953,7 +937,7 @@ void SplitterOrView::restoreState(const QByteArray &state)
         stream >> fileName >> id >> editorState;
         if (!QFile::exists(fileName))
             return;
-        IEditor *e = EditorManagerPrivate::openEditor(view(), fileName, Id::fromString(id),
+        IEditor *e = EditorManagerPrivate::openEditor(view(), FilePath::fromString(fileName), Id::fromString(id),
                                                       EditorManager::IgnoreNavigationHistory
                                                       | EditorManager::DoNotChangeCurrentEditor);
 
