@@ -1,40 +1,24 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "icore.h"
 
-#include "windowsupport.h"
+#include "coreplugintr.h"
 #include "dialogs/settingsdialog.h"
+#include "windowsupport.h"
 
 #include <app/app_version.h>
+
 #include <extensionsystem/pluginmanager.h>
 
+#include <utils/algorithm.h>
+#include <utils/environment.h>
+#include <utils/fileutils.h>
 #include <utils/qtcassert.h>
 
 #include <QApplication>
 #include <QDebug>
+#include <QLibraryInfo>
 #include <QStandardPaths>
 #include <QSysInfo>
 
@@ -141,6 +125,7 @@
     from the focus object as well as the additional context.
 */
 
+#include "dialogs/newdialogwidget.h"
 #include "dialogs/newdialog.h"
 #include "iwizardfactory.h"
 #include "mainwindow.h"
@@ -165,6 +150,13 @@ namespace Core {
 static ICore *m_instance = nullptr;
 static MainWindow *m_mainwindow = nullptr;
 
+static NewDialog *defaultDialogFactory(QWidget *parent)
+{
+    return new NewDialogWidget(parent);
+}
+
+static std::function<NewDialog *(QWidget *)> m_newDialogFactory = defaultDialogFactory;
+
 /*!
     Returns the pointer to the instance. Only use for connecting to signals.
 */
@@ -204,12 +196,20 @@ ICore::ICore(MainWindow *mainwindow)
     // Save settings once after all plugins are initialized:
     connect(PluginManager::instance(), &PluginManager::initializationDone,
             this, [] { ICore::saveSettings(ICore::InitializationDone); });
-    connect(PluginManager::instance(), &PluginManager::testsFinished, [this] (int failedTests) {
+    connect(PluginManager::instance(), &PluginManager::testsFinished,
+            this, [this](int failedTests) {
         emit coreAboutToClose();
         if (failedTests != 0)
             qWarning("Test run was not successful: %d test(s) failed.", failedTests);
         QCoreApplication::exit(failedTests);
     });
+    connect(PluginManager::instance(), &PluginManager::scenarioFinished,
+            this, [this](int exitCode) {
+        emit coreAboutToClose();
+        QCoreApplication::exit(exitCode);
+    });
+
+    FileUtils::setDialogParentGetter(&ICore::dialogParent);
 }
 
 /*!
@@ -239,12 +239,28 @@ ICore::~ICore()
 */
 void ICore::showNewItemDialog(const QString &title,
                               const QList<IWizardFactory *> &factories,
-                              const QString &defaultLocation,
+                              const FilePath &defaultLocation,
                               const QVariantMap &extraVariables)
 {
     QTC_ASSERT(!isNewItemDialogRunning(), return);
-    auto newDialog = new NewDialog(dialogParent());
-    connect(newDialog, &QObject::destroyed, m_instance, &ICore::updateNewItemDialogState);
+
+    /* This is a workaround for QDS: In QDS, we currently have a "New Project" dialog box but we do
+     * not also have a "New file" dialog box (yet). Therefore, when requested to add a new file, we
+     * need to use QtCreator's dialog box. In QDS, if `factories` contains project wizard factories
+     * (even though it may contain file wizard factories as well), then we consider it to be a
+     * request for "New Project". Otherwise, if we only have file wizard factories, we defer to
+     * QtCreator's dialog and request "New File"
+     */
+    auto dialogFactory = m_newDialogFactory;
+    bool haveProjectWizards = Utils::anyOf(factories, [](IWizardFactory *f) {
+        return f->kind() == IWizardFactory::ProjectWizard;
+    });
+
+    if (!haveProjectWizards)
+        dialogFactory = defaultDialogFactory;
+
+    NewDialog *newDialog = dialogFactory(dialogParent());
+    connect(newDialog->widget(), &QObject::destroyed, m_instance, &ICore::updateNewItemDialogState);
     newDialog->setWizardFactories(factories, defaultLocation, extraVariables);
     newDialog->setWindowTitle(title);
     newDialog->showDialog();
@@ -275,7 +291,7 @@ bool ICore::showOptionsDialog(const Id page, QWidget *parent)
 */
 QString ICore::msgShowOptionsDialog()
 {
-    return QCoreApplication::translate("Core", "Configure...", "msgShowOptionsDialog");
+    return Tr::tr("Configure...", "msgShowOptionsDialog");
 }
 
 /*!
@@ -287,11 +303,9 @@ QString ICore::msgShowOptionsDialog()
 QString ICore::msgShowOptionsDialogToolTip()
 {
     if (Utils::HostOsInfo::isMacHost())
-        return QCoreApplication::translate("Core", "Open Preferences dialog.",
-                                           "msgShowOptionsDialogToolTip (mac version)");
+        return Tr::tr("Open Preferences dialog.", "msgShowOptionsDialogToolTip (mac version)");
     else
-        return QCoreApplication::translate("Core", "Open Options dialog.",
-                                           "msgShowOptionsDialogToolTip (non-mac version)");
+        return Tr::tr("Open Options dialog.", "msgShowOptionsDialogToolTip (non-mac version)");
 }
 
 /*!
@@ -315,6 +329,7 @@ bool ICore::showWarningWithOptions(const QString &title, const QString &text,
         parent = m_mainwindow;
     QMessageBox msgBox(QMessageBox::Warning, title, text,
                        QMessageBox::Ok, parent);
+    msgBox.setEscapeButton(QMessageBox::Ok);
     if (!details.isEmpty())
         msgBox.setDetailedText(details);
     QAbstractButton *settingsButton = nullptr;
@@ -386,8 +401,16 @@ QString ICore::userInterfaceLanguage()
     return qApp->property("qtc_locale").toString();
 }
 
+static QString pathHelper(const QString &rel)
+{
+    if (rel.isEmpty())
+        return rel;
+    if (rel.startsWith('/'))
+        return rel;
+    return '/' + rel;
+}
 /*!
-    Returns the absolute path that is used for resources like
+    Returns the absolute path for the relative path \a rel that is used for resources like
     project templates and the debugger macros.
 
     This abstraction is needed to avoid platform-specific code all over
@@ -396,13 +419,15 @@ QString ICore::userInterfaceLanguage()
 
     \sa userResourcePath()
 */
-QString ICore::resourcePath()
+FilePath ICore::resourcePath(const QString &rel)
 {
-    return QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH);
+    return FilePath::fromString(
+               QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH))
+           / rel;
 }
 
 /*!
-    Returns the absolute path in the users directory that is used for
+    Returns the absolute path for the relative path \a rel in the users directory that is used for
     resources like project templates.
 
     Use this function for finding the place for resources that the user may
@@ -411,7 +436,7 @@ QString ICore::resourcePath()
     \sa resourcePath()
 */
 
-QString ICore::userResourcePath()
+FilePath ICore::userResourcePath(const QString &rel)
 {
     // Create qtcreator dir if it doesn't yet exist
     const QString configDir = QFileInfo(settings(QSettings::UserScope)->fileName()).path();
@@ -423,24 +448,26 @@ QString ICore::userResourcePath()
             qWarning() << "could not create" << urp;
     }
 
-    return urp;
+    return FilePath::fromString(urp + pathHelper(rel));
 }
 
 /*!
-    Returns a writable path that can be used for persistent cache files.
+    Returns a writable path for the relative path \a rel that can be used for persistent cache files.
 */
-QString ICore::cacheResourcePath()
+FilePath ICore::cacheResourcePath(const QString &rel)
 {
-    return QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
+    return FilePath::fromString(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)
+                                + pathHelper(rel));
 }
 
 /*!
-    Returns the path to resources written by the installer, for example
-    pre-defined kits and toolchains.
+    Returns the path, based on the relative path \a rel, to resources written by the installer,
+    for example pre-defined kits and toolchains.
 */
-QString ICore::installerResourcePath()
+FilePath ICore::installerResourcePath(const QString &rel)
 {
-    return QFileInfo(settings(QSettings::SystemScope)->fileName()).path() + '/' + Constants::IDE_ID;
+    return FilePath::fromString(settings(QSettings::SystemScope)->fileName()).parentDir()
+                                / Constants::IDE_ID / rel;
 }
 
 /*!
@@ -473,20 +500,22 @@ QString ICore::userPluginPath()
 }
 
 /*!
-    Returns the path to the command line tools that are included in the \QC
-    installation.
+    Returns the path, based on the relative path \a rel, to the command line tools that are
+    included in the \QC installation.
  */
-QString ICore::libexecPath()
+FilePath ICore::libexecPath(const QString &rel)
 {
-    return QDir::cleanPath(QApplication::applicationDirPath() + '/' + RELATIVE_LIBEXEC_PATH);
+    return FilePath::fromString(QDir::cleanPath(QApplication::applicationDirPath()
+                                                + pathHelper(RELATIVE_LIBEXEC_PATH)))
+           / rel;
 }
 
-QString ICore::crashReportsPath()
+FilePath ICore::crashReportsPath()
 {
     if (Utils::HostOsInfo::isMacHost())
-        return libexecPath() + "/crashpad_reports/completed";
+        return Core::ICore::userResourcePath("crashpad_reports/completed");
     else
-        return libexecPath() + "/crashpad_reports/reports";
+        return libexecPath("crashpad_reports/reports");
 }
 
 QString ICore::ideDisplayName()
@@ -502,31 +531,31 @@ static QString clangIncludePath(const QString &clangVersion)
 /*!
     \internal
 */
-QString ICore::clangIncludeDirectory(const QString &clangVersion,
-                                     const QString &clangFallbackIncludeDir)
+FilePath ICore::clangIncludeDirectory(const QString &clangVersion,
+                                      const FilePath &clangFallbackIncludeDir)
 {
-    QDir dir(libexecPath() + "/clang" + clangIncludePath(clangVersion));
-    if (!dir.exists() || !QFileInfo(dir, "stdint.h").exists())
-        dir = QDir(clangFallbackIncludeDir);
-    return QDir::toNativeSeparators(dir.canonicalPath());
+    FilePath dir = libexecPath("clang" + clangIncludePath(clangVersion));
+    if (!dir.exists() || !dir.pathAppended("stdint.h").exists())
+        dir = clangFallbackIncludeDir;
+    return dir.canonicalPath();
 }
 
 /*!
     \internal
 */
-static QString clangBinary(const QString &binaryBaseName, const QString &clangBinDirectory)
+static FilePath clangBinary(const QString &binaryBaseName, const FilePath &clangBinDirectory)
 {
-    const QString hostExeSuffix(QTC_HOST_EXE_SUFFIX);
-    QFileInfo executable(ICore::libexecPath() + "/clang/bin/" + binaryBaseName + hostExeSuffix);
+    FilePath executable =
+        ICore::libexecPath("clang/bin").pathAppended(binaryBaseName).withExecutableSuffix();
     if (!executable.exists())
-        executable = QFileInfo(clangBinDirectory + "/" + binaryBaseName + hostExeSuffix);
-    return QDir::toNativeSeparators(executable.canonicalFilePath());
+        executable = clangBinDirectory.pathAppended(binaryBaseName).withExecutableSuffix();
+    return executable.canonicalPath();
 }
 
 /*!
     \internal
 */
-QString ICore::clangExecutable(const QString &clangBinDirectory)
+FilePath ICore::clangExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clang", clangBinDirectory);
 }
@@ -534,7 +563,15 @@ QString ICore::clangExecutable(const QString &clangBinDirectory)
 /*!
     \internal
 */
-QString ICore::clangTidyExecutable(const QString &clangBinDirectory)
+FilePath ICore::clangdExecutable(const FilePath &clangBinDirectory)
+{
+    return clangBinary("clangd", clangBinDirectory);
+}
+
+/*!
+    \internal
+*/
+FilePath ICore::clangTidyExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clang-tidy", clangBinDirectory);
 }
@@ -542,7 +579,7 @@ QString ICore::clangTidyExecutable(const QString &clangBinDirectory)
 /*!
     \internal
 */
-QString ICore::clazyStandaloneExecutable(const QString &clangBinDirectory)
+FilePath ICore::clazyStandaloneExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clazy-standalone", clangBinDirectory);
 }
@@ -550,17 +587,21 @@ QString ICore::clazyStandaloneExecutable(const QString &clangBinDirectory)
 static QString compilerString()
 {
 #if defined(Q_CC_CLANG) // must be before GNU, because clang claims to be GNU too
-    QString isAppleString;
+    QString platformSpecific;
 #if defined(__apple_build_version__) // Apple clang has other version numbers
-    isAppleString = QLatin1String(" (Apple)");
+    platformSpecific = QLatin1String(" (Apple)");
+#elif defined(Q_CC_MSVC)
+    platformSpecific = QLatin1String(" (clang-cl)");
 #endif
     return QLatin1String("Clang " ) + QString::number(__clang_major__) + QLatin1Char('.')
-            + QString::number(__clang_minor__) + isAppleString;
+            + QString::number(__clang_minor__) + platformSpecific;
 #elif defined(Q_CC_GNU)
     return QLatin1String("GCC " ) + QLatin1String(__VERSION__);
 #elif defined(Q_CC_MSVC)
     if (_MSC_VER > 1999)
         return QLatin1String("MSVC <unknown>");
+    if (_MSC_VER >= 1930)
+        return QLatin1String("MSVC 2022");
     if (_MSC_VER >= 1920)
         return QLatin1String("MSVC 2019");
     if (_MSC_VER >= 1910)
@@ -579,10 +620,10 @@ QString ICore::versionString()
 {
     QString ideVersionDescription;
     if (QLatin1String(Constants::IDE_VERSION_LONG) != QLatin1String(Constants::IDE_VERSION_DISPLAY))
-        ideVersionDescription = tr(" (%1)").arg(QLatin1String(Constants::IDE_VERSION_LONG));
-    return tr("%1 %2%3").arg(QLatin1String(Constants::IDE_DISPLAY_NAME),
-                             QLatin1String(Constants::IDE_VERSION_DISPLAY),
-                             ideVersionDescription);
+        ideVersionDescription = Tr::tr(" (%1)").arg(QLatin1String(Constants::IDE_VERSION_LONG));
+    return Tr::tr("%1 %2%3").arg(QLatin1String(Constants::IDE_DISPLAY_NAME),
+                                 QLatin1String(Constants::IDE_VERSION_DISPLAY),
+                                 ideVersionDescription);
 }
 
 /*!
@@ -590,9 +631,9 @@ QString ICore::versionString()
 */
 QString ICore::buildCompatibilityString()
 {
-    return tr("Based on Qt %1 (%2, %3 bit)").arg(QLatin1String(qVersion()),
+    return Tr::tr("Based on Qt %1 (%2, %3)").arg(QLatin1String(qVersion()),
                                                  compilerString(),
-                                                 QString::number(QSysInfo::WordSize));
+                                                 QSysInfo::buildCpuArchitecture());
 }
 
 /*!
@@ -649,7 +690,7 @@ QWidget *ICore::dialogParent()
     QWidget *active = QApplication::activeModalWidget();
     if (!active)
         active = QApplication::activeWindow();
-    if (!active || (active && active->windowFlags().testFlag(Qt::SplashScreen)))
+    if (!active || (active && active->windowFlags().testAnyFlags(Qt::SplashScreen | Qt::Popup)))
         active = m_mainwindow;
     return active;
 }
@@ -681,7 +722,9 @@ void ICore::raiseWindow(QWidget *widget)
     if (!widget)
         return;
     QWidget *window = widget->window();
-    if (window && window == m_mainwindow) {
+    if (!window)
+        return;
+    if (window == m_mainwindow) {
         m_mainwindow->raiseWindow();
     } else {
         window->raise();
@@ -773,15 +816,20 @@ void ICore::registerWindow(QWidget *window, const Context &context)
     new WindowSupport(window, context); // deletes itself when widget is destroyed
 }
 
+void ICore::restartTrimmer()
+{
+    m_mainwindow->restartTrimmer();
+}
+
 /*!
-    Opens files using \a arguments and \a flags like it would be
+    Opens files using \a filePaths and \a flags like it would be
     done if they were given to \QC on the command line, or
     they were opened via \uicontrol File > \uicontrol Open.
 */
 
-void ICore::openFiles(const QStringList &arguments, ICore::OpenFilesFlags flags)
+void ICore::openFiles(const FilePaths &filePaths, ICore::OpenFilesFlags flags)
 {
-    MainWindow::openFiles(arguments, flags);
+    MainWindow::openFiles(filePaths, flags);
 }
 
 /*!
@@ -814,9 +862,9 @@ QString ICore::systemInformation()
      return result;
 }
 
-static const QByteArray &screenShotsPath()
+static const QString &screenShotsPath()
 {
-    static const QByteArray path = qgetenv("QTC_SCREENSHOTS_PATH");
+    static const QString path = qtcEnvironmentVariable("QTC_SCREENSHOTS_PATH");
     return path;
 }
 
@@ -914,4 +962,12 @@ void ICore::updateNewItemDialogState()
     emit instance()->newItemDialogStateChanged();
 }
 
+/*!
+    \internal
+*/
+void ICore::setNewDialogFactory(const std::function<NewDialog *(QWidget *)> &newFactory)
+{
+    m_newDialogFactory = newFactory;
+}
+
 } // namespace Core
