@@ -1,34 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "fileshareprotocol.h"
+
+#include "cpastertr.h"
 #include "fileshareprotocolsettingspage.h"
 
-#include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
 
+#include <utils/fileutils.h>
+
 #include <QXmlStreamReader>
 #include <QXmlStreamAttribute>
 #include <QTemporaryFile>
@@ -38,30 +19,22 @@
 
 enum { debug = 0 };
 
-static const char tempPatternC[] = "pasterXXXXXX.xml";
-static const char tempGlobPatternC[] = "paster*.xml";
-static const char pasterElementC[] = "paster";
-static const char userElementC[] = "user";
-static const char descriptionElementC[] = "description";
-static const char textElementC[] = "text";
+const char tempPatternC[] = "pasterXXXXXX.xml";
+const char tempGlobPatternC[] = "paster*.xml";
+const char pasterElementC[] = "paster";
+const char userElementC[] = "user";
+const char descriptionElementC[] = "description";
+const char textElementC[] = "text";
 
 namespace CodePaster {
 
-FileShareProtocol::FileShareProtocol() :
-    m_settings(new FileShareProtocolSettings),
-    m_settingsPage(new FileShareProtocolSettingsPage(m_settings))
-{
-    m_settings->fromSettings(Core::ICore::settings());
-}
+FileShareProtocol::FileShareProtocol() = default;
 
-FileShareProtocol::~FileShareProtocol()
-{
-    delete m_settingsPage;
-}
+FileShareProtocol::~FileShareProtocol() = default;
 
 QString FileShareProtocol::name() const
 {
-    return m_settingsPage->displayName();
+    return m_settings.displayName();
 }
 
 unsigned FileShareProtocol::capabilities() const
@@ -74,9 +47,9 @@ bool FileShareProtocol::hasSettings() const
     return true;
 }
 
-Core::IOptionsPage *FileShareProtocol::settingsPage() const
+const Core::IOptionsPage *FileShareProtocol::settingsPage() const
 {
-    return m_settingsPage;
+    return &m_settings;
 }
 
 static bool parse(const QString &fileName,
@@ -95,16 +68,16 @@ static bool parse(const QString &fileName,
 
     QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly|QIODevice::Text)) {
-        *errorMessage = FileShareProtocol::tr("Cannot open %1: %2").arg(fileName, file.errorString());
+        *errorMessage = Tr::tr("Cannot open %1: %2").arg(fileName, file.errorString());
         return false;
     }
     QXmlStreamReader reader(&file);
     while (!reader.atEnd()) {
         if (reader.readNext() == QXmlStreamReader::StartElement) {
-            const QStringRef elementName = reader.name();
+            const auto elementName = reader.name();
             // Check start element
             if (elementCount == 0 && elementName != QLatin1String(pasterElementC)) {
-                *errorMessage = FileShareProtocol::tr("%1 does not appear to be a paster file.").arg(fileName);
+                *errorMessage = Tr::tr("%1 does not appear to be a paster file.").arg(fileName);
                 return false;
             }
             // Parse elements
@@ -118,7 +91,7 @@ static bool parse(const QString &fileName,
         }
     }
     if (reader.hasError()) {
-        *errorMessage = FileShareProtocol::tr("Error in %1 at %2: %3")
+        *errorMessage = Tr::tr("Error in %1 at %2: %3")
                         .arg(fileName).arg(reader.lineNumber()).arg(reader.errorString());
         return false;
     }
@@ -127,9 +100,9 @@ static bool parse(const QString &fileName,
 
 bool FileShareProtocol::checkConfiguration(QString *errorMessage)
 {
-    if (m_settings->path.isEmpty()) {
+    if (m_settings.path.value().isEmpty()) {
         if (errorMessage)
-            *errorMessage = tr("Please configure a path.");
+            *errorMessage = Tr::tr("Please configure a path.");
         return false;
     }
     return true;
@@ -140,10 +113,10 @@ void FileShareProtocol::fetch(const QString &id)
     // Absolute or relative path name.
     QFileInfo fi(id);
     if (fi.isRelative())
-        fi = QFileInfo(m_settings->path + QLatin1Char('/') + id);
+        fi = QFileInfo(m_settings.path.value() + '/' + id);
     QString errorMessage;
     QString text;
-    if (parse(fi.absoluteFilePath(), &errorMessage, 0, 0, &text))
+    if (parse(fi.absoluteFilePath(), &errorMessage, nullptr, nullptr, &text))
         emit fetchDone(id, text, false);
     else
         emit fetchDone(id, errorMessage, true);
@@ -152,7 +125,7 @@ void FileShareProtocol::fetch(const QString &id)
 void FileShareProtocol::list()
 {
     // Read out directory, display by date (latest first)
-    QDir dir(m_settings->path, QLatin1String(tempGlobPatternC),
+    QDir dir(m_settings.path.value(), tempGlobPatternC,
              QDir::Time, QDir::Files|QDir::NoDotAndDotDot|QDir::Readable);
     QStringList entries;
     QString user;
@@ -160,7 +133,7 @@ void FileShareProtocol::list()
     QString errorMessage;
     const QChar blank = QLatin1Char(' ');
     const QFileInfoList entryInfoList = dir.entryInfoList();
-    const int count = qMin(m_settings->displayCount, entryInfoList.size());
+    const int count = qMin(int(m_settings.displayCount()), entryInfoList.size());
     for (int i = 0; i < count; i++) {
         const QFileInfo& entryFi = entryInfoList.at(i);
         if (parse(entryFi.absoluteFilePath(), &errorMessage, &user, &description)) {
@@ -177,14 +150,17 @@ void FileShareProtocol::list()
     emit listDone(name(), entries);
 }
 
-void FileShareProtocol::paste(const QString &text,
-                              ContentType /* ct */, int /* expiryDays */,
-                              const QString &username,
-                              const QString & /* comment */,
-                              const QString &description)
+void FileShareProtocol::paste(
+        const QString &text,
+        ContentType /* ct */,
+        int /* expiryDays */,
+        const QString &username,
+        const QString & /* comment */,
+        const QString &description
+        )
 {
     // Write out temp XML file
-    Utils::TempFileSaver saver(m_settings->path + QLatin1Char('/') + QLatin1String(tempPatternC));
+    Utils::TempFileSaver saver(m_settings.path.value() + '/' + tempPatternC);
     saver.setAutoRemove(false);
     if (!saver.hasError()) {
         // Flat text sections embedded into pasterElement
@@ -202,10 +178,10 @@ void FileShareProtocol::paste(const QString &text,
         saver.setResult(&writer);
     }
     if (!saver.finalize()) {
-        Core::MessageManager::write(saver.errorString());
+        Core::MessageManager::writeDisrupting(saver.errorString());
         return;
     }
 
-    Core::MessageManager::write(tr("Pasted: %1").arg(saver.fileName()));
+    emit pasteDone(saver.filePath().toUserOutput());
 }
 } // namespace CodePaster
