@@ -1,8 +1,9 @@
 // Copyright (C) 2019 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qtmarketplacewelcomepage.h"
 
+#include "marketplacetr.h"
 #include "productlistmodel.h"
 
 #include <coreplugin/welcomepagehelper.h>
@@ -25,7 +26,7 @@ using namespace Utils;
 
 QString QtMarketplaceWelcomePage::title() const
 {
-    return tr("Marketplace");
+    return Tr::tr("Marketplace");
 }
 
 int QtMarketplaceWelcomePage::priority() const
@@ -45,7 +46,7 @@ public:
     {
         auto searchBox = new Core::SearchBox(this);
         m_searcher = searchBox->m_lineEdit;
-        m_searcher->setPlaceholderText(QtMarketplaceWelcomePage::tr("Search in Marketplace..."));
+        m_searcher->setPlaceholderText(Tr::tr("Search in Marketplace..."));
 
         auto vbox = new QVBoxLayout(this);
         vbox->setContentsMargins(0, 0, 0, Core::WelcomePageHelpers::ItemGap);
@@ -82,7 +83,7 @@ public:
             f.setPixelSize(20);
             m_errorLabel->setFont(f);
             const QString txt
-                    = QtMarketplaceWelcomePage::tr(
+                    = Tr::tr(
                         "<p>Could not fetch data from Qt Marketplace.</p><p>Try with your browser "
                         "instead: <a href='https://marketplace.qt.io'>https://marketplace.qt.io</a>"
                         "</p><br/><p><small><i>Error: %1</i></small></p>").arg(message);
