@@ -1,72 +1,128 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "buildconfiguration.h"
 
 #include "buildaspects.h"
-#include "buildenvironmentwidget.h"
 #include "buildinfo.h"
 #include "buildsteplist.h"
 #include "buildstepspage.h"
 #include "buildsystem.h"
-#include "namedwidget.h"
+#include "customparser.h"
+#include "environmentwidget.h"
 #include "kit.h"
 #include "kitinformation.h"
-#include "kitmanager.h"
-#include "project.h"
-#include "projectexplorer.h"
+#include "namedwidget.h"
 #include "projectexplorerconstants.h"
-#include "projectmacroexpander.h"
+#include "projectexplorer.h"
+#include "projectexplorertr.h"
+#include "project.h"
+#include "projectmanager.h"
 #include "projecttree.h"
 #include "target.h"
-#include "session.h"
-#include "toolchain.h"
 
+#include <coreplugin/fileutils.h>
+#include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-#include <coreplugin/variablechooser.h>
+
+#include <projectexplorer/devicesupport/idevice.h>
 
 #include <utils/algorithm.h>
 #include <utils/detailswidget.h>
+#include <utils/layoutbuilder.h>
 #include <utils/macroexpander.h>
-#include <utils/mimetypes/mimedatabase.h>
-#include <utils/mimetypes/mimetype.h>
+#include <utils/mimeutils.h>
 #include <utils/qtcassert.h>
+#include <utils/variablechooser.h>
 
+#include <QCheckBox>
 #include <QDebug>
 #include <QFormLayout>
+#include <QLoggingCategory>
+#include <QVBoxLayout>
 
 using namespace Utils;
 
 const char BUILD_STEP_LIST_COUNT[] = "ProjectExplorer.BuildConfiguration.BuildStepListCount";
 const char BUILD_STEP_LIST_PREFIX[] = "ProjectExplorer.BuildConfiguration.BuildStepList.";
-const char CLEAR_SYSTEM_ENVIRONMENT_KEY[] = "ProjectExplorer.BuildConfiguration.ClearSystemEnvironment";
-const char USER_ENVIRONMENT_CHANGES_KEY[] = "ProjectExplorer.BuildConfiguration.UserEnvironmentChanges";
+const char CUSTOM_PARSERS_KEY[] = "ProjectExplorer.BuildConfiguration.CustomParsers";
+const char PARSE_STD_OUT_KEY[] = "ProjectExplorer.BuildConfiguration.ParseStandardOutput";
+
+Q_LOGGING_CATEGORY(bcLog, "qtc.buildconfig", QtWarningMsg)
 
 namespace ProjectExplorer {
 namespace Internal {
 
+class BuildEnvironmentWidget : public NamedWidget
+{
+
+public:
+    explicit BuildEnvironmentWidget(BuildConfiguration *bc)
+        : NamedWidget(Tr::tr("Build Environment"))
+    {
+        auto clearBox = new QCheckBox(Tr::tr("Clear system environment"), this);
+        clearBox->setChecked(!bc->useSystemEnvironment());
+
+        auto envWidget = new EnvironmentWidget(this, EnvironmentWidget::TypeLocal, clearBox);
+        envWidget->setBaseEnvironment(bc->baseEnvironment());
+        envWidget->setBaseEnvironmentText(bc->baseEnvironmentText());
+        envWidget->setUserChanges(bc->userEnvironmentChanges());
+
+        const EnvironmentWidget::OpenTerminalFunc openTerminalFunc
+                = [bc](const Utils::Environment &env) {
+            Core::FileUtils::openTerminal(bc->buildDirectory(), env);
+        };
+        envWidget->setOpenTerminalFunc(openTerminalFunc);
+
+        connect(envWidget, &EnvironmentWidget::userChangesChanged, this, [bc, envWidget] {
+            bc->setUserEnvironmentChanges(envWidget->userChanges());
+        });
+
+        connect(clearBox, &QAbstractButton::toggled, this, [bc, envWidget](bool checked) {
+            bc->setUseSystemEnvironment(!checked);
+            envWidget->setBaseEnvironment(bc->baseEnvironment());
+            envWidget->setBaseEnvironmentText(bc->baseEnvironmentText());
+        });
+
+        connect(bc, &BuildConfiguration::environmentChanged, this, [bc, envWidget] {
+            envWidget->setBaseEnvironment(bc->baseEnvironment());
+            envWidget->setBaseEnvironmentText(bc->baseEnvironmentText());
+        });
+
+        auto vbox = new QVBoxLayout(this);
+        vbox->setContentsMargins(0, 0, 0, 0);
+        vbox->addWidget(clearBox);
+        vbox->addWidget(envWidget);
+    }
+};
+
+class CustomParsersBuildWidget : public NamedWidget
+{
+public:
+    CustomParsersBuildWidget(BuildConfiguration *bc) : NamedWidget(Tr::tr("Custom Output Parsers"))
+    {
+        const auto layout = new QVBoxLayout(this);
+        layout->setContentsMargins(0, 0, 0, 0);
+
+        const auto pasteStdOutCB = new QCheckBox(Tr::tr("Parse standard output during build"), this);
+        pasteStdOutCB->setToolTip(Tr::tr("Makes output parsers look for diagnostics "
+                                     "on stdout rather than stderr."));
+        pasteStdOutCB->setChecked(bc->parseStdOut());
+        layout->addWidget(pasteStdOutCB);
+
+        connect(pasteStdOutCB, &QCheckBox::clicked, bc, &BuildConfiguration::setParseStdOut);
+        const auto selectionWidget = new CustomParsersSelectionWidget(this);
+        layout->addWidget(selectionWidget);
+
+        connect(selectionWidget, &CustomParsersSelectionWidget::selectionChanged,
+                [selectionWidget, bc] {
+            bc->setCustomParsers(selectionWidget->selectedParsers());
+        });
+        selectionWidget->setSelectedParsers(bc->customParsers());
+    }
+};
+
+
 class BuildConfigurationPrivate
 {
 public:
@@ -80,12 +136,16 @@ public:
     BuildStepList m_buildSteps;
     BuildStepList m_cleanSteps;
     BuildDirectoryAspect *m_buildDirectoryAspect = nullptr;
+    StringAspect *m_tooltipAspect = nullptr;
     FilePath m_lastEmittedBuildDirectory;
     mutable Environment m_cachedEnvironment;
     QString m_configWidgetDisplayName;
     bool m_configWidgetHasFrame = false;
-    QList<Core::Id> m_initialBuildSteps;
-    QList<Core::Id> m_initialCleanSteps;
+    QList<Utils::Id> m_initialBuildSteps;
+    QList<Utils::Id> m_initialCleanSteps;
+    Utils::MacroExpander m_macroExpander;
+    bool m_parseStdOut = false;
+    QList<Utils::Id> m_customParsers;
 
     // FIXME: Remove.
     BuildConfiguration::BuildType m_initialBuildType = BuildConfiguration::Unknown;
@@ -94,52 +154,66 @@ public:
 
 } // Internal
 
-BuildConfiguration::BuildConfiguration(Target *target, Core::Id id)
+BuildConfiguration::BuildConfiguration(Target *target, Utils::Id id)
     : ProjectConfiguration(target, id), d(new Internal::BuildConfigurationPrivate(this))
 {
     QTC_CHECK(target && target == this->target());
 
     MacroExpander *expander = macroExpander();
-    expander->setDisplayName(tr("Build Settings"));
+    expander->setDisplayName(Tr::tr("Build Settings"));
     expander->setAccumulating(true);
     expander->registerSubProvider([target] { return target->macroExpander(); });
 
-    expander->registerVariable("buildDir", tr("Build directory"),
+    expander->registerVariable("buildDir", Tr::tr("Build directory"),
             [this] { return buildDirectory().toUserOutput(); });
 
-    expander->registerVariable(Constants::VAR_CURRENTBUILD_NAME, tr("Name of current build"),
-            [this] { return displayName(); }, false);
+    expander->registerFileVariables("BuildConfig:BuildDirectory",
+                                    Tr::tr("Build directory"),
+                                    [this] { return buildDirectory(); });
 
-    expander->registerPrefix(Constants::VAR_CURRENTBUILD_ENV,
-                             tr("Variables in the current build environment"),
+    expander->registerVariable("BuildConfig:Name", Tr::tr("Name of the build configuration"),
+            [this] { return displayName(); });
+
+    expander->registerPrefix("BuildConfig:Env",
+                             Tr::tr("Variables in the build configuration's environment"),
                              [this](const QString &var) { return environment().expandedValueForKey(var); });
 
-    updateCacheAndEmitEnvironmentChanged();
+    connect(Core::ICore::instance(), &Core::ICore::systemEnvironmentChanged,
+            this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     connect(target, &Target::kitChanged,
             this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     connect(this, &BuildConfiguration::environmentChanged,
             this, &BuildConfiguration::emitBuildDirectoryChanged);
     connect(target->project(), &Project::environmentChanged,
-            this, &BuildConfiguration::environmentChanged);
+            this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     // Many macroexpanders are based on the current project, so they may change the environment:
     connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
             this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
 
-    d->m_buildDirectoryAspect = addAspect<BuildDirectoryAspect>();
+    d->m_buildDirectoryAspect = addAspect<BuildDirectoryAspect>(this);
     d->m_buildDirectoryAspect->setBaseFileName(target->project()->projectDirectory());
     d->m_buildDirectoryAspect->setEnvironment(environment());
     d->m_buildDirectoryAspect->setMacroExpanderProvider([this] { return macroExpander(); });
-    connect(d->m_buildDirectoryAspect, &BaseStringAspect::changed,
-            this, &BuildConfiguration::buildDirectoryChanged);
+    connect(d->m_buildDirectoryAspect, &StringAspect::changed,
+            this, &BuildConfiguration::emitBuildDirectoryChanged);
     connect(this, &BuildConfiguration::environmentChanged, this, [this] {
         d->m_buildDirectoryAspect->setEnvironment(environment());
-        this->target()->buildEnvironmentChanged(this);
+        emit this->target()->buildEnvironmentChanged(this);
+    });
+
+    d->m_tooltipAspect = addAspect<StringAspect>();
+    d->m_tooltipAspect->setLabelText(Tr::tr("Tooltip in target selector:"));
+    d->m_tooltipAspect->setToolTip(Tr::tr("Appears as a tooltip when hovering the build configuration"));
+    d->m_tooltipAspect->setDisplayStyle(StringAspect::LineEditDisplay);
+    d->m_tooltipAspect->setSettingsKey("ProjectExplorer.BuildConfiguration.Tooltip");
+    connect(d->m_tooltipAspect, &StringAspect::changed, this, [this] {
+        setToolTip(d->m_tooltipAspect->value());
     });
 
     connect(target, &Target::parsingStarted, this, &BuildConfiguration::enabledChanged);
     connect(target, &Target::parsingFinished, this, &BuildConfiguration::enabledChanged);
     connect(this, &BuildConfiguration::enabledChanged, this, [this] {
-        if (isActive() && project() == SessionManager::startupProject()) {
+        if (isActive() && project() == ProjectManager::startupProject()) {
             ProjectExplorerPlugin::updateActions();
             ProjectExplorerPlugin::updateRunActions();
         }
@@ -153,9 +227,14 @@ BuildConfiguration::~BuildConfiguration()
 
 FilePath BuildConfiguration::buildDirectory() const
 {
-    QString path = environment().expandVariables(d->m_buildDirectoryAspect->value().trimmed());
-    path = QDir::cleanPath(macroExpander()->expand(path));
-    return FilePath::fromString(QDir::cleanPath(QDir(target()->project()->projectDirectory().toString()).absoluteFilePath(path)));
+    FilePath path = FilePath::fromUserInput(
+        environment().expandVariables(d->m_buildDirectoryAspect->value().trimmed()));
+    path = macroExpander()->expand(path);
+    path = path.cleanPath();
+
+    const FilePath projectDir = target()->project()->projectDirectory();
+
+    return projectDir.resolvePath(path);
 }
 
 FilePath BuildConfiguration::rawBuildDirectory() const
@@ -168,6 +247,9 @@ void BuildConfiguration::setBuildDirectory(const FilePath &dir)
     if (dir == d->m_buildDirectoryAspect->filePath())
         return;
     d->m_buildDirectoryAspect->setFilePath(dir);
+    const FilePath fixedDir = BuildDirectoryAspect::fixupDir(buildDirectory());
+    if (!fixedDir.isEmpty())
+        d->m_buildDirectoryAspect->setFilePath(fixedDir);
     emitBuildDirectoryChanged();
 }
 
@@ -179,31 +261,43 @@ void BuildConfiguration::addConfigWidgets(const std::function<void(NamedWidget *
     adder(new Internal::BuildStepListWidget(buildSteps()));
     adder(new Internal::BuildStepListWidget(cleanSteps()));
 
-    QList<NamedWidget *> subConfigWidgets = createSubConfigWidgets();
-    foreach (NamedWidget *subConfigWidget, subConfigWidgets)
+    const QList<NamedWidget *> subConfigWidgets = createSubConfigWidgets();
+    for (NamedWidget *subConfigWidget : subConfigWidgets)
         adder(subConfigWidget);
 }
 
 void BuildConfiguration::doInitialize(const BuildInfo &info)
 {
+    updateCacheAndEmitEnvironmentChanged();
+
     setDisplayName(info.displayName);
     setDefaultDisplayName(info.displayName);
     setBuildDirectory(info.buildDirectory);
 
     d->m_initialBuildType = info.buildType;
 
-    for (Core::Id id : qAsConst(d->m_initialBuildSteps))
+    for (Utils::Id id : std::as_const(d->m_initialBuildSteps))
         d->m_buildSteps.appendStep(id);
 
-    for (Core::Id id : qAsConst(d->m_initialCleanSteps))
+    for (Utils::Id id : std::as_const(d->m_initialCleanSteps))
         d->m_cleanSteps.appendStep(id);
 
-    acquaintAspects();
-
     if (d->m_initializer)
         d->m_initializer(info);
 }
 
+MacroExpander *BuildConfiguration::macroExpander() const
+{
+    return &d->m_macroExpander;
+}
+
+bool BuildConfiguration::createBuildDirectory()
+{
+    const bool result = buildDirectory().ensureWritableDir();
+    buildDirectoryAspect()->validateInput();
+    return result;
+}
+
 void BuildConfiguration::setInitializer(const std::function<void(const BuildInfo &)> &initializer)
 {
     d->m_initializer = initializer;
@@ -228,18 +322,25 @@ NamedWidget *BuildConfiguration::createConfigWidget()
         widget = named;
     }
 
-    LayoutBuilder builder(widget);
-    for (ProjectConfigurationAspect *aspect : aspects()) {
-        if (aspect->isVisible())
-            aspect->addToLayout(builder.startNewRow());
+    Layouting::Form form;
+    for (BaseAspect *aspect : aspects()) {
+        if (aspect->isVisible()) {
+            form.addItem(aspect);
+            form.addItem(Layouting::br);
+        }
     }
+    form.addItem(Layouting::noMargin);
+    form.attachTo(widget);
 
     return named;
 }
 
 QList<NamedWidget *> BuildConfiguration::createSubConfigWidgets()
 {
-    return {new BuildEnvironmentWidget(this)};
+    return {
+        new Internal::BuildEnvironmentWidget(this),
+        new Internal::CustomParsersBuildWidget(this)
+    };
 }
 
 BuildSystem *BuildConfiguration::buildSystem() const
@@ -258,12 +359,12 @@ BuildStepList *BuildConfiguration::cleanSteps() const
     return &d->m_cleanSteps;
 }
 
-void BuildConfiguration::appendInitialBuildStep(Core::Id id)
+void BuildConfiguration::appendInitialBuildStep(Utils::Id id)
 {
     d->m_initialBuildSteps.append(id);
 }
 
-void BuildConfiguration::appendInitialCleanStep(Core::Id id)
+void BuildConfiguration::appendInitialCleanStep(Utils::Id id)
 {
     d->m_initialCleanSteps.append(id);
 }
@@ -272,20 +373,26 @@ QVariantMap BuildConfiguration::toMap() const
 {
     QVariantMap map = ProjectConfiguration::toMap();
 
-    map.insert(QLatin1String(CLEAR_SYSTEM_ENVIRONMENT_KEY), d->m_clearSystemEnvironment);
-    map.insert(QLatin1String(USER_ENVIRONMENT_CHANGES_KEY), EnvironmentItem::toStringList(d->m_userEnvironmentChanges));
+    map.insert(QLatin1String(Constants::CLEAR_SYSTEM_ENVIRONMENT_KEY), d->m_clearSystemEnvironment);
+    map.insert(QLatin1String(Constants::USER_ENVIRONMENT_CHANGES_KEY),
+               EnvironmentItem::toStringList(d->m_userEnvironmentChanges));
 
     map.insert(QLatin1String(BUILD_STEP_LIST_COUNT), 2);
     map.insert(QLatin1String(BUILD_STEP_LIST_PREFIX) + QString::number(0), d->m_buildSteps.toMap());
     map.insert(QLatin1String(BUILD_STEP_LIST_PREFIX) + QString::number(1), d->m_cleanSteps.toMap());
 
+    map.insert(PARSE_STD_OUT_KEY, d->m_parseStdOut);
+    map.insert(CUSTOM_PARSERS_KEY, transform(d->m_customParsers,&Utils::Id::toSetting));
+
     return map;
 }
 
 bool BuildConfiguration::fromMap(const QVariantMap &map)
 {
-    d->m_clearSystemEnvironment = map.value(QLatin1String(CLEAR_SYSTEM_ENVIRONMENT_KEY)).toBool();
-    d->m_userEnvironmentChanges = EnvironmentItem::fromStringList(map.value(QLatin1String(USER_ENVIRONMENT_CHANGES_KEY)).toStringList());
+    d->m_clearSystemEnvironment = map.value(QLatin1String(Constants::CLEAR_SYSTEM_ENVIRONMENT_KEY))
+                                      .toBool();
+    d->m_userEnvironmentChanges = EnvironmentItem::fromStringList(
+        map.value(QLatin1String(Constants::USER_ENVIRONMENT_CHANGES_KEY)).toStringList());
 
     updateCacheAndEmitEnvironmentChanged();
 
@@ -299,7 +406,7 @@ bool BuildConfiguration::fromMap(const QVariantMap &map)
             qWarning() << "No data for build step list" << i << "found!";
             continue;
         }
-        Core::Id id = idFromMap(data);
+        Utils::Id id = idFromMap(data);
         if (id == Constants::BUILDSTEPS_BUILD) {
             if (!d->m_buildSteps.fromMap(data))
                 qWarning() << "Failed to restore build step list";
@@ -311,7 +418,12 @@ bool BuildConfiguration::fromMap(const QVariantMap &map)
         }
     }
 
-    return ProjectConfiguration::fromMap(map);
+    d->m_parseStdOut = map.value(PARSE_STD_OUT_KEY).toBool();
+    d->m_customParsers = transform(map.value(CUSTOM_PARSERS_KEY).toList(), &Utils::Id::fromSetting);
+
+    const bool res = ProjectConfiguration::fromMap(map);
+    setToolTip(d->m_tooltipAspect->value());
+    return res;
 }
 
 void BuildConfiguration::updateCacheAndEmitEnvironmentChanged()
@@ -360,10 +472,12 @@ void BuildConfiguration::setBuildDirectorySettingsKey(const QString &key)
 Environment BuildConfiguration::baseEnvironment() const
 {
     Environment result;
-    if (useSystemEnvironment())
-        result = Environment::systemEnvironment();
+    if (useSystemEnvironment()) {
+        ProjectExplorer::IDevice::ConstPtr devicePtr = BuildDeviceKitAspect::device(kit());
+        result = devicePtr ? devicePtr->systemEnvironment() : Environment::systemEnvironment();
+    }
     addToEnvironment(result);
-    target()->kit()->addToEnvironment(result);
+    kit()->addToBuildEnvironment(result);
     result.modify(project()->additionalEnvironment());
     return result;
 }
@@ -371,9 +485,9 @@ Environment BuildConfiguration::baseEnvironment() const
 QString BuildConfiguration::baseEnvironmentText() const
 {
     if (useSystemEnvironment())
-        return tr("System Environment");
+        return Tr::tr("System Environment");
     else
-        return tr("Clean Environment");
+        return Tr::tr("Clean Environment");
 }
 
 Environment BuildConfiguration::environment() const
@@ -394,6 +508,19 @@ void BuildConfiguration::addToEnvironment(Environment &env) const
     Q_UNUSED(env)
 }
 
+const QList<Utils::Id> BuildConfiguration::customParsers() const
+{
+    return d->m_customParsers;
+}
+
+void BuildConfiguration::setCustomParsers(const QList<Utils::Id> &parsers)
+{
+    d->m_customParsers = parsers;
+}
+
+bool BuildConfiguration::parseStdOut() const { return d->m_parseStdOut; }
+void BuildConfiguration::setParseStdOut(bool b) { d->m_parseStdOut = b; }
+
 bool BuildConfiguration::useSystemEnvironment() const
 {
     return !d->m_clearSystemEnvironment;
@@ -414,15 +541,13 @@ void BuildConfiguration::setUserEnvironmentChanges(const EnvironmentItems &diff)
 
 bool BuildConfiguration::isEnabled() const
 {
-    return !buildSystem()->isParsing() && buildSystem()->hasParsingData();
+    return buildSystem()->hasParsingData();
 }
 
 QString BuildConfiguration::disabledReason() const
 {
-    if (buildSystem()->isParsing())
-        return (tr("The project is currently being parsed."));
     if (!buildSystem()->hasParsingData())
-        return (tr("The project was not parsed successfully."));
+        return (Tr::tr("The project was not parsed successfully."));
     return QString();
 }
 
@@ -462,25 +587,60 @@ bool BuildConfiguration::isActive() const
     return target()->isActive() && target()->activeBuildConfiguration() == this;
 }
 
-/*!
- * Helper function that prepends the directory containing the C++ toolchain to
- * PATH. This is used to in build configurations targeting broken build systems
- * to provide hints about which compiler to use.
- */
-
-void BuildConfiguration::prependCompilerPathToEnvironment(Kit *k, Environment &env)
+FilePath BuildConfiguration::buildDirectoryFromTemplate(const FilePath &projectDir,
+                                                        const FilePath &mainFilePath,
+                                                        const QString &projectName,
+                                                        const Kit *kit,
+                                                        const QString &bcName,
+                                                        BuildType buildType,
+                                                        const QString &buildSystem)
 {
-    const ToolChain *tc
-            = ToolChainKitAspect::toolChain(k, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
+    MacroExpander exp;
 
-    if (!tc)
-        return;
+    qCDebug(bcLog) << Q_FUNC_INFO << projectDir << mainFilePath << projectName << bcName;
 
-    const FilePath compilerDir = tc->compilerCommand().parentDir();
-    if (!compilerDir.isEmpty())
-        env.prependOrSetPath(compilerDir.toString());
+    exp.registerFileVariables("Project",
+                              Tr::tr("Main file of the project"),
+                              [mainFilePath] { return mainFilePath; });
+    exp.registerVariable("Project:Name",
+                         Tr::tr("Name of the project"),
+                         [projectName] { return projectName; });
+    exp.registerVariable("BuildConfig:Name",
+                         Tr::tr("Name of the project's active build configuration"),
+                         [bcName] { return bcName; });
+    exp.registerVariable("BuildSystem:Name",
+                         Tr::tr("Name of the project's active build system"),
+                         [buildSystem] { return buildSystem; });
+    exp.registerVariable("CurrentBuild:Type",
+                         Tr::tr("Type of current build"),
+                         [buildType] { return buildTypeName(buildType); }, false);
+    exp.registerVariable("BuildConfig:Type",
+                         Tr::tr("Type of the project's active build configuration"),
+                         [buildType] { return buildTypeName(buildType); });
+    exp.registerSubProvider([kit] { return kit->macroExpander(); });
+
+    FilePath buildDir = FilePath::fromUserInput(ProjectExplorerPlugin::buildDirectoryTemplate());
+    qCDebug(bcLog) << "build dir template:" << buildDir.toUserOutput();
+    buildDir = exp.expand(buildDir);
+    qCDebug(bcLog) << "expanded build:" << buildDir.toUserOutput();
+    buildDir = buildDir.withNewPath(buildDir.path().replace(" ", "-"));
+
+    auto buildDevice = BuildDeviceKitAspect::device(kit);
+
+    if (buildDir.isAbsolutePath()) {
+        bool isReachable = buildDevice->ensureReachable(buildDir);
+        if (!isReachable)
+            return {};
+        return buildDevice->rootPath().withNewMappedPath(buildDir);
+    }
+
+    bool isReachable = buildDevice->ensureReachable(projectDir);
+    if (!isReachable)
+        return {};
+
+    const FilePath baseDir = buildDevice->rootPath().withNewMappedPath(projectDir);
+    return baseDir.resolvePath(buildDir);
 }
-
 ///
 // IBuildConfigurationFactory
 ///
@@ -498,8 +658,8 @@ BuildConfigurationFactory::~BuildConfigurationFactory()
     g_buildConfigurationFactories.removeOne(this);
 }
 
-const Tasks BuildConfigurationFactory::reportIssues(ProjectExplorer::Kit *kit, const QString &projectPath,
-                                                          const QString &buildDir) const
+const Tasks BuildConfigurationFactory::reportIssues(Kit *kit, const FilePath &projectPath,
+                                                    const FilePath &buildDir) const
 {
     if (m_issueReporter)
         return m_issueReporter(kit, projectPath, buildDir);
@@ -529,7 +689,7 @@ const QList<BuildInfo>
     return list;
 }
 
-bool BuildConfigurationFactory::supportsTargetDeviceType(Core::Id id) const
+bool BuildConfigurationFactory::supportsTargetDeviceType(Utils::Id id) const
 {
     if (m_supportedTargetDeviceTypes.isEmpty())
         return true;
@@ -540,11 +700,10 @@ bool BuildConfigurationFactory::supportsTargetDeviceType(Core::Id id) const
 BuildConfigurationFactory *BuildConfigurationFactory::find(const Kit *k, const FilePath &projectPath)
 {
     QTC_ASSERT(k, return nullptr);
-    const Core::Id deviceType = DeviceTypeKitAspect::deviceTypeId(k);
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
-        if (Utils::mimeTypeForFile(projectPath.toString())
-                .matchesName(factory->m_supportedProjectMimeTypeName)
-                && factory->supportsTargetDeviceType(deviceType))
+    const Utils::Id deviceType = DeviceTypeKitAspect::deviceTypeId(k);
+    for (BuildConfigurationFactory *factory : std::as_const(g_buildConfigurationFactories)) {
+        if (Utils::mimeTypeForFile(projectPath).matchesName(factory->m_supportedProjectMimeTypeName)
+            && factory->supportsTargetDeviceType(deviceType))
             return factory;
     }
     return nullptr;
@@ -553,14 +712,14 @@ BuildConfigurationFactory *BuildConfigurationFactory::find(const Kit *k, const F
 // create
 BuildConfigurationFactory * BuildConfigurationFactory::find(Target *parent)
 {
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
+    for (BuildConfigurationFactory *factory : std::as_const(g_buildConfigurationFactories)) {
         if (factory->canHandle(parent))
             return factory;
     }
     return nullptr;
 }
 
-void BuildConfigurationFactory::setSupportedProjectType(Core::Id id)
+void BuildConfigurationFactory::setSupportedProjectType(Utils::Id id)
 {
     m_supportedProjectType = id;
 }
@@ -570,7 +729,7 @@ void BuildConfigurationFactory::setSupportedProjectMimeTypeName(const QString &m
     m_supportedProjectMimeTypeName = mimeTypeName;
 }
 
-void BuildConfigurationFactory::addSupportedTargetDeviceType(Core::Id id)
+void BuildConfigurationFactory::addSupportedTargetDeviceType(Utils::Id id)
 {
     m_supportedTargetDeviceTypes.append(id);
 }
@@ -614,15 +773,14 @@ BuildConfiguration *BuildConfigurationFactory::create(Target *parent, const Buil
 
 BuildConfiguration *BuildConfigurationFactory::restore(Target *parent, const QVariantMap &map)
 {
-    const Core::Id id = idFromMap(map);
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
+    const Utils::Id id = idFromMap(map);
+    for (BuildConfigurationFactory *factory : std::as_const(g_buildConfigurationFactories)) {
         QTC_ASSERT(factory->m_creator, return nullptr);
         if (!factory->canHandle(parent))
             continue;
         if (!id.name().startsWith(factory->m_buildConfigId.name()))
             continue;
         BuildConfiguration *bc = factory->m_creator(parent);
-        bc->acquaintAspects();
         QTC_ASSERT(bc, return nullptr);
         if (!bc->fromMap(map)) {
             delete bc;
