@@ -1,34 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "nodeinstance.h"
 
-#include <QPainter>
 #include <modelnode.h>
 
 #include <QDebug>
+#include <QPainter>
+#include <QVector2D>
+#include <QVector3D>
+#include <QVector4D>
 
 QT_BEGIN_NAMESPACE
 void qt_blurImage(QPainter *painter, QImage &blurImage, qreal radius, bool quality, bool alphaOnly, int transposed = 0);
@@ -40,20 +21,12 @@ class ProxyNodeInstanceData
 {
 public:
     ProxyNodeInstanceData()
-        : parentInstanceId(-1),
-          penWidth(1),
-          isAnchoredBySibling(false),
-          isAnchoredByChildren(false),
-          hasContent(false),
-          isMovable(false),
-          isResizable(false),
-          isInLayoutable(false),
-          directUpdates(false)
     {}
 
-    qint32 parentInstanceId;
+    qint32 parentInstanceId{-1};
     ModelNode modelNode;
     QRectF boundingRect;
+    QRectF boundingRectPixmap;
     QRectF contentItemBoundingRect;
     QPointF position;
     QSizeF size;
@@ -61,14 +34,14 @@ public:
     QTransform contentTransform;
     QTransform contentItemTransform;
     QTransform sceneTransform;
-    int penWidth;
-    bool isAnchoredBySibling;
-    bool isAnchoredByChildren;
-    bool hasContent;
-    bool isMovable;
-    bool isResizable;
-    bool isInLayoutable;
-    bool directUpdates;
+    int penWidth{1};
+    bool isAnchoredBySibling{false};
+    bool isAnchoredByChildren{false};
+    bool hasContent{false};
+    bool isMovable{false};
+    bool isResizable{false};
+    bool isInLayoutable{false};
+    bool directUpdates{false};
 
 
     QHash<PropertyName, QVariant> propertyValues;
@@ -82,11 +55,10 @@ public:
     QString errorMessage;
 
     QHash<PropertyName, QPair<PropertyName, qint32> > anchors;
+    QStringList allStates;
 };
 
-NodeInstance::NodeInstance()
-{
-}
+NodeInstance::NodeInstance() = default;
 
 NodeInstance::NodeInstance(ProxyNodeInstanceData *dPointer)
     : d(dPointer)
@@ -95,27 +67,18 @@ NodeInstance::NodeInstance(ProxyNodeInstanceData *dPointer)
 
 NodeInstance NodeInstance::create(const ModelNode &node)
 {
-    ProxyNodeInstanceData *d = new ProxyNodeInstanceData;
+    auto d = new ProxyNodeInstanceData;
 
     d->modelNode = node;
 
     return NodeInstance(d);
 }
 
-NodeInstance::~NodeInstance()
-{
-}
+NodeInstance::~NodeInstance() = default;
 
-NodeInstance::NodeInstance(const NodeInstance &other)
-  : d(other.d)
-{
-}
+NodeInstance::NodeInstance(const NodeInstance &other) = default;
 
-NodeInstance &NodeInstance::operator=(const NodeInstance &other)
-{
-    d = other.d;
-    return *this;
-}
+NodeInstance &NodeInstance::operator=(const NodeInstance &other) = default;
 
 ModelNode NodeInstance::modelNode() const
 {
@@ -186,6 +149,10 @@ bool NodeInstance::hasError() const
     return !d->errorMessage.isEmpty();
 }
 
+QStringList NodeInstance::allStateNames() const
+{
+    return d->allStates;
+}
 
 bool NodeInstance::isValid() const
 {
@@ -200,9 +167,11 @@ void NodeInstance::makeInvalid()
 
 QRectF NodeInstance::boundingRect() const
 {
-    if (isValid())
+    if (isValid()) {
+        if (d->boundingRectPixmap.isValid())
+            return d->boundingRectPixmap;
         return d->boundingRect;
-    else
+    } else
         return QRectF();
 }
 
@@ -318,8 +287,76 @@ int NodeInstance::penWidth() const
 
 QVariant NodeInstance::property(const PropertyName &name) const
 {
-    if (isValid())
-        return d->propertyValues.value(name);
+    if (isValid()) {
+        if (d->propertyValues.contains(name)) {
+            return d->propertyValues.value(name);
+        } else {
+            // Query may be for a subproperty, e.g. scale.x
+            const int index = name.indexOf('.');
+            if (index != -1) {
+                PropertyName parentPropName = name.left(index);
+                QVariant varValue = d->propertyValues.value(parentPropName);
+                if (varValue.type() == QVariant::Vector2D) {
+                    auto value = varValue.value<QVector2D>();
+                    char subProp = name.right(1)[0];
+                    float subValue = 0.f;
+                    switch (subProp) {
+                    case 'x':
+                        subValue = value.x();
+                        break;
+                    case 'y':
+                        subValue = value.y();
+                        break;
+                    default:
+                        subValue = 0.f;
+                        break;
+                    }
+                    return QVariant(subValue);
+                } else if (varValue.type() == QVariant::Vector3D) {
+                    auto value = varValue.value<QVector3D>();
+                    char subProp = name.right(1)[0];
+                    float subValue = 0.f;
+                    switch (subProp) {
+                    case 'x':
+                        subValue = value.x();
+                        break;
+                    case 'y':
+                        subValue = value.y();
+                        break;
+                    case 'z':
+                        subValue = value.z();
+                        break;
+                    default:
+                        subValue = 0.f;
+                        break;
+                    }
+                    return QVariant(subValue);
+                } else if (varValue.type() == QVariant::Vector4D) {
+                    auto value = varValue.value<QVector4D>();
+                    char subProp = name.right(1)[0];
+                    float subValue = 0.f;
+                    switch (subProp) {
+                    case 'x':
+                        subValue = value.x();
+                        break;
+                    case 'y':
+                        subValue = value.y();
+                        break;
+                    case 'z':
+                        subValue = value.z();
+                        break;
+                    case 'w':
+                        subValue = value.w();
+                        break;
+                    default:
+                        subValue = 0.f;
+                        break;
+                    }
+                    return QVariant(subValue);
+                }
+            }
+        }
+    }
 
     return QVariant();
 }
@@ -374,6 +411,64 @@ QPair<PropertyName, qint32> NodeInstance::anchor(const PropertyName &name) const
 
 void NodeInstance::setProperty(const PropertyName &name, const QVariant &value)
 {
+    const int index = name.indexOf('.');
+    if (index != -1) {
+        PropertyName parentPropName = name.left(index);
+        QVariant oldValue = d->propertyValues.value(parentPropName);
+        QVariant newValueVar;
+        bool update = false;
+        if (oldValue.type() == QVariant::Vector2D) {
+            QVector2D newValue;
+            if (oldValue.type() == QVariant::Vector2D)
+                newValue = oldValue.value<QVector2D>();
+            if (name.endsWith(".x")) {
+                newValue.setX(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".y")) {
+                newValue.setY(value.toFloat());
+                update = true;
+            }
+            newValueVar = newValue;
+        } else if (oldValue.type() == QVariant::Vector3D) {
+            QVector3D newValue;
+            if (oldValue.type() == QVariant::Vector3D)
+                newValue = oldValue.value<QVector3D>();
+            if (name.endsWith(".x")) {
+                newValue.setX(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".y")) {
+                newValue.setY(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".z")) {
+                newValue.setZ(value.toFloat());
+                update = true;
+            }
+            newValueVar = newValue;
+        } else if (oldValue.type() == QVariant::Vector4D) {
+            QVector4D newValue;
+            if (oldValue.type() == QVariant::Vector4D)
+                newValue = oldValue.value<QVector4D>();
+            if (name.endsWith(".x")) {
+                newValue.setX(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".y")) {
+                newValue.setY(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".z")) {
+                newValue.setZ(value.toFloat());
+                update = true;
+            } else if (name.endsWith(".w")) {
+                newValue.setW(value.toFloat());
+                update = true;
+            }
+            newValueVar = newValue;
+        }
+        if (update) {
+            d->propertyValues.insert(parentPropName, newValueVar);
+            return;
+        }
+    }
+
     d->propertyValues.insert(name, value);
 }
 
@@ -397,6 +492,7 @@ QPixmap NodeInstance::blurredRenderPixmap() const
 void NodeInstance::setRenderPixmap(const QImage &image)
 {
     d->renderPixmap = QPixmap::fromImage(image);
+
     d->blurredRenderPixmap = QPixmap();
 }
 
@@ -434,6 +530,16 @@ InformationName NodeInstance::setInformationBoundingRect(const QRectF &rectangle
     return NoInformationChange;
 }
 
+InformationName NodeInstance::setInformationBoundingRectPixmap(const QRectF &rectangle)
+{
+    if (d->boundingRectPixmap != rectangle) {
+        d->boundingRectPixmap = rectangle;
+        return BoundingRectPixmap;
+    }
+
+    return NoInformationChange;
+}
+
 InformationName NodeInstance::setInformationContentItemBoundingRect(const QRectF &rectangle)
 {
     if (d->contentItemBoundingRect != rectangle) {
@@ -606,11 +712,24 @@ InformationName NodeInstance::setInformationHasBindingForProperty(const Property
     return NoInformationChange;
 }
 
+InformationName NodeInstance::setAllStates(const QStringList &states)
+{
+    if (d->allStates != states) {
+        d->allStates = states;
+        return AllStates;
+    }
+
+    return NoInformationChange;
+}
+
 InformationName NodeInstance::setInformation(InformationName name, const QVariant &information, const QVariant &secondInformation, const QVariant &thirdInformation)
 {
     switch (name) {
     case Size: return setInformationSize(information.toSizeF());
-    case BoundingRect: return setInformationBoundingRect(information.toRectF());
+    case BoundingRect:
+        return setInformationBoundingRect(information.toRectF());
+    case BoundingRectPixmap:
+        return setInformationBoundingRectPixmap(information.toRectF());
     case ContentItemBoundingRect: return setInformationContentItemBoundingRect(information.toRectF());
     case Transform: return setInformationTransform(information.value<QTransform>());
     case ContentTransform: return setInformationContentTransform(information.value<QTransform>());
@@ -628,6 +747,7 @@ InformationName NodeInstance::setInformation(InformationName name, const QVarian
     case Anchor: return setInformationAnchor(information.toByteArray(), secondInformation.toByteArray(), thirdInformation.value<qint32>());
     case InstanceTypeForProperty: return setInformationInstanceTypeForProperty(information.toByteArray(), secondInformation.toByteArray());
     case HasBindingForProperty: return setInformationHasBindingForProperty(information.toByteArray(), secondInformation.toBool());
+    case AllStates: return setAllStates(information.toStringList());
     case NoName:
     default: break;
     }
