@@ -1,36 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include "graphicsscene.h"
 #include "actionhandler.h"
 #include "actionprovider.h"
 #include "finalstateitem.h"
+#include "graphicsscene.h"
 #include "initialstateitem.h"
 #include "parallelitem.h"
 #include "sceneutils.h"
 #include "scxmleditorconstants.h"
+#include "scxmleditortr.h"
 #include "scxmltagutils.h"
 #include "scxmluifactory.h"
 #include "snapline.h"
@@ -41,13 +20,13 @@
 #include "warningitem.h"
 
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QAction>
 #include <QGuiApplication>
 #include <QClipboard>
 #include <QMenu>
 #include <QMimeData>
-#include <QThread>
 #include <QUndoStack>
 
 using namespace ScxmlEditor::PluginInterface;
@@ -67,28 +46,28 @@ GraphicsScene::~GraphicsScene()
 void GraphicsScene::unselectAll()
 {
     const QList<QGraphicsItem*> selectedItems = this->selectedItems();
-    foreach (QGraphicsItem *it, selectedItems)
+    for (QGraphicsItem *it : selectedItems)
         it->setSelected(false);
     if (m_document)
-        m_document->setCurrentTag(0);
+        m_document->setCurrentTag(nullptr);
 }
 
 void GraphicsScene::unhighlightAll()
 {
-    foreach (BaseItem *it, m_baseItems)
+    for (BaseItem *it : std::as_const(m_baseItems))
         it->setHighlight(false);
 }
 
 void GraphicsScene::highlightItems(const QVector<ScxmlTag*> &lstIds)
 {
-    foreach (BaseItem *it, m_baseItems)
+    for (BaseItem *it : std::as_const(m_baseItems))
         it->setHighlight(lstIds.contains(it->tag()));
 }
 
 QRectF GraphicsScene::selectedBoundingRect() const
 {
     QRectF r;
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected())
             r = r.united(item->sceneBoundingRect());
     }
@@ -98,7 +77,7 @@ QRectF GraphicsScene::selectedBoundingRect() const
 qreal GraphicsScene::selectedMaxWidth() const
 {
     qreal maxw = 0;
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected() && item->type() >= InitialStateType)
             maxw = qMax(maxw, item->sceneBoundingRect().width());
     }
@@ -108,7 +87,7 @@ qreal GraphicsScene::selectedMaxWidth() const
 qreal GraphicsScene::selectedMaxHeight() const
 {
     qreal maxh = 0;
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected() && item->type() >= InitialStateType)
             maxh = qMax(maxh, item->sceneBoundingRect().height());
     }
@@ -118,43 +97,43 @@ qreal GraphicsScene::selectedMaxHeight() const
 void GraphicsScene::alignStates(int alignType)
 {
     if (alignType >= ActionAlignLeft && alignType <= ActionAlignVertical) {
-        m_document->undoStack()->beginMacro(tr("Align states"));
+        m_document->undoStack()->beginMacro(Tr::tr("Align states"));
         QRectF r = selectedBoundingRect();
 
         if (r.isValid()) {
             switch (alignType) {
             case ActionAlignLeft:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(r.left() - item->sceneBoundingRect().left(), 0);
                 }
                 break;
             case ActionAlignRight:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(r.right() - item->sceneBoundingRect().right(), 0);
                 }
                 break;
             case ActionAlignTop:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(0, r.top() - item->sceneBoundingRect().top());
                 }
                 break;
             case ActionAlignBottom:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(0, r.bottom() - item->sceneBoundingRect().bottom());
                 }
                 break;
             case ActionAlignHorizontal:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(0, r.center().y() - item->sceneBoundingRect().center().y());
                 }
                 break;
             case ActionAlignVertical:
-                foreach (BaseItem *item, m_baseItems) {
+                for (BaseItem *item : std::as_const(m_baseItems)) {
                     if (item->isSelected() && item->type() >= InitialStateType)
                         item->moveStateBy(r.center().x() - item->sceneBoundingRect().center().x(), 0);
                 }
@@ -171,11 +150,11 @@ void GraphicsScene::alignStates(int alignType)
 void GraphicsScene::adjustStates(int adjustType)
 {
     if (adjustType >= ActionAdjustWidth && adjustType <= ActionAdjustSize) {
-        m_document->undoStack()->beginMacro(tr("Adjust states"));
+        m_document->undoStack()->beginMacro(Tr::tr("Adjust states"));
         qreal maxw = selectedMaxWidth();
         qreal maxh = selectedMaxHeight();
 
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (item->isSelected() && item->type() >= InitialStateType) {
                 QRectF rr = item->boundingRect();
                 if ((adjustType == ActionAdjustWidth || adjustType == ActionAdjustSize) && !qFuzzyCompare(rr.width(), maxw))
@@ -192,7 +171,7 @@ void GraphicsScene::adjustStates(int adjustType)
 }
 void GraphicsScene::cut()
 {
-    m_document->undoStack()->beginMacro(tr("Cut"));
+    m_document->undoStack()->beginMacro(Tr::tr("Cut"));
     copy();
     removeSelectedItems();
     m_document->undoStack()->endMacro();
@@ -201,15 +180,15 @@ void GraphicsScene::cut()
 void GraphicsScene::removeSelectedItems()
 {
     QVector<ScxmlTag*> tags = SceneUtils::findRemovedTags(m_baseItems);
-    if (tags.count() > 0) {
-        m_document->undoStack()->beginMacro(tr("Remove item(s)"));
+    if (!tags.isEmpty()) {
+        m_document->undoStack()->beginMacro(Tr::tr("Remove items"));
 
         // Then remove found tags
         for (int i = tags.count(); i--;) {
             m_document->setCurrentTag(tags[i]);
             m_document->removeTag(tags[i]);
         }
-        m_document->setCurrentTag(0);
+        m_document->setCurrentTag(nullptr);
         m_document->undoStack()->endMacro();
     }
 }
@@ -223,7 +202,7 @@ void GraphicsScene::copy()
     QVector<ScxmlTag*> tags;
     if (m_document->currentTag()->tagType() == Scxml) {
         QVector<BaseItem*> items;
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (!item->parentItem())
                 items << item;
         }
@@ -235,15 +214,15 @@ void GraphicsScene::copy()
     if (tags.isEmpty() && m_document->currentTag())
         tags << m_document->currentTag();
 
-    if (tags.count() > 0) {
+    if (!tags.isEmpty()) {
         auto mime = new QMimeData;
         QByteArray result = m_document->content(tags);
         mime->setText(QLatin1String(result));
         mime->setData("StateChartEditor/StateData", result);
         QStringList strTypes;
-        foreach (const ScxmlTag *tag, tags) {
+        for (const ScxmlTag *tag : std::as_const(tags))
             strTypes << tag->tagName(false);
-        }
+
         mime->setData("StateChartEditor/CopiedTagTypes", strTypes.join(",").toLocal8Bit());
         mime->setData("StateChartEditor/CopiedMinPos", QString::fromLatin1("%1:%2").arg(minPos.x()).arg(minPos.y()).toLocal8Bit());
         QGuiApplication::clipboard()->setMimeData(mime);
@@ -267,7 +246,7 @@ void GraphicsScene::paste(const QPointF &targetPos)
     QPointF startPos(targetPos);
 
     BaseItem *targetItem = nullptr;
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected() && item->type() >= StateType) {
             targetItem = item;
             break;
@@ -286,7 +265,7 @@ void GraphicsScene::paste(const QPointF &targetPos)
     QString strMinPos = QLatin1String(mimeData->data("StateChartEditor/CopiedMinPos"));
     QPointF minPos(0, 0);
     if (!strMinPos.isEmpty()) {
-        QStringList coords = strMinPos.split(":", QString::SkipEmptyParts);
+        QStringList coords = strMinPos.split(":", Qt::SkipEmptyParts);
         if (coords.count() == 2)
             minPos = QPointF(coords[0].toDouble(), coords[1].toDouble());
     }
@@ -296,7 +275,7 @@ void GraphicsScene::paste(const QPointF &targetPos)
 
 void GraphicsScene::setEditorInfo(const QString &key, const QString &value)
 {
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected() && item->type() >= TransitionType)
             item->setEditorInfo(key, value);
     }
@@ -305,7 +284,7 @@ void GraphicsScene::setEditorInfo(const QString &key, const QString &value)
 void GraphicsScene::setDocument(ScxmlDocument *document)
 {
     if (m_document)
-        disconnect(m_document, 0, this, 0);
+        disconnect(m_document, nullptr, this, nullptr);
 
     m_document = document;
 
@@ -361,17 +340,17 @@ void GraphicsScene::init()
     }
 
     m_initializing = false;
-    warningVisibilityChanged(0, 0);
+    warningVisibilityChanged(0, nullptr);
     emit selectedStateCountChanged(0);
     emit selectedBaseItemCountChanged(0);
 }
 
 void GraphicsScene::runLayoutToSelectedStates()
 {
-    m_document->undoStack()->beginMacro(tr("Relayout"));
+    m_document->undoStack()->beginMacro(Tr::tr("Re-layout"));
 
     QVector<BaseItem*> selectedItems;
-    foreach (BaseItem *node, m_baseItems) {
+    for (BaseItem *node : std::as_const(m_baseItems)) {
         if (node->isSelected()) {
             int index = 0;
             for (int i = 0; i < selectedItems.count(); ++i) {
@@ -391,22 +370,20 @@ void GraphicsScene::runLayoutToSelectedStates()
     // Layout scene items if necessary
     if (selectedItems.isEmpty()) {
         QList<QGraphicsItem*> sceneItems;
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (item->type() >= InitialStateType && !item->parentItem())
                 sceneItems << item;
         }
         SceneUtils::layout(sceneItems);
 
-        foreach (QGraphicsItem *item, sceneItems) {
+        for (QGraphicsItem *item : std::as_const(sceneItems))
             if (item->type() >= StateType)
                 static_cast<StateItem*>(item)->shrink();
-        }
     }
 
     // Update properties
-    foreach (BaseItem *node, selectedItems) {
+    for (BaseItem *node : std::as_const(selectedItems))
         node->updateUIProperties();
-    }
 
     m_document->undoStack()->endMacro();
 }
@@ -417,31 +394,31 @@ void GraphicsScene::runAutomaticLayout()
 
     // 1. Find max depth
     int maxDepth = 0;
-    foreach (BaseItem *node, m_baseItems) {
+    for (BaseItem *node : std::as_const(m_baseItems)) {
         maxDepth = qMax(maxDepth, node->depth());
         node->setBlockUpdates(true);
     }
 
     // 2. Layout every depth-level separately
     for (int d = (maxDepth + 1); d--;) {
-        foreach (BaseItem *node, m_baseItems)
+        for (BaseItem *node : std::as_const(m_baseItems))
             node->doLayout(d);
     }
 
     // 3. Layout scene items
     QList<QGraphicsItem*> sceneItems;
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->type() >= InitialStateType && !item->parentItem())
             sceneItems << item;
     }
     SceneUtils::layout(sceneItems);
 
-    foreach (QGraphicsItem *item, sceneItems) {
+    for (QGraphicsItem *item : std::as_const(sceneItems)) {
         if (item->type() >= StateType)
             static_cast<StateItem*>(item)->shrink();
     }
 
-    foreach (BaseItem *node, m_baseItems) {
+    for (BaseItem *node : std::as_const(m_baseItems)) {
         node->updateUIProperties();
         node->setBlockUpdates(false);
     }
@@ -465,25 +442,24 @@ void GraphicsScene::beginTagChange(ScxmlDocument::TagChange change, ScxmlTag *ta
 void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag, const QVariant &value)
 {
     Q_UNUSED(value)
-    QTC_ASSERT(tag, return);
 
     switch (change) {
     case ScxmlDocument::TagAttributesChanged: {
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (item->tag() == tag)
                 item->updateAttributes();
         }
         break;
     }
     case ScxmlDocument::TagEditorInfoChanged: {
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (item->tag() == tag)
                 item->updateEditorInfo();
         }
         break;
     }
     case ScxmlDocument::TagCurrentChanged: {
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (!item->isSelected() && item->tag() == tag)
                 item->setSelected(true);
         }
@@ -491,9 +467,9 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
         break;
     }
     case ScxmlDocument::TagChangeParent: {
-        auto childItem = qobject_cast<ConnectableItem*>(findItem(tag));
-
+        auto childItem = findItem(tag);
         if (childItem) {
+            QTC_ASSERT(tag, break);
             BaseItem *newParentItem = findItem(tag->parentTag());
             BaseItem *oldParentItem = childItem->parentBaseItem();
 
@@ -508,8 +484,11 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
 
             childItem->setParentItem(newParentItem);
             childItem->updateUIProperties();
-            childItem->updateTransitions(true);
-            childItem->updateTransitionAttributes(true);
+            if (auto childConItem = qobject_cast<ConnectableItem*>(findItem(tag))) {
+                childConItem->updateTransitions(true);
+                childConItem->updateTransitionAttributes(true);
+            }
+
             childItem->checkWarnings();
             childItem->checkInitial();
             if (newParentItem) {
@@ -518,6 +497,8 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
                 newParentItem->checkWarnings();
                 newParentItem->checkOverlapping();
                 newParentItem->updateUIProperties();
+                if (auto newConItem = qobject_cast<StateItem*>(newParentItem))
+                    newConItem->updateBoundingRect();
             }
 
             if (oldParentItem)
@@ -530,6 +511,9 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
     }
     case ScxmlDocument::TagAddTags: {
         // Finalize transitions
+        if (!tag)
+            break;
+
         QVector<ScxmlTag*> childTransitionTags;
         if (tag->tagName(false) == "transition")
             childTransitionTags << tag;
@@ -541,9 +525,9 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
                 item->finalizeCreation();
         }
     }
-    // FIXME: intended fallthrough?
+    break;
     case ScxmlDocument::TagAddChild: {
-        ScxmlTag *childTag = tag->child(value.toInt());
+        ScxmlTag *childTag = tag ? tag->child(value.toInt()) : nullptr;
         if (childTag) {
             // Check that there is no any item with this tag
             BaseItem *childItem = findItem(childTag);
@@ -553,7 +537,7 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
                     auto transition = new TransitionItem;
                     addItem(transition);
                     transition->setStartItem(qgraphicsitem_cast<ConnectableItem*>(parentItem));
-                    transition->init(childTag, 0, false, false);
+                    transition->init(childTag, nullptr, false, false);
                     transition->updateAttributes();
                 } else {
                     childItem = SceneUtils::createItemByTagType(childTag->tagType(), QPointF());
@@ -569,6 +553,9 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
             }
 
             if (parentItem) {
+                if (childItem == nullptr)
+                    parentItem->addChild(childTag);
+
                 parentItem->updateAttributes();
                 parentItem->updateUIProperties();
                 parentItem->checkInitial();
@@ -578,21 +565,26 @@ void GraphicsScene::endTagChange(ScxmlDocument::TagChange change, ScxmlTag *tag,
         break;
     }
     case ScxmlDocument::TagRemoveChild: {
-        BaseItem *parentItem = findItem(tag);
-        if (parentItem) {
-            parentItem->updateAttributes();
-            parentItem->checkInitial();
-        } else {
-            checkInitialState();
+        if (tag) {
+            BaseItem *parentItem = findItem(tag);
+            if (parentItem) {
+                parentItem->updateAttributes();
+                parentItem->checkInitial();
+            } else {
+                checkInitialState();
+            }
         }
         break;
     }
     case ScxmlDocument::TagChangeOrder: {
-        BaseItem *parentItem = findItem(tag->parentTag());
-        if (parentItem)
-            parentItem->updateAttributes();
-        else
-            checkInitialState();
+        if (tag) {
+            BaseItem *parentItem = findItem(tag->parentTag());
+            if (parentItem)
+                parentItem->updateAttributes();
+            else
+                checkInitialState();
+        }
+        break;
     }
     default:
         break;
@@ -641,7 +633,7 @@ ScxmlUiFactory *GraphicsScene::uiFactory() const
 
 void GraphicsScene::addConnectableItem(ItemType type, const QPointF &pos, BaseItem *parentItem)
 {
-    m_document->undoStack()->beginMacro(tr("Add new state"));
+    m_document->undoStack()->beginMacro(Tr::tr("Add new state"));
     ConnectableItem *newItem = SceneUtils::createItem(type, pos);
 
     if (newItem) {
@@ -671,7 +663,7 @@ void GraphicsScene::keyPressEvent(QKeyEvent *event)
 {
     QGraphicsItem *focusItem = this->focusItem();
     if (!focusItem || focusItem->type() != TextType) {
-        if (event->key() == Qt::Key_Delete)
+        if (event->key() == Qt::Key_Delete || event->key() == Qt::Key_Backspace)
             removeSelectedItems();
     }
     QGraphicsScene::keyPressEvent(event);
@@ -717,7 +709,7 @@ BaseItem *GraphicsScene::findItem(const ScxmlTag *tag) const
     if (!tag)
         return nullptr;
 
-    foreach (BaseItem *it, m_baseItems) {
+    for (BaseItem *it : std::as_const(m_baseItems)) {
         if (it->tag() == tag)
             return it;
     }
@@ -730,14 +722,14 @@ void GraphicsScene::removeItems(const ScxmlTag *tag)
     if (tag) {
         // Find right items
         QVector<BaseItem*> items;
-        foreach (BaseItem *it, m_baseItems) {
+        for (BaseItem *it : std::as_const(m_baseItems)) {
             if (it->tag() == tag)
                 items << it;
         }
 
         // Then delete them
         for (int i = items.count(); i--;) {
-            items[i]->setTag(0);
+            items[i]->setTag(nullptr);
             delete items[i];
         }
     }
@@ -756,7 +748,7 @@ QPair<bool, bool> GraphicsScene::checkSnapToItem(BaseItem *item, const QPointF &
     qreal diffY = 8;
     qreal diffYdX = 2000;
 
-    foreach (BaseItem *it, m_baseItems) {
+    for (BaseItem *it : std::as_const(m_baseItems)) {
         if (!it->isSelected() && it != item && it->parentItem() == parentItem && it->type() >= InitialStateType) {
             QPointF c = it->sceneCenter();
             qreal dX = qAbs(c.x() - p.x());
@@ -794,7 +786,7 @@ void GraphicsScene::selectionChanged(bool para)
     int baseCount = 0;
     int stateTypeCount = 0;
 
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         if (item->isSelected()) {
             if (item->type() >= TransitionType)
                 baseCount++;
@@ -823,7 +815,8 @@ void GraphicsScene::addWarningItem(WarningItem *item)
     if (!m_allWarnings.contains(item)) {
         m_allWarnings << item;
         if (!m_autoLayoutRunning && !m_initializing)
-            QMetaObject::invokeMethod(this, "warningVisibilityChanged", Qt::QueuedConnection, Q_ARG(int, 0));
+            QMetaObject::invokeMethod(this, [this] { warningVisibilityChanged(0); },
+                                      Qt::QueuedConnection);
     }
 }
 
@@ -832,28 +825,27 @@ void GraphicsScene::removeWarningItem(WarningItem *item)
     m_allWarnings.removeAll(item);
 
     if (!m_autoLayoutRunning && !m_initializing)
-        QMetaObject::invokeMethod(this, "warningVisibilityChanged", Qt::QueuedConnection, Q_ARG(int, 0));
+        QMetaObject::invokeMethod(this, [this] { warningVisibilityChanged(0); },
+                                  Qt::QueuedConnection);
 }
 
 void GraphicsScene::warningVisibilityChanged(int type, WarningItem *item)
 {
     if (!m_autoLayoutRunning && !m_initializing) {
-        foreach (WarningItem *it, m_allWarnings) {
+        for (WarningItem *it : std::as_const(m_allWarnings))
             if (it != item && (type == 0 || it->type() == type))
                 it->check();
-        }
     }
 }
 
 ScxmlTag *GraphicsScene::tagByWarning(const ScxmlEditor::OutputPane::Warning *w) const
 {
     ScxmlTag *tag = nullptr;
-    foreach (WarningItem *it, m_allWarnings) {
+    for (WarningItem *it : std::as_const(m_allWarnings))
         if (it->warning() == w) {
             tag = it->tag();
             break;
         }
-    }
     return tag;
 }
 
@@ -862,7 +854,7 @@ void GraphicsScene::highlightWarningItem(const ScxmlEditor::OutputPane::Warning
     ScxmlTag *tag = tagByWarning(w);
 
     if (tag)
-        highlightItems(QVector<ScxmlTag*>() << tag);
+        highlightItems({tag});
     else
         unhighlightAll();
 }
@@ -894,7 +886,7 @@ void GraphicsScene::addChild(BaseItem *item)
     if (!m_baseItems.contains(item)) {
         connect(item, &BaseItem::selectedStateChanged, this, &GraphicsScene::selectionChanged);
         connect(item, &BaseItem::openToDifferentView, this, [=](BaseItem *item){
-            openStateView(item);
+            emit openStateView(item);
         }, Qt::QueuedConnection);
         m_baseItems << item;
     }
@@ -903,7 +895,7 @@ void GraphicsScene::addChild(BaseItem *item)
 void GraphicsScene::removeChild(BaseItem *item)
 {
     if (item)
-        disconnect(item, 0, this, 0);
+        disconnect(item, nullptr, this, nullptr);
     m_baseItems.removeAll(item);
 
     selectionChanged(false);
@@ -911,7 +903,7 @@ void GraphicsScene::removeChild(BaseItem *item)
 
 void GraphicsScene::checkItemsVisibility(double scaleFactor)
 {
-    foreach (BaseItem *item, m_baseItems) {
+    for (BaseItem *item : std::as_const(m_baseItems)) {
         item->checkVisibility(scaleFactor);
     }
 }
@@ -920,7 +912,7 @@ void GraphicsScene::checkInitialState()
 {
     if (m_document) {
         QList<QGraphicsItem*> sceneItems;
-        foreach (BaseItem *item, m_baseItems) {
+        for (BaseItem *item : std::as_const(m_baseItems)) {
             if (item->type() >= InitialStateType && !item->parentItem())
                 sceneItems << item;
         }
@@ -934,14 +926,14 @@ void GraphicsScene::checkInitialState()
 
 void GraphicsScene::clearAllTags()
 {
-    foreach (BaseItem *it, m_baseItems) {
-        it->setTag(0);
+    for (BaseItem *it : std::as_const(m_baseItems)) {
+        it->setTag(nullptr);
     }
 }
 
 void GraphicsScene::setBlockUpdates(bool block)
 {
-    foreach (BaseItem *it, m_baseItems) {
+    for (BaseItem *it : std::as_const(m_baseItems)) {
         it->setBlockUpdates(block);
     }
 }
