@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "vcsbasesubmiteditor.h"
 
@@ -32,33 +10,39 @@
 #include "submitfieldwidget.h"
 #include "submitfilemodel.h"
 #include "vcsbaseplugin.h"
+#include "vcsbasetr.h"
 #include "vcsoutputwindow.h"
 #include "vcsplugin.h"
 
 #include <aggregation/aggregate.h>
-#include <cpptools/cppmodelmanager.h>
+
+#include <coreplugin/find/basetextfind.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/editormanager.h>
+
+#include <extensionsystem/invoker.h>
+#include <extensionsystem/pluginmanager.h>
+
+#include <utils/algorithm.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/completingtextedit.h>
-#include <utils/synchronousprocess.h>
 #include <utils/fileutils.h>
 #include <utils/icon.h>
-#include <utils/theme/theme.h>
+#include <utils/process.h>
 #include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
-#include <coreplugin/find/basetextfind.h>
+#include <utils/theme/theme.h>
+
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorsettings.h>
 
 #include <projectexplorer/project.h>
-#include <projectexplorer/session.h>
+#include <projectexplorer/projectmanager.h>
 
 #include <QDir>
 #include <QFileInfo>
 #include <QPointer>
 #include <QProcess>
-#include <QPushButton>
 #include <QSet>
 #include <QStringListModel>
 #include <QStyle>
@@ -71,33 +55,11 @@
 #include <cstring>
 
 enum { debug = 0 };
-enum { wantToolBar = 0 };
 
 // Return true if word is meaningful and can be added to a completion model
-static bool acceptsWordForCompletion(const char *word)
+static bool acceptsWordForCompletion(const QString &word)
 {
-    if (!word)
-        return false;
-    static const std::size_t minWordLength = 7;
-    return std::strlen(word) >= minWordLength;
-}
-
-// Return the class name which function belongs to
-static const char *belongingClassName(const CPlusPlus::Function *function)
-{
-    if (!function)
-        return nullptr;
-
-    if (auto funcName = function->name()) {
-        if (auto qualifiedNameId =  funcName->asQualifiedNameId()) {
-            if (const CPlusPlus::Name *funcBaseName = qualifiedNameId->base()) {
-                if (auto identifier = funcBaseName->identifier())
-                    return identifier->chars();
-            }
-        }
-    }
-
-    return nullptr;
+    return word.size() >= 7;
 }
 
 /*!
@@ -141,37 +103,28 @@ namespace VcsBase {
 using namespace Internal;
 using namespace Utils;
 
-static inline QString submitMessageCheckScript()
-{
-    return VcsPlugin::instance()->settings().submitMessageCheckScript;
-}
-
 class VcsBaseSubmitEditorPrivate
 {
 public:
-    VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditorParameters *parameters,
-                               SubmitEditorWidget *editorWidget,
+    VcsBaseSubmitEditorPrivate(SubmitEditorWidget *editorWidget,
                                VcsBaseSubmitEditor *q);
 
     SubmitEditorWidget *m_widget;
-    QToolBar *m_toolWidget = nullptr;
-    const VcsBaseSubmitEditorParameters *m_parameters;
+    VcsBaseSubmitEditorParameters m_parameters;
     QString m_displayName;
-    QString m_checkScriptWorkingDirectory;
-    SubmitEditorFile *m_file;
+    FilePath m_checkScriptWorkingDirectory;
+    SubmitEditorFile m_file;
 
     QPointer<QAction> m_diffAction;
     QPointer<QAction> m_submitAction;
 
     NickNameDialog *m_nickNameDialog = nullptr;
+    bool m_disablePrompt = false;
 };
 
-VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditorParameters *parameters,
-                                                       SubmitEditorWidget *editorWidget,
+VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(SubmitEditorWidget *editorWidget,
                                                        VcsBaseSubmitEditor *q) :
-    m_widget(editorWidget),
-    m_parameters(parameters),
-    m_file(new SubmitEditorFile(parameters, q))
+    m_widget(editorWidget), m_file(q)
 {
     auto completer = new QCompleter(q);
     completer->setCaseSensitivity(Qt::CaseSensitive);
@@ -180,15 +133,23 @@ VcsBaseSubmitEditorPrivate::VcsBaseSubmitEditorPrivate(const VcsBaseSubmitEditor
     m_widget->descriptionEdit()->setCompletionLengthThreshold(4);
 }
 
-VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *parameters,
-                                         SubmitEditorWidget *editorWidget) :
-    d(new VcsBaseSubmitEditorPrivate(parameters, editorWidget, this))
+VcsBaseSubmitEditor::VcsBaseSubmitEditor(SubmitEditorWidget *editorWidget)
 {
+    setWidget(editorWidget);
+    d = new VcsBaseSubmitEditorPrivate(editorWidget, this);
+}
+
+void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &parameters)
+{
+    d->m_parameters = parameters;
+    d->m_file.setId(parameters.id);
+    d->m_file.setMimeType(QLatin1String(parameters.mimeType));
+
     setWidget(d->m_widget);
-    document()->setPreferredDisplayName(QCoreApplication::translate("VCS", d->m_parameters->displayName));
+    document()->setPreferredDisplayName(Tr::tr(d->m_parameters.displayName));
 
     // Message font according to settings
-    CompletingTextEdit *descriptionEdit = editorWidget->descriptionEdit();
+    CompletingTextEdit *descriptionEdit = d->m_widget->descriptionEdit();
     const TextEditor::FontSettings fs = TextEditor::TextEditorSettings::fontSettings();
     const QTextCharFormat tf = fs.toTextCharFormat(TextEditor::C_TEXT);
     descriptionEdit->setFont(tf.font());
@@ -196,13 +157,13 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     QPalette pal;
     pal.setColor(QPalette::Base, tf.background().color());
     pal.setColor(QPalette::Text, tf.foreground().color());
-    pal.setColor(QPalette::Foreground, tf.foreground().color());
+    pal.setColor(QPalette::WindowText, tf.foreground().color());
     if (selectionFormat.background().style() != Qt::NoBrush)
         pal.setColor(QPalette::Highlight, selectionFormat.background().color());
     pal.setBrush(QPalette::HighlightedText, selectionFormat.foreground());
     descriptionEdit->setPalette(pal);
 
-    d->m_file->setModified(false);
+    d->m_file.setModified(false);
     // We are always clean to prevent the editor manager from asking to save.
 
     connect(d->m_widget, &SubmitEditorWidget::diffSelected,
@@ -210,36 +171,37 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     connect(descriptionEdit, &QTextEdit::textChanged,
             this, &VcsBaseSubmitEditor::fileContentsChanged);
 
-    const CommonVcsSettings settings = VcsPlugin::instance()->settings();
+    const CommonVcsSettings &settings = commonSettings();
     // Add additional context menu settings
-    if (!settings.submitMessageCheckScript.isEmpty() || !settings.nickNameMailMap.isEmpty()) {
+    if (!settings.submitMessageCheckScript().isEmpty()
+            || !settings.nickNameMailMap.value().isEmpty()) {
         auto sep = new QAction(this);
         sep->setSeparator(true);
         d->m_widget->addDescriptionEditContextMenuAction(sep);
         // Run check action
-        if (!settings.submitMessageCheckScript.isEmpty()) {
-            auto checkAction = new QAction(tr("Check Message"), this);
+        if (!settings.submitMessageCheckScript().isEmpty()) {
+            auto checkAction = new QAction(Tr::tr("Check Message"), this);
             connect(checkAction, &QAction::triggered,
                     this, &VcsBaseSubmitEditor::slotCheckSubmitMessage);
             d->m_widget->addDescriptionEditContextMenuAction(checkAction);
         }
         // Insert nick
-        if (!settings.nickNameMailMap.isEmpty()) {
-            auto insertAction = new QAction(tr("Insert Name..."), this);
+        if (!settings.nickNameMailMap.value().isEmpty()) {
+            auto insertAction = new QAction(Tr::tr("Insert Name..."), this);
             connect(insertAction, &QAction::triggered, this, &VcsBaseSubmitEditor::slotInsertNickName);
             d->m_widget->addDescriptionEditContextMenuAction(insertAction);
         }
     }
     // Do we have user fields?
-    if (!settings.nickNameFieldListFile.isEmpty())
-        createUserFields(settings.nickNameFieldListFile);
+    if (!settings.nickNameFieldListFile.value().isEmpty())
+        createUserFields(settings.nickNameFieldListFile.value());
 
     // wrapping. etc
-    slotUpdateEditorSettings(settings);
-    connect(VcsPlugin::instance(), &VcsPlugin::settingsChanged,
+    slotUpdateEditorSettings();
+    connect(&settings, &CommonVcsSettings::changed,
             this, &VcsBaseSubmitEditor::slotUpdateEditorSettings);
     connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
-            this, [this]() {
+            this, [this] {
                 if (Core::EditorManager::currentEditor() == this)
                     updateFileModel();
             });
@@ -256,15 +218,14 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
 
 VcsBaseSubmitEditor::~VcsBaseSubmitEditor()
 {
-    delete d->m_toolWidget;
     delete d->m_widget;
     delete d;
 }
 
-void VcsBaseSubmitEditor::slotUpdateEditorSettings(const CommonVcsSettings &s)
+void VcsBaseSubmitEditor::slotUpdateEditorSettings()
 {
-    setLineWrapWidth(s.lineWrapWidth);
-    setLineWrap(s.lineWrap);
+    setLineWrapWidth(commonSettings().lineWrapWidth());
+    setLineWrap(commonSettings().lineWrap());
 }
 
 // Return a trimmed list of non-empty field texts
@@ -272,7 +233,7 @@ static inline QStringList fieldTexts(const QString &fileContents)
 {
     QStringList rc;
     const QStringList rawFields = fileContents.trimmed().split(QLatin1Char('\n'));
-    foreach (const QString &field, rawFields) {
+    for (const QString &field : rawFields) {
         const QString trimmedField = field.trimmed();
         if (!trimmedField.isEmpty())
             rc.push_back(trimmedField);
@@ -283,8 +244,11 @@ static inline QStringList fieldTexts(const QString &fileContents)
 void VcsBaseSubmitEditor::createUserFields(const QString &fieldConfigFile)
 {
     FileReader reader;
-    if (!reader.fetch(fieldConfigFile, QIODevice::Text, Core::ICore::mainWindow()))
+    if (!reader.fetch(FilePath::fromString(fieldConfigFile),
+                      QIODevice::Text,
+                      Core::ICore::dialogParent())) {
         return;
+    }
     // Parse into fields
     const QStringList fields = fieldTexts(QString::fromUtf8(reader.data()));
     if (fields.empty())
@@ -353,56 +317,34 @@ void VcsBaseSubmitEditor::setLineWrapWidth(int w)
 
 Core::IDocument *VcsBaseSubmitEditor::document() const
 {
-    return d->m_file;
+    return &d->m_file;
 }
 
-QString VcsBaseSubmitEditor::checkScriptWorkingDirectory() const
+FilePath VcsBaseSubmitEditor::checkScriptWorkingDirectory() const
 {
     return d->m_checkScriptWorkingDirectory;
 }
 
-void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const QString &s)
+void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const FilePath &s)
 {
     d->m_checkScriptWorkingDirectory = s;
 }
 
-static QToolBar *createToolBar(const QWidget *someWidget, QAction *submitAction, QAction *diffAction)
-{
-    // Create
-    auto toolBar = new QToolBar;
-    toolBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-    const int size = someWidget->style()->pixelMetric(QStyle::PM_SmallIconSize);
-    toolBar->setIconSize(QSize(size, size));
-    toolBar->addSeparator();
-
-    if (submitAction)
-        toolBar->addAction(submitAction);
-    if (diffAction)
-        toolBar->addAction(diffAction);
-    return toolBar;
-}
-
-QWidget *VcsBaseSubmitEditor::toolBar()
-{
-    if (!wantToolBar)
-        return nullptr;
-
-    if (d->m_toolWidget)
-        return d->m_toolWidget;
-
-    if (!d->m_diffAction && !d->m_submitAction)
-        return nullptr;
-
-    // Create
-    d->m_toolWidget = createToolBar(d->m_widget, d->m_submitAction, d->m_diffAction);
-    return d->m_toolWidget;
-}
-
 QStringList VcsBaseSubmitEditor::checkedFiles() const
 {
     return d->m_widget->checkedFiles();
 }
 
+static QSet<FilePath> filesFromModel(SubmitFileModel *model)
+{
+    QSet<FilePath> result;
+    result.reserve(model->rowCount());
+    for (int row = 0; row < model->rowCount(); ++row) {
+        result.insert(model->repositoryRoot().resolvePath(model->file(row)).absoluteFilePath());
+    }
+    return result;
+}
+
 void VcsBaseSubmitEditor::setFileModel(SubmitFileModel *model)
 {
     QTC_ASSERT(model, return);
@@ -417,49 +359,21 @@ void VcsBaseSubmitEditor::setFileModel(SubmitFileModel *model)
     if (!selected.isEmpty())
         d->m_widget->setSelectedRows(selected);
 
-    QSet<QString> uniqueSymbols;
-    const CPlusPlus::Snapshot cppSnapShot = CppTools::CppModelManager::instance()->snapshot();
-
-    // Iterate over the files and get interesting symbols
-    for (int row = 0; row < model->rowCount(); ++row) {
-        const QFileInfo fileInfo(model->repositoryRoot(), model->file(row));
-
-        // Add file name
-        uniqueSymbols.insert(fileInfo.fileName());
-
-        const QString filePath = fileInfo.absoluteFilePath();
-        // Add symbols from the C++ code model
-        const CPlusPlus::Document::Ptr doc = cppSnapShot.document(filePath);
-        if (!doc.isNull() && doc->control()) {
-            const CPlusPlus::Control *ctrl = doc->control();
-            CPlusPlus::Symbol **symPtr = ctrl->firstSymbol(); // Read-only
-            while (symPtr != ctrl->lastSymbol()) {
-                const CPlusPlus::Symbol *sym = *symPtr;
-
-                const CPlusPlus::Identifier *symId = sym->identifier();
-                // Add any class, function or namespace identifiers
-                if ((sym->isClass() || sym->isFunction() || sym->isNamespace())
-                        && (symId && acceptsWordForCompletion(symId->chars())))
-                {
-                    uniqueSymbols.insert(QString::fromUtf8(symId->chars()));
-                }
-
-                // Handle specific case : get "Foo" in "void Foo::function() {}"
-                if (sym->isFunction() && !sym->asFunction()->isDeclaration()) {
-                    const char *className = belongingClassName(sym->asFunction());
-                    if (acceptsWordForCompletion(className))
-                        uniqueSymbols.insert(QString::fromUtf8(className));
-                }
-
-                ++symPtr;
-            }
-        }
+    const QSet<FilePath> files = filesFromModel(model);
+    // add file names to completion
+    QSet<QString> completionItems = Utils::transform(files, &FilePath::fileName);
+    QObject *cppModelManager = ExtensionSystem::PluginManager::getObjectByName("CppModelManager");
+    if (cppModelManager) {
+        const auto symbols = ExtensionSystem::invoke<QSet<QString>>(cppModelManager,
+                                                                    "symbolsInFiles",
+                                                                    files);
+        completionItems += Utils::filtered(symbols, acceptsWordForCompletion);
     }
 
     // Populate completer with symbols
-    if (!uniqueSymbols.isEmpty()) {
+    if (!completionItems.isEmpty()) {
         QCompleter *completer = d->m_widget->descriptionEdit()->completer();
-        QStringList symbolsList = uniqueSymbols.toList();
+        QStringList symbolsList = Utils::toList(completionItems);
         symbolsList.sort();
         completer->setModel(new QStringListModel(symbolsList, completer));
     }
@@ -485,7 +399,7 @@ QStringList VcsBaseSubmitEditor::rowsToFiles(const QList<int> &rows) const
 
 void VcsBaseSubmitEditor::slotDiffSelectedVcsFiles(const QList<int> &rawList)
 {
-    if (d->m_parameters->diffType == VcsBaseSubmitEditorParameters::DiffRows)
+    if (d->m_parameters.diffType == VcsBaseSubmitEditorParameters::DiffRows)
         emit diffSelectedRows(rawList);
     else
         emit diffSelectedFiles(rowsToFiles(rawList));
@@ -524,89 +438,49 @@ void VcsBaseSubmitEditor::setDescriptionMandatory(bool v)
 
 enum { checkDialogMinimumWidth = 500 };
 
-static QString withUnusedMnemonic(QString string, const QList<QPushButton *> &otherButtons)
+void VcsBaseSubmitEditor::accept(VcsBasePluginPrivate *plugin)
 {
-    QSet<QChar> mnemonics;
-    for (QPushButton *button : otherButtons) {
-        const QString text = button->text();
-        const int ampersandPos = text.indexOf('&');
-        if (ampersandPos >= 0 && ampersandPos < text.size() - 1)
-            mnemonics.insert(text.at(ampersandPos + 1));
-    }
-    for (int i = 0, total = string.length(); i < total; ++i) {
-        if (!mnemonics.contains(string.at(i)))
-            return string.insert(i, '&');
-    }
-    return string;
-}
-
-VcsBaseSubmitEditor::PromptSubmitResult
-        VcsBaseSubmitEditor::promptSubmit(VcsBasePlugin *plugin,
-                                          bool *promptSetting,
-                                          bool forcePrompt,
-                                          bool canCommitOnFailure)
-{
-    bool dummySetting = false;
-    if (!promptSetting)
-        promptSetting = &dummySetting;
     auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
 
     Core::EditorManager::activateEditor(this, Core::EditorManager::IgnoreNavigationHistory);
 
-    if (!submitWidget->isEnabled())
-        return SubmitDiscarded;
-
     QString errorMessage;
-
-    const bool prompt = forcePrompt || *promptSetting;
-
-    // Pop up a message depending on whether the check succeeded and the
-    // user wants to be prompted
-    bool canCommit = checkSubmitMessage(&errorMessage) && submitWidget->canSubmit(&errorMessage);
-    if (canCommit && !prompt)
-        return SubmitConfirmed;
-    CheckableMessageBox mb(Core::ICore::dialogParent());
-    const QString commitName = plugin->commitDisplayName();
-    mb.setWindowTitle(tr("Close %1 %2 Editor")
-                      .arg(plugin->versionControl()->displayName(), commitName));
-    mb.setIconPixmap(QMessageBox::standardIcon(QMessageBox::Question));
-    QString message;
-    if (canCommit) {
-        message = tr("What do you want to do with these changes?");
-    } else {
-        message = tr("Cannot %1%2.\nWhat do you want to do?",
-                     "%2 is an optional error message with ': ' prefix. Don't add space in front.")
-                .arg(commitName.toLower(),
-                     errorMessage.isEmpty() ? errorMessage : ": " + errorMessage);
+    const bool canCommit = checkSubmitMessage(&errorMessage) && submitWidget->canSubmit(&errorMessage);
+    if (!canCommit) {
+        VcsOutputWindow::appendError(plugin->commitErrorMessage(errorMessage));
+    } else if (plugin->activateCommit()) {
+        close();
     }
-    mb.setText(message);
-    mb.setCheckBoxText(tr("Prompt to %1").arg(commitName.toLower()));
-    mb.setChecked(*promptSetting);
-    // Provide check box to turn off prompt ONLY if it was not forced
-    mb.setCheckBoxVisible(*promptSetting && !forcePrompt);
-    QDialogButtonBox::StandardButtons buttons = QDialogButtonBox::Close | QDialogButtonBox::Cancel;
-    if (canCommit || canCommitOnFailure)
-        buttons |= QDialogButtonBox::Ok;
-    mb.setStandardButtons(buttons);
-    QPushButton *cancelButton = mb.button(QDialogButtonBox::Cancel);
+}
+
+void VcsBaseSubmitEditor::close()
+{
+    d->m_disablePrompt = true;
+    Core::EditorManager::closeDocuments({document()});
+}
+
+bool VcsBaseSubmitEditor::promptSubmit(VcsBasePluginPrivate *plugin)
+{
+    if (d->m_disablePrompt)
+        return true;
+
+    Core::EditorManager::activateEditor(this, Core::EditorManager::IgnoreNavigationHistory);
+
+    auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
+    if (!submitWidget->isEnabled() || !submitWidget->isEdited())
+        return true;
+
+    QMessageBox mb(Core::ICore::dialogParent());
+    mb.setWindowTitle(plugin->commitAbortTitle());
+    mb.setIcon(QMessageBox::Warning);
+    mb.setText(plugin->commitAbortMessage());
+    mb.setStandardButtons(QMessageBox::Close | QMessageBox::Cancel);
     // On Windows there is no mnemonic for Close. Set it explicitly.
-    mb.button(QDialogButtonBox::Close)->setText(tr("&Close"));
-    cancelButton->setText(tr("&Keep Editing"));
-    // forcePrompt is true when the editor is closed, and false when triggered by the submit action
-    if (forcePrompt)
-        cancelButton->setDefault(true);
-    if (QPushButton *commitButton = mb.button(QDialogButtonBox::Ok)) {
-        commitButton->setText(withUnusedMnemonic(commitName,
-                              {cancelButton, mb.button(QDialogButtonBox::Close)}));
-    }
-    if (mb.exec() == QDialog::Accepted)
-        *promptSetting = mb.isChecked();
-    QAbstractButton *chosen = mb.clickedButton();
-    if (!chosen || chosen == cancelButton)
-        return SubmitCanceled;
-    if (chosen == mb.button(QDialogButtonBox::Close))
-        return SubmitDiscarded;
-    return SubmitConfirmed;
+    mb.button(QMessageBox::Close)->setText(Tr::tr("&Close"));
+    mb.button(QMessageBox::Cancel)->setText(Tr::tr("&Keep Editing"));
+    mb.setDefaultButton(QMessageBox::Cancel);
+    mb.exec();
+    return mb.result() == QMessageBox::Close;
 }
 
 QString VcsBaseSubmitEditor::promptForNickName()
@@ -638,7 +512,7 @@ void VcsBaseSubmitEditor::slotCheckSubmitMessage()
 {
     QString errorMessage;
     if (!checkSubmitMessage(&errorMessage)) {
-        QMessageBox msgBox(QMessageBox::Warning, tr("Submit Message Check Failed"),
+        QMessageBox msgBox(QMessageBox::Warning, Tr::tr("Submit Message Check Failed"),
                            errorMessage, QMessageBox::Ok, d->m_widget);
         msgBox.setMinimumWidth(checkDialogMinimumWidth);
         msgBox.exec();
@@ -647,7 +521,7 @@ void VcsBaseSubmitEditor::slotCheckSubmitMessage()
 
 bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
 {
-    const QString checkScript = submitMessageCheckScript();
+    const QString checkScript = commonSettings().submitMessageCheckScript.value();
     if (checkScript.isEmpty())
         return true;
     QApplication::setOverrideCursor(Qt::WaitCursor);
@@ -656,63 +530,43 @@ bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
     return rc;
 }
 
-static inline QString msgCheckScript(const QString &workingDir, const QString &cmd)
+static QString msgCheckScript(const FilePath &workingDir, const QString &cmd)
 {
     const QString nativeCmd = QDir::toNativeSeparators(cmd);
     return workingDir.isEmpty() ?
-           VcsBaseSubmitEditor::tr("Executing %1").arg(nativeCmd) :
-           VcsBaseSubmitEditor::tr("Executing [%1] %2").
-           arg(QDir::toNativeSeparators(workingDir), nativeCmd);
+           Tr::tr("Executing %1").arg(nativeCmd) :
+           Tr::tr("Executing [%1] %2").
+           arg(workingDir.toUserOutput(), nativeCmd);
 }
 
 bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript, QString *errorMessage) const
 {
     // Write out message
-    TempFileSaver saver(Utils::TemporaryDirectory::masterDirectoryPath() + "/msgXXXXXX.txt");
+    TempFileSaver saver(TemporaryDirectory::masterDirectoryPath() + "/msgXXXXXX.txt");
     saver.write(fileContents());
     if (!saver.finalize(errorMessage))
         return false;
     // Run check process
     VcsOutputWindow::appendShellCommandLine(msgCheckScript(d->m_checkScriptWorkingDirectory,
                                                            checkScript));
-    QProcess checkProcess;
+    Process checkProcess;
     if (!d->m_checkScriptWorkingDirectory.isEmpty())
         checkProcess.setWorkingDirectory(d->m_checkScriptWorkingDirectory);
-    checkProcess.start(checkScript, QStringList(saver.fileName()));
-    checkProcess.closeWriteChannel();
-    if (!checkProcess.waitForStarted()) {
-        *errorMessage = tr("The check script \"%1\" could not be started: %2").arg(checkScript, checkProcess.errorString());
-        return false;
-    }
-    QByteArray stdOutData;
-    QByteArray stdErrData;
-    if (!SynchronousProcess::readDataFromProcess(checkProcess, 30, &stdOutData, &stdErrData, false)) {
-        SynchronousProcess::stopProcess(checkProcess);
-        *errorMessage = tr("The check script \"%1\" timed out.").
-                        arg(QDir::toNativeSeparators(checkScript));
-        return false;
-    }
-    if (checkProcess.exitStatus() != QProcess::NormalExit) {
-        *errorMessage = tr("The check script \"%1\" crashed.").
-                        arg(QDir::toNativeSeparators(checkScript));
-        return false;
-    }
-    if (!stdOutData.isEmpty())
-        VcsOutputWindow::appendSilently(QString::fromLocal8Bit(stdOutData));
-    const QString stdErr = QString::fromLocal8Bit(stdErrData);
+    checkProcess.setCommand({FilePath::fromString(checkScript), {saver.filePath().toString()}});
+    checkProcess.start();
+    const bool succeeded = checkProcess.waitForFinished();
+
+    const QString stdOut = checkProcess.stdOut();
+    if (!stdOut.isEmpty())
+        VcsOutputWindow::appendSilently(stdOut);
+    const QString stdErr = checkProcess.stdErr();
     if (!stdErr.isEmpty())
         VcsOutputWindow::appendSilently(stdErr);
-    const int exitCode = checkProcess.exitCode();
-    if (exitCode != 0) {
-        const QString exMessage = tr("The check script returned exit code %1.").
-                                  arg(exitCode);
-        VcsOutputWindow::appendError(exMessage);
-        *errorMessage = stdErr;
-        if (errorMessage->isEmpty())
-            *errorMessage = exMessage;
-        return false;
-    }
-    return true;
+
+    if (!succeeded)
+        *errorMessage = checkProcess.exitMessage();
+
+    return succeeded;
 }
 
 QIcon VcsBaseSubmitEditor::diffIcon()
@@ -735,13 +589,12 @@ QIcon VcsBaseSubmitEditor::submitIcon()
 
 // Reduce a list of untracked files reported by a VCS down to the files
 // that are actually part of the current project(s).
-void VcsBaseSubmitEditor::filterUntrackedFilesOfProject(const QString &repositoryDirectory,
+void VcsBaseSubmitEditor::filterUntrackedFilesOfProject(const FilePath &repositoryDirectory,
                                                         QStringList *untrackedFiles)
 {
-    const QDir repoDir(repositoryDirectory);
     for (QStringList::iterator it = untrackedFiles->begin(); it != untrackedFiles->end(); ) {
-        const QString path = repoDir.absoluteFilePath(*it);
-        if (ProjectExplorer::SessionManager::projectForFile(FileName::fromString(path)))
+        const FilePath path = repositoryDirectory.resolvePath(*it).absoluteFilePath();
+        if (ProjectExplorer::ProjectManager::projectForFile(path))
             ++it;
         else
             it = untrackedFiles->erase(it);
